//init_proc.h

//SU_CONTROL application - Initialisation & Processing

//JIVE, May 1996

//_________________________________________________________________________

#ifndef INIT_PROC_H
#define INIT_PROC_H

#define DO_ACTUAL_RECORDING //when ready to test with net2disk
#include <math.h>
#include <iostream>
#include <fstream>
#include <su_control/su_controller.h>
#include <limits>
#include <su_control/su_defs.h>
#include <su_control/transp.h>
#include <su_control/su_cmd.h>
#include <su_control/peak_head.h>
#include <jccs_util/defs.h>
#include <controllers/thread.h>
#include <herbert/nicetimer.h>
#include <herbert/signal_flag.h>
#include <herbert/service_wrapper.h>
#include <controllers/state_machine.h>
#include <jive_msgs/ccc_action_stat.h>
#include <message_base/msg_handler.h>
#include <jive_msgs/ccc_msg_codes.h>
#include <message_base/comm_msg.h>
#include <jive_msgs/tables_data.h>
#include <jive_msgs/action_stat.h>
#include <job_descr/corr_job_descr.h>
#include <job_descr/t_job_descr.h>
#include <jccs_util/strobj.h>
#include <store_classes/order_list.h>
#include <herbert/t_base.h>
#include <time.h>
#include <errors/texts.h>
#include <errors/errors.h>
#include <jccs_util/comp_mediumid.h>

#include <prep_job/polynomials.h>
#include <prep_job/polys_db.h>

#include <data_handler/PhaseCalFile.h>

#include <fundamental/std_types.h>

#include <hw_sw_setup_db/hw_sw_setup_db.h>
#include <jccs_util/db_choice.h>

#include <string>
#include <vector>

//for socket:
#include <sys/types.h>  // For socket() and connect()
#include <sys/socket.h> // For socket() and connect()
#include <netinet/in.h> // For socket() with PF_INET
#include <netdb.h>      // For getservbyname() and gethostbyname()
#include <unistd.h>     // For close()

class SU_Connection;
class SU_Command_Channel;
class Transparent_channel;
class Peak_Head;

const int DEFAULT_EVLBI_NBLOCK = 8; // the number of blocks for the mark5 software to transport data from its input to output (e.g. net2out)

class MarkV_Socket
{
 public:

  MarkV_Socket();
  ~MarkV_Socket();

 private:

  int    sock;
  struct sockaddr_in socaddin;

 public:

  char   outLine[1024];

  bool connect_sock(char* ip_address);
  bool open_sock();
  bool send_sock(const char* command);
  bool receive_sock();
  void remove_sock();
};


class DU_Status
{
 private:
  char                _ip[16];
  char                _e_station_name[16];
  char                _e_station_control_ip[16];
  char                _e_station_data_ip[16];
  char                _ext_ip[16];
  int_32              _general_status;
  int_32              _e_station_general_status;
  int_64              _play_ptr;
  ieee_dbl            _start_rot;
  char                _play_status[16];
  char                _e_station_play_status[16];
  char                _data_mode[16];
  char                _data_submode[16];
  char                _data_time[24];
  int_64              _data_byte_offset;
  ieee_dbl	      _track_frame_period;
  char                _outp_data_mode[16];
  char                _outp_data_submode[16];
  int_32              _track_select;
  char                _VSN[MAX_NR_BANKS][MEDIUM_ID_LEN];
  int                 _active_bank; // -1 == no active bank, 0 == A, 1 == B
  int_32              _disk_unit_id;
  ieee_dbl            _data_rate;
  ieee_dbl            _clock_rate;
  ieee_dbl            _clock_gen_freq;
  int_32              _num_sync_attempts;
  char               _protocol[4];
  int_32              _sockbuf_size;
  int_32              _workbuf_size;
  int_32	      _mtu_size;
  int_32	      _connectivity;
  int_32              _ipd;
  uint64              _trackmask;

 public:
  DU_Status();
  ~DU_Status();

  static const int MTU_NOT_SET; //numeric_limits<int_32>::max();
  static const int CONNECTIVITY_NOT_SET; //numeric_limits<int_32>::max();

  char*    get_ip()                                  {return _ip; }
  char*    get_e_station_name()                  {return _e_station_name; }
  char*    get_e_station_control_ip()                    {return _e_station_control_ip; }
  char*    get_e_station_data_ip()                    {return _e_station_data_ip; }
  char*    get_ext_ip()                              {return _ext_ip; }
  int_32     get_general_status()                      {return _general_status;}
  int_32     get_e_station_general_status()        {return _e_station_general_status;}
  int_64     get_play_ptr()                            {return _play_ptr;}
  ieee_dbl   get_start_rot()                           {return _start_rot;}
  char*      get_play_status()                         {return _play_status;}
  char*    get_e_station_play_status()               {return _e_station_play_status;}
  char*      get_data_mode()                           {return _data_mode;}
  char*      get_data_submode()                        {return _data_submode;}
  char*      get_data_time()                           {return _data_time;}
  int_64     get_data_byte_offset()                    {return _data_byte_offset;}
  ieee_dbl   get_track_frame_period()                  {return _track_frame_period;}
  char*      get_outp_data_mode()                      {return _outp_data_mode;}
  char*      get_outp_data_submode()                   {return _outp_data_submode;}
  int_32     get_track_select()                        {return _track_select;}
  const char*    get_VSN(int bank) const               {return _VSN[bank];}
  int_32     get_disk_unit_id()                        {return _disk_unit_id;} 
  ieee_dbl   get_data_rate()                           {return _data_rate;}
  ieee_dbl   get_clock_rate()                          {return _clock_rate;}
  ieee_dbl   get_clock_gen_freq()                      {return _clock_gen_freq;}
  int_32     get_num_sync_attempts()                   {return _num_sync_attempts;}
  char*    get_protocol()                              {return _protocol;}
  int_32     get_sockbuf_size()                        {return _sockbuf_size;}
  int_32     get_workbuf_size()                        {return _workbuf_size;}
  int_32     get_mtu_size()                            {return _mtu_size;}
  int_32     get_connectivity()                        {return _connectivity;}
  int_32     get_ipd()                                 {return _ipd;}
  uint64     get_trackmask()                           {return _trackmask;}

  std::string get_VSN_active();

  int set_active_bank(int new_bank) { return _active_bank = new_bank; }

  char*    set_ip(const char* newval)                {return strcpy (_ip, newval);}
  char*    set_e_station_name(const char* newval)  {return strcpy (_e_station_name, newval);}
  char*    set_e_station_control_ip(const char* newval)  {return strcpy (_e_station_control_ip, newval);}
  char*    set_e_station_data_ip(const char* newval)  {return strcpy (_e_station_data_ip, newval);}
  char*    set_ext_ip(const char* newval)            {return strcpy (_ext_ip, newval);}
  int_32     set_general_status(int_32 newval)         {return _general_status = newval;}
  int_32     set_e_station_general_status(int_32 newval)  {return _e_station_general_status = newval;}
  int_64     set_play_ptr(int_64 newval)               {return _play_ptr = newval;}
  ieee_dbl   set_start_rot(ieee_dbl newval)            {return _start_rot = newval;}
  char*      set_play_status(const char* newval)       {return strcpy(_play_status, newval);}
  char*    set_e_station_play_status(const char* newval)       {return strcpy(_e_station_play_status, newval);}
  char*      set_data_mode(const char* newval)         {return strcpy(_data_mode,newval);}
  char*      set_data_submode(const char* newval)      {return strcpy(_data_submode,newval);}
  char*      set_data_time(const char* newval)         {return strcpy(_data_time,newval);}
  int_64     set_data_byte_offset(int_64 newval)       {return _data_byte_offset = newval;}
  ieee_dbl   set_track_frame_period(ieee_dbl newval)   {return _track_frame_period = newval;}
  char*      set_outp_data_mode(const char* newval)    {return strcpy(_outp_data_mode, newval);}
  char*      set_outp_data_submode(const char* newval) {return strcpy(_outp_data_submode, newval);}
  int_32     set_track_select(int_32 newval)           {return _track_select = newval;}
  void      set_VSN(const char newval[MAX_NR_BANKS][MEDIUM_ID_LEN], int new_active_bank);
 
  int_32     set_disk_unit_id(int_32 newval)           {return _disk_unit_id = newval;} 
  ieee_dbl   set_data_rate(ieee_dbl newval)            {return _data_rate = newval;} 
  ieee_dbl   set_clock_rate(ieee_dbl newval)           {return _clock_rate = newval;}
  ieee_dbl   set_clock_gen_freq(ieee_dbl newval)       {return _clock_gen_freq = newval;}
  int_32     set_num_sync_attempts(int_32 newval)      {return _num_sync_attempts = newval;}
  char*    set_protocol(const char* newval)            {return strcpy(_protocol, newval);}
  int_32     set_sockbuf_size(int_32 newval)           {return _sockbuf_size = newval;}
  int_32     set_workbuf_size(int_32 newval)           {return _workbuf_size = newval;}
  int_32     set_mtu_size(int_32 newval)               {return _mtu_size = newval;}
  int_32     set_connectivity(int_32 newval)           {return _connectivity = newval;}
  int_32     set_ipd(int_32 newval)                    {return _ipd = newval;}
  uint64     set_trackmask(uint64 newval)              {return _trackmask = newval;}
};

class Disk_Unit
{
 public:

  Disk_Unit();
  ~Disk_Unit();

  DU_Status du_status;  
  MarkV_Socket markv_socket;
  bool disk_available;
  bool merlin_slave;
  

  enum GENERAL_STATUS 
  {
    SYSTEM_READY,              //Bit 0    0x00000001
    ERROR,                     //Bit 1    0x00000002
    BIT2_UNUSED,               //Bit 2
    COMMAND_PENDING,           //Bit 3    0x00000008
    QUERY_PENDING,             //Bit 4    0x00000010
    BIT5_UNUSED,               //Bit 5
    RECORD_ON,                 //Bit 6    0x00000040
    RECORD_HALTED,             //Bit 7    0x00000080
    PLAYBACK_ON,               //Bit 8    0x00000100
    PLAYBACK_HALTED,           //Bit 9    0x00000200
    BIT10_UNUSED,              //Bit 10
    BIT11_UNUSED,              //Bit 11
    DISK2FILE_ON,              //Bit 12   0x00001000 
    FILE2DISK_ON,              //Bit 13   0x00002000 
    DISK2NET_ON,               //Bit 14   0x00004000
    NET2DISK_ON,               //Bit 15   0x00008000
    IN2NET_ON,                 //Bit 16   0x00010000 
    NET2OUT_ON,                //Bit 17   0x00020000
    BIT18_UNUSED,              //Bit 18
    BIT19_UNUSED,              //Bit 19
    BANK_A_SELECTED,           //Bit 20   0x00100000
    BANK_A_READY,              //Bit 21   0x00200000
    BANK_A_FULL,               //Bit 22   0x00400000
    BANK_A_WRITE_PROTECTED,    //Bit 23   0x00800000
    BANK_B_SELECTED,           //Bit 24   0x01000000
    BANK_B_READY,              //Bit 25   0x02000000
    BANK_B_FULL,               //Bit 26   0x04000000
    BANK_B_WRITE_PROTECTED     //Bit 27   0x08000000
  };


  bool check_status(GENERAL_STATUS status_bit);
  bool check_e_station_status(GENERAL_STATUS status_bit);
  void print_status_error();
  void print_command_response(const char* command, int return_code);
  bool handle_markv_command(const char* command, bool ext_station = false);
  bool start_play(int_64 playback_ptr = 0, int_64 obey_ROT = 0, int_32 speedup = 1);	
  int_64 calc_rot(char* time);
  bool set_play_mode(const char* outp_data_mode, const char* outp_data_submode);
  bool set_e_station_play_mode(const char* outp_data_mode, const char* outp_data_submode);
  bool set_play_rate(ieee_dbl data, ieee_dbl clock = 0.0, ieee_dbl clockgen = 0.0);
  bool set_e_station_play_rate(ieee_dbl clock);
  bool stop_play(int_64 playback_ptr, int_64 obey_ROT);
  bool stop_e_station_play(void);
  bool stop_play(void);
  bool set_protocol(char* protocol, int_32 recvbuf_size, int_32 workbuf_size, int nblock, int buffered_blocks); 
  bool set_e_station_protocol(char* protocol, int_32 recvbuf_size, int_32 workbuf_size);
  bool set_inter_packet_delay(int_32 us);
  bool net2out(char* action);
  bool net2disk(const char* action);
  bool in2net(char* action);
  bool set_disk_task_ID(int_32 task_ID);//used in conjunction with play for synchronisation with ROT
  bool reconnect_eVLBI(bool recording = false, const std::string& net2disk_command = "");
  bool bank_switch(void);
  bool bank_set(char bank);
  bool restart_transfer(void);
  bool query_net2out_on(void);
  bool query_net2disk_on(void);
  bool query_in2net_on(void);
  bool query_play_on(void);
  bool query_e_station_play_on(void);
  bool query_du_status(void);    
  bool query_e_station_status(void);    
  bool query_play_mode(void);    
  bool query_VSN(void);    
  bool query_play_rate(void);    
  bool query_du_ready(void); 
  bool query_du_ready_4evil(void); 
  bool check_data(void);   
  bool disk_servo(ieee_dbl servo_error, int_32 speedup = 1, bool eVLBI = false);
  bool query_pointer_position(void);

  bool set_remote_version(); // checks whether remote mark5 is an A or B, updates internal variable, returns whether query has succeeded

  enum Remote_Version {mark5a, mark5b, unknown};

  Remote_Version remote_version() { return _remote_version; }
  bool set_e_station_clock(int freq, std::string mode);
  bool set_e_station_dot();
  bool set_e_station_dim_mode(std::string data_source, unsigned int bit_stream_mask, int decimation);

 protected:
  Remote_Version _remote_version;
};




enum lc_trk_lnk {sine, mag, unknown, active, lsb, usb};
typedef std::string    channel_reference;
typedef int_32 head_pstion[4]; 

class Init_Proc : public Thread, public State_Machine, public SU_Controller
{
   friend class SU_Connection;
   friend class SU_Command_Channel;
   friend class Transparent_channel; 
   friend class Peak_Head;
   
   public : 
      Init_Proc(Thread_ID my_id, Thread_ID parent, Thread_Master *owner, unsigned int suid_code,
		bool disk, bool evilBI, SU_Command_Channel *comm1,  SU_Connection *comm3);
		
      ~Init_Proc(); //destructor
      
      //Thread interface
      int run() { return cycle();}
      int shutdown(); //thread
      int install() {current_state = wait_for_SU1; return 0;}
            
      //state machine interface
      int do_state();
      int do_transition() { return 0;}
      int find_state() { return 0;}
      int find_transition() { return 0;}
      int pre_state();
      int pre_transition() { return 0;}
      
      int quit();
      Disk_Unit disk_unit;
      
      static void get_du_parameters(Object_Store& mystore, Disk_Unit& disk_unit, bool eVLBI, int the_suid_code); // queries the database for the disk unit parameters, updates internal memebers

   private :
      Object_Store mystore;
      Polynomials_DB poly_lib;
      Polys *get_polys(void);//asks db for persistent polys
      HW_SW_Setup_DB hw_sw_setup_lib;
      
      bool queue_response(Incoming_Message_Info* info);
      SU_Status *get_su_status(void) const { return su_status; }
      SU_Configuration *get_su_configuration(void) const { return su_configuration; }

      virtual bool is_mounted() const { return get_su_status()->get_medium_mounted_status() != 0; }
      virtual const char* get_barcode(int bank) const { disk_unit.du_status.get_VSN(bank); }
      virtual void get_head_positions(int_32 head_pos[MAX_HEADS_PER_DPU]) const { get_su_status()->get_head_positions(head_pos); }
      virtual int_32 get_DPU_id() const { return get_su_configuration()->get_DPU_id(); }

      unsigned int get_suid_code(void)const {return the_suid_code;}
      unsigned int get_job_id(void)const {return id_of_job;}
      Msg_Handler_ID &get_SUC_ID(void) {return SU_CONTROL_ID;}
      bool initiated(void)const {return initialised;} //in Run State - idle or mounted
      bool get_allocation(void)const {return allocated;}
      
      //ofstream headerFile;//Temporary
      std::ofstream outFile;//Temporary
      std::ofstream logFile;
      std::ofstream peakFile;
      char *outFilename;
      char *peakFilename;
      //char *headerFilename;
      Errors errors;

      // XXX Should forward Phase Cal messages to data_handler.
      void write_data_message(Data_Message&);
      PhaseCalFile *pcalFile;

      //to enable timeouts
      Signal_Flag signal_flag;//for timer
      NiceTimer timer;
      int timer_key; //key for timer channel
      int key_stop_su_control;//for traqping ^C
      
      //to determine local time, etc for messages
      struct timeval tod;
      struct timezone tzp;
      Long_Time_Value time_stamp;
      Long_Time_Value local_time;
      void set_local_time(void);
      
      //sundry values

		struct scan_values
		{
		  ieee_dbl start_rot;
		  ieee_dbl stop_rot;
		  int_64 start_byte;
		  bool startdpu;
		  bool firstscan;
		};

		std::vector<scan_values> scan_values_list;

		bool swap_head;
      Msg_Handler_ID SU_CONTROL_ID;
      unsigned int the_suid_code; 
      bool use_mark5;
      bool eVLBI;
      ieee_dbl eVLBI_clock_delay;
      unsigned int id_of_job;//obtained from initialise_SU_Control cmd or at startup
      unsigned int abort_id;//of any abort cmd
      unsigned int load_polys_flag;
      int the_tape_format;//Mk III, IV or vlba
      ieee_dbl formatter_error; // Formatter error in integral seconds
      bool initialised;//true if in post wait_for_SU4 state
      bool allocated;//ie allocated to a job
      bool clk_msgs_disabled;//set true when ...
      bool suim_confgd;
      ieee_dbl begin_ROT, end_ROT, poly_obey_ROT;//ROT's from load polys boundaries
      ieee_dbl poly_offset; // the offset that should be applied to the delay poly when send to to the SU, used for on the fly clock searching, in seconds, the offset with which the first coefficient of the delay polynomials shall be altered

      ieee_dbl last_received_rot;

      unsigned int num_head_stacks;
      int head_0_fwd_offset, head_0_rev_offset, head_1_fwd_offset ,head_1_rev_offset;
      Thread *peaking_thread;
      int head1_best_track; // Track for peaking/autotracking etc
      int head2_best_track;
      Short_Time_Value last_tot; // Last TOT recored by average statistics

      // Global storage for external config override hacks
      bool external_dim_xbar;
      bool external_trm_active;
      bool external_suim_pg;
      bool external_crm_shuffle;
      //bool external_header_multiplier;
      int force_bits;
      //int header_multiplier;
      char dim_xbar_override[NUM_TRACKS]; // EVN42.3 Table III.10a
      char trm_active_override[NUM_TRACKS]; // EVN42.3 Table III.10b
      int crm_shuffle[NUM_CHANNELS]; // Shuffle CRM channels around
      int_16 suim_pg[17]; // SUIM pulsar gating hack

      // Values from feature bits
     //not needed int corr_head     // The actual head being correlated

      //BA has defined seven fundamental parameters :
      bool set_fundamental_parameters(Scan *scan);
	 
		float  fs;                // data sampling rate (Hz)
		float  muxmode;           // fan_in or 1:1 or fan_out    
		float  speedup;           // tape speed up e.g. 2.0 
		float  bpi;               // bits per inch          
		int    dpu_type;          //  0 = P+G, 1 = Metrum   
		int    dpu_accel;         // acceleration bpi/s     
		int    num_bits;          // number of data bits    
		int    nr_tracks_in_scan;
		unsigned int    bit_stream_mask;   // for the mode set in mark5b
	 
	 //derived parameter
	 
         int oversampling_factor;
	 int the_nominal_tape_speed;
	 ieee_dbl the_nominal_data_rate;
	 float  bandwidth;         // correlation bandwidth 
	 int itsRecirculationFactor;
	 
	 //tempory additions for the Load_Polys_Aux table
	 unsigned short int the_model_parameters;
	 unsigned short int the_sidebands;
	 Ordered_List* load_polys_aux;

      unsigned int nr_pcal_channels; // Number of channels with Phase Cal tones.

      //ids for the data handler and ROT clock connections
      unsigned int dat_handler_ID;//service conn id to data handler
      unsigned int ROT_clock_ID;//ditto to ROT clock

      // for sending sampler stats to the data status monitor (if present)
      Location_Client location_if;
      Service_Wrapper dsm_service;

      //various arrays for various tables
      unsigned int the_mask[2];//see Table III.10c
      unsigned int the_aux_data[NUM_CHANNELS][NUM_TRACKS][2];//see Table III.10c
      char dim_x_bar_configuration[NUM_TRACKS];//see Table III.10a
      int the_frames_per_barrel_roll_ph[NUM_CHANNELS];//Table III.10c
      char the_cross_bar_config[NUM_CHANNELS][NUM_TRACKS];//Table III.10d
      char the_active_tracks[NUM_TRACKS];//Table III.10b
      
      //intermediary arrays
      lc_trk_lnk log_chan_track_link[NUM_CHANNELS][73][2];//logical_channel/track link table
      unsigned int output_mapping[NUM_CHANNELS];//for table VII.14
      int mux_table[NUM_CHANNELS][9];
      channel_reference log_chan_ref_table[NUM_CHANNELS];//to keep track of Logical_Channel references wrt Logical_Channel #'s
      
      //array configuration methods
      void do_the_mask(const Record_Format *record_format);//also initialises the_tape_format
      bool do_config_arrays(Scan *scan, int &nr_tracks); //fills in the above arrays, and nr_tracks
      void do_aux_data(Scan *scan);
      
      bool do_q_1_mux_table(const int chan_nr,  Scan *scan, int &xbc_l_index, int &xbc_u_index,
			    int &nr_tracks);//2 or 4 channels per track
      bool do_mux_table(const int chan_nr,  Scan *scan, int &xbc_l_index, int &xbc_u_index, int &nr_tracks);
      //1, 2 or 4 tracks per channel
      
      bool do_the_cross_bar_config_table(void);
      bool construct_tables(Scan *scan, Table_Data *&table1, Table_Data *&table2, Table_Data *&table3,
			    Table_Data *&table4, Table_Data *&table5, Table_Data *&table6);

      
      //received from tape handler
      T_Job_Descriptor *current_jd;
      Corr_Job_Descr *cjd;//encapsulated in current_jd
      
      //scans, station and tape from cjd
      Ordered_List* scans;//in cjd
      Scan *current_scan;
      Logical_Station *current_station;
      Medium *current_medium;
      ieee_dbl sub_job_start_ROT;
      ieee_dbl sub_job_ending_ROT;
      
      std::string form_net2disk_command(ieee_dbl rot); // return "open:scan_ID", where if rot >= 0 : scan_ID is the name of the first scan  with end ROT > rot, else scan_ID is current_scan->get_scan_ID

      enum states {wait_for_SU1, wait_for_SU2, wait_for_SU3, wait_for_SU4, idle, idle_2, idle_3, mounted, // 0 .. 7 
                   wait_for_cjd, configure_dpu, configure_su, form_pcal_tables,  // 8 .. 11
                   pos_to_footage, wait_for_abort, peak_heads, heads_peaked, wait_for_footage, // 12 .. 16
                   pos_to_footage_2, positioning1, wait_for_ROT, wait_job_starting, get_polynomials, // 17 .. 21
                   configure_su2, running, open_disk_stats_file}; // 22 .. 24
      
      states current_state;
      
      unsigned int op_code;//from enquire_SU_Control
      
      SU_Command_Channel *su_cmd;//link to su_cmd_channel object
      SU_Connection *su_conn;//connection to SU
      
      //responses to cmds
      
      SU_Configuration *su_configuration;//to maintain configuration of SU
      SU_Status *su_status;//to maintain status of SU
      
      //persistent error message - configure to requirements
      SU_Control_Error error_msg;
        
      //queues for cmds and su responses
      Iter_List<Incoming_Message_Info *> response_queue;
      Forward_List_Iter<Incoming_Message_Info *> *response_iter;
	 
      void clear_queues(void);
      bool in_response_queue(const unsigned int action_status)const;
      bool data_in_response_queue(void)const; //returns true if a data message in the su Response queue
      
      //various transmit msgs functions

      void do_exit_to_mounted(unsigned int error_code, unsigned int action_code = 0, unsigned int ref_id = 0);
      void transmit_error(unsigned int error_code, unsigned int action_code = 0, unsigned int ref_id = 0);
      //to PROCESSOR CONTROL  and medium_handlers   
      bool request_su_status(void);
      int transmit_tape_status(unsigned int status = 0, unsigned int error_code = 0); 
      int transmit_disk_status(unsigned int status = 0, unsigned int error_code = 0); 
      // form Form SU_Control_tape_status message and transmit to SU_MONITOR
      bool transmit_to_SU(CCC_Msg *cccmsg);
      bool transmit_data_reqs(Scan *scan, ieee_dbl start);//sends data request cmds to SU for a given scan
      bool monitor_power(ieee_dbl start_ROT, ieee_dbl end_ROT);
      bool resume_clock_msgs(void);
      bool suspend_clock_msgs(void);
      bool send_load_poly_cmd(Command_Message *cmd, ieee_dbl rot, bool update_rot = true);
      bool send_alarm_request(unsigned int parameter, ieee_dbl obey_rot);
      bool send_req_tpe_frm_hdr(const int track_nr, int &counter, unsigned int &msg_id);
      bool send_set_ROT(ieee_dbl rot);
      bool flush_suim(void);
      bool configure_suim(void);
      bool configure_suim_2(void);
      bool pos_tape_to_footage(short reqd_tape_ftg, int the_tape_ftg_time_out);
      bool pos_tape_to_TOT(short reqd_tape_ftg, ieee_dbl reqd_TOT, int tape_TOT_time_out);
      bool config_dpu(const int head_nr);
      bool read_bar_code(void);//DPU_Command
      bool get_bar_code(void);
      bool rewind_unload(void);
      bool request_tape_mounted_status(void);//DPU_Command
      bool do_auto_tracking(ieee_dbl obey_ROT, bool enable);
      bool send_flush_su(void);
      bool config_su(General_CCC_Table *gen_tab1, General_CCC_Table *gen_tab2,
		     General_CCC_Table *gen_tab3, General_CCC_Table *gen_tab4,
		     General_CCC_Table *gen_tab5, General_CCC_Table *gen_tab6,
		     Command_Message * cmd);
      bool do_servoing(bool enable, ieee_dbl obey_rot);
      bool ccc_servoing(bool enable, ieee_dbl obey_rot);
      const char* rot2time(ieee_dbl rot, const int places=0);
      bool do_scan_start_stop(ieee_dbl start, ieee_dbl stop, ieee_dbl obey);
      
      //create command messages
      Command_Message *form_command(unsigned int command_code);
      
      //various fns to respond to specific messages, etc
      bool do_tape_dismounted(void);//checks if tape has been dismounted
      bool do_abort(void);
      bool check_timeout(float period); //checks if period has elapsed 
      void exit_process(unsigned int error_code, unsigned int command_code = 0);//exit process because of failed command with code...
      void SU_finished(int_32 why);//returns to mounted state, sending msg to medium_handlers
      bool handle_enquire(SU_Control_Enquiry_Aux *aux_table);
      
      //utility functions      
      unsigned int get_bcd_rep(int value)const;
      unsigned int rev_bcd(unsigned int arg)const;//reverses a bcd representation
      unsigned int vlba_group_track_rep(int track)const;//returns bit representation given a track #
      void make_tab(int tones[4], int pcmtab[4][3200])const;
      unsigned short int get_mux_demux(int bits);//for crm table
      int get_f(float TC, float CC)const;// See Module Software addresses page 15, Table 5.4.1

      //find correct medium, etc in cjd
      bool set_current_scan(void);
      bool run_children(bool head_0, bool head_1);
      bool get_dpu_footage(ieee_dbl repeat_interval = 0);
      bool dpu_ftg_changed(void);
      bool start_dpu(bool value, ieee_dbl obey_ROT = 0);
      bool set_dpuspeed(ieee_dbl obey_ROT = 0);
      bool start_tape(bool start = true, bool fwd = true, bool fast = false);
      void do_external_config();
      bool configure_su_next_scans(int &nr_tracks, ieee_dbl start_ROT);
      bool su_scan_config(bool firstscan, ieee_dbl start_ROT, ieee_dbl stop_ROT,
			  ieee_dbl prev_stop, bool nextDIM, bool nextTRM, 
			  bool nextCRM, bool nextDCM, bool nextAUX, 
			  bool nextbarrel, bool startdpu);
		bool su_scan_config_disk();
      bool check_alarm(void);
      bool do_tape_stats(int &lock_resyncs);
      bool do_disk_stats(int &lock_resyncs);
      bool set_disk_servo_error();

      bool recording; // whether the mark5 has received a net2out(false) or net2disk(true) command last (only for eVLBI)
      bool closeEVLBIout(bool query_first);
      void update_params(void);

      bool check_reset_peripherals();
      bool reset_peripherals_required;
      time_t next_SU_command_time;

      bool sending_dummy_data; // when a station is out of the array before the subjob end, data handlers expects it to send data with the proper frame headers, so we have to postpone resetting the SU until the subjob ends (to guard against missing a message, the SU will be reset when a new job enquire msg is received)
      void check_end_subjob_message(); // updates the state of sending_dummy_data and resets SU if possible
      void forward_data_msg_to_dsm(Data_Message* msg);
      
};

class Storeable_Aux_Data : public StoreObject
{
 private:
    
    Load_Polynomials_Aux* the_data;
    
 public:
    
    Storeable_Aux_Data(Load_Polynomials_Aux *data) { the_data = data; }
    ~Storeable_Aux_Data(void) {delete the_data;}
    
    Load_Polynomials_Aux &get_data(void) {return *the_data; }
    //superclass pure virtual functions
    char* hasName (void)const {return "Storeable_Aux_Data";}
    ClassID isType(void)const {return 0;}
};


#endif
