#ifndef PEAK_HEAD_H
#define PEAK_HEAD_H

#include <iostream>
#include <fstream>

#include <jive_message_sets/msg_defns.h>
#include <message_base/msg_handler.h>
#include <jive_msgs/ccc_msg_codes.h>
#include <message_base/comm_msg.h>

#include <su_control/init_proc.h>
#include <su_control/su_defs.h>
#include <controllers/thread.h>
#include <controllers/state_machine.h>

class Init_Proc;


class Peak_Head : public Thread, public State_Machine
{
   friend class Init_Proc;
   
   public : 
      Peak_Head(Thread_ID my_id, Thread_ID parent, Thread_Master *owner, Init_Proc *i_p, int nom_pos, 
		int nom_pos_off, unsigned int head_nr, unsigned int logical_head_nr, bool restart = true,
		bool peaking = true);
		
      ~Peak_Head(void); //destructor
      
      //Thread interface
      int run();
      int shutdown() { return 0;}
      int install() { return 0;}
      
      //state machine interface
      int do_state();
      int do_transition() { return 0;}
      int find_state() { return 0;}
      int find_transition() { return 0;}
      int pre_state();
      int pre_transition() { return 0;}
      
  private :
     
      enum states {peak_heads, request_headers, check_headers, wait_for_footage };
      states current_state;
      
      Init_Proc *_init_proc;
      Medium_Frame_Header_Data *header[3];//responses to request for headers
      
      int _nominal_position;
      int _nominal_position_offset;
      int _current_guess_position;
      short _actual_position;//after peaking attempt
      int _calculated_position;

      // Feature bit values
      int corr_head;
      
      unsigned int _head_nr;//head number (0 .. 3) which is to be peaked
      unsigned int _logical_head_nr; // 0 or 1
      unsigned int _track_nr;//track on which to peak (0 to 35)
      unsigned int _req_tpe_frm_hdr_id;
      unsigned int _peak_hd_cmd_id;
      unsigned int _req_tpe_frm_hdr_count;
      unsigned int _number_attempts;
      unsigned int _total_number_attempts;
      int initial_fishbone_direction;
      bool _back_to_initial_footage;//if true stops and rewinds after MAX_PEAK_HEAD_ATTEMPTS
      
      bool send_peak_heads_cmd(const int head_position, unsigned int the_peak_heads_time_out);
      bool send_req_tpe_frm_hdr(void);
      void do_head_not_peaked(void);
      bool get_dead_reckon_movement(void);
      bool is_header_good(void)const;
      bool go_to_start_footage(void)const;
      
      int quit();//terminates this thread
      
};

#endif
