//su_cmd.h

//SU_CONTROL application - SU command channel

//JIVE, May 1996

//_________________________________________________________________________

//for RCS

//$Header: /export/home/jive_cvs/JCCS/CVS_Repository/su_control/include/su_cmd.h,v 3.4 2009/03/20 07:24:08 jive_cc Exp $

//$Locker:  $

//$Log: su_cmd.h,v $
//Revision 3.4  2009/03/20 07:24:08  jive_cc
//BE: added/changed some communication between SU control and processing job
//
//Revision 3.3  2008/04/09 11:44:27  jive_cc
//BE: first version of mark5b control, also added the possibility to use the spare mk5b for production
//
//Revision 3.2  2005/12/23 13:16:30  verkout
//HV: Phew. This is the big thing. Merged all developments with both the port to Linux and the mods needed for a byte-order safe COF
//
//Revision 3.1  2003/05/15 11:46:27  jive_cc
//Arpad: merged MarkV version into main
//
//Revision 3.0.2.1  2003/05/14 10:03:58  jive_cc
//Arpad: changed tape_handler to medium_handler, many cosmetic changes
//
//Revision 3.0  2002/04/17 14:42:50  jive_cc
//Included gnu branch
//
//Revision 1.10  1999/02/08 13:05:57  jive_cc
//various mods at jive
//
//Revision 1.9  1999/01/29 16:08:10  pwm
//Integrated peaking branch into root
//
//Revision 1.8  1998/12/15 20:11:18  ps
//Message library reorganisation, part three
//
//Revision 1.7.2.1  1999/01/19 15:44:35  pwm
//added location_broker
//
//Revision 1.7  1998/08/26 20:01:47  jive_cc
//added quit()
//
//Revision 1.6  1998/07/01 11:55:40  pwm
//friends, interface
//
//Revision 1.5  1998/02/23 11:42:17  klijn
//Added service, etc to receive ROT Clock broadcasts
//
//Revision 1.4  1998/01/26 20:28:47  pwm
//added fns to determine tape_id. Cosmetic changes
//
//Revision 1.3  1997/11/07 15:36:12  pwm
//various more or less major changes
//
//Revision 1.2  97/06/09  18:39:03  18:39:03  pwm (Paul William Maguire)
//minor change
//
//Revision 1.1  97/04/03  15:52:15  15:52:15  pwm (Paul William Maguire)
//Initial revision
//

//___________________________________________________________________________

#ifndef SU_COMMAND_H
#define SU_COMMAND_H

#include <su_control/su_defs.h>
#include <su_control/su_controller.h>
#include <controllers/thread.h>
#include <standard_threads/service_mgr.h>
#include <herbert/service.h>
#include <jive_message_sets/msg_defns.h>
#include <message_base/msg_handler.h>
#include <jive_msgs/ccc_msg_codes.h>
#include <herbert/t_base.h>
#include <location_client/location_client.h>
#include <jive_locations/service_codes.h>
#include <jive_locations/jive_locations.h>
#include <time.h>


class SU_Controller;
class Init_Proc;
class Mark5B_Controller;
class SU_Connection;
class Transparent_channel;

class SU_Command_Channel : public Thread
{
   friend class SU_Controller;
   friend class Init_Proc;
   friend class Mark5B_Control;
   friend class SU_Connection;
   friend class Transparent_channel;
   
   public :
      SU_Command_Channel(Thread_ID my_id, Thread_ID parent, Thread_Master *owner, unsigned int suid_code);
      //constructor with thread parameters
      ~SU_Command_Channel(); //destructor
      
      //Thread interface
      int run();
      int shutdown(); //close connection
      int install() {return 0;}
      
      //initialise connection, etc
      bool initialise(SU_Controller *suc); //initiate connection
      int quit();
      
   private :
      
      //to determine local time, etc for messages
      struct timeval tod;
      struct timezone tzp;
      Long_Time_Value time_stamp;
      Long_Time_Value local_time;
      void set_local_time(void);
      
      bool enquiry_flag;//if an Enquiry cmd has already been queued to su_controller in this cycle
      
      Location_Client location_if;
      
      Service_Client *pc_client;//Client for conn to Processor_Control
      Service_Thread_Interface pc_client_interface;
      Service_Manager *pc_client_mgr;
      
      Service_Client *sum_client;//in effect conns to SU_MONITOR
      Service_Thread_Interface sum_client_interface;
      Service_Manager *sum_client_mgr;
      
      Service_Server* th_server;//in effect conns to MEDIUM_HANDLERS
      Service_Thread_Interface th_server_interface;
      Service_Manager *th_server_mgr;
      
      Service_Client *dh_client;//Client for conn to Data_handler
      Service_Thread_Interface dh_client_interface;
      Service_Manager *dh_client_mgr;
      
      Service_Client *clk_client;//Client for conn to ROT_Clock
      Service_Thread_Interface clk_client_interface;
      Service_Manager *clk_client_mgr;
      
      Service_Client *clkbrd_client;//Client for conn to ROT_Clock broadcast service
      Service_Thread_Interface clkbrd_client_interface;
      Service_Manager *clkbrd_client_mgr;
      
      SU_Controller *su_controller;
      
      unsigned int su_id;
      int serv_type;
      Msg_Handler_ID PROCESSOR_CONTROL_ID;
      Msg_Handler_ID SU_MONITOR_ID;
      
      bool init; //if cmd_service and connections successfully created
      SU_Control_Error error_msg;//persistent error message - configure to requirements
      
      bool handle_messages(void);//handle incomming messages
      bool check_message(Incoming_Message_Info* info);
      void transmit_error(Connection* target, Msg_Handler_ID dest_ID, unsigned int error_code,
			  unsigned int command_code = 0, unsigned int ref_id = 0);
      //transmits error msg to target - default unsolicited
      
      Msg_Handler_ID& get_PC_Handle_ID(void) {return PROCESSOR_CONTROL_ID;}
      Msg_Handler_ID& get_SUM_Handle_ID(void) {return SU_MONITOR_ID;}
      Connection* get_connection(int target);//returns connection to given target.
      bool transmit_medium_handlers(CCC_Msg *msg);//transmit msg to all medium handlers
      void close_medium_handler_conns(void);
      void close_data_handler_conn();
      bool open_DATA_HANDLER_Connection(int servtype);
      bool open_ROT_CLOCK_Connection(void);
      bool open_P_C_Connection(void);
      bool open_SUM_Connection(void);

};


#endif
