//transp.h

//SU_CONTROL application - transparent command channel

//JIVE, May 1996

//_________________________________________________________________________

//for RCS

//$Header: /export/home/jive_cvs/JCCS/CVS_Repository/su_control/include/transp.h,v 3.2 2008/04/09 11:44:27 jive_cc Exp $

//$Locker:  $

//$Log: transp.h,v $
//Revision 3.2  2008/04/09 11:44:27  jive_cc
//BE: first version of mark5b control, also added the possibility to use the spare mk5b for production
//
//Revision 3.1  2005/12/23 13:16:30  verkout
//HV: Phew. This is the big thing. Merged all developments with both the port to Linux and the mods needed for a byte-order safe COF
//
//Revision 3.0.8.1  2005/10/18 09:29:45  jive_cc
//HV: * JCCS port to Linux/i386 * COF is now endian safe * libraryname clash on Linux - libutil is a systemlib on Linux, so renamed util -> jccs_util * put CountedPointer in JCCS namespace
//
//Revision 3.0  2002/04/17 14:42:51  jive_cc
//Included gnu branch
//
//Revision 1.9  1999/08/30 09:56:52  jive_cc
//delete bad connectiond, use conn_id to find correct connection
//
//Revision 1.8  1999/03/04 17:18:44  pwm
//fixed dangling connection
//
//Revision 1.7  1999/01/29 16:08:11  pwm
//Integrated peaking branch into root
//
//Revision 1.6  1998/12/15 20:11:18  ps
//Message library reorganisation, part three
//
//Revision 1.5.2.1  1999/01/19 15:44:36  pwm
//added location_broker
//
//Revision 1.5  1998/08/26 20:01:48  jive_cc
//added quit()
//
//Revision 1.4  1998/07/01 11:55:41  pwm
//friends, interface
//
//Revision 1.3  1998/01/26 20:28:49  pwm
//added fns to determine tape_id. Cosmetic changes
//
//Revision 1.2  1997/11/07 15:36:12  pwm
//various more or less major changes
//
//Revision 1.1  1997/04/03 15:56:10  pwm
//Initial revision
//

//___________________________________________________________________________

#ifndef TRANSPARENT_H
#define TRANSPARENT_H

#include <su_control/su_conn.h>
#include <su_control/su_controller.h>
#include <su_control/su_defs.h>
#include <jccs_util/defs.h>
#include <controllers/thread.h>
#include <standard_threads/service_mgr.h>
#include <herbert/service.h>
#include <jive_message_sets/msg_defns.h>
#include <message_base/msg_handler.h>
#include <message_base/comm_msg.h>
#include <jive_msgs/ccc_msg_codes.h>
#include <herbert/t_base.h>
#include <time.h>
#include <store_classes/order_list.h> // for Ordered_List
#include <jccs_util/strobj.h>
#include <herbert/connection.h>
#include <location_client/location_client.h>
#include <jive_locations/service_codes.h>
#include <jive_locations/jive_locations.h>

class SU_Controller;
class SU_Command_Channel;
class SU_Connection;
class T_int;

class Transparent_channel : public Thread
{
   friend class SU_Connection;
   
   
   public :
      Transparent_channel(Thread_ID my_id, Thread_ID parent, Thread_Master *owner, SU_Command_Channel *comm1, 
			  unsigned int suid_code);
      ~Transparent_channel(); //destructor
      
      int run();
      int shutdown(); //close connection
      int install() {current_state = initial; return 0;}
      
      bool initialise(SU_Controller *suc, SU_Connection *comm3); //initiate connection 
      int quit();
      
   private :
      enum states {initial = 0, running, closedown};
      states current_state;
      
      //to determine local time, etc for messages
      struct timeval tod;
      struct timezone tzp;
      Long_Time_Value time_stamp;
      Long_Time_Value local_time;
      void set_local_time(void);
      
      Ordered_List connections;//list of connections ordered on ip_and_obj of source Msg_Handler
      SU_Connection *su_conn;//pointer to su_connection 
      SU_Controller *su_controller;
      SU_Command_Channel *su_cmd;
      
      Location_Client location_if;
      Service_Server* service;
      Service_Thread_Interface transp_interface;
      Service_Manager *transp_mgr;
      
      unsigned int su_id;
      int serv_type;
      
      Connection* get_connection(Msg_Handler_ID& destination);//returns connection to given target.
      
      bool init; //if service is successfully created
      SU_Control_Error error_msg;//persistent error message - configure to requirements
      
      void transmit_error(Connection* target, Msg_Handler_ID dest_ID, unsigned int error_code,
			  unsigned int command_code = 0, unsigned int ref_id = 0);
      //transmit error msg to target - default unsolicited
      
      void check_manager();
      void check_connections(void);
      
};

class Storeable_Conn : public StoreObject
{
   friend class Transparent_channel;
   
 private:
    
    Connection* the_conn;
    Connection_ID the_connid;
    
 public:
    
    Storeable_Conn(Connection *conn, Connection_ID& connid);
    ~Storeable_Conn(void) {;}
    
    Connection *get_conn(void) {return the_conn; }
    //superclass pure virtual functions
    char* hasName (void)const {return "Storeable_Conn";}
    ClassID isType(void)const {return 0;}
};

#endif
