#include <su_control/mark5b_control.h>
#include <iostream>

using namespace std;

int Mark5B_Control::construct_bit_stream_mask(Corr_Job_Descr* cjd, Logical_Array* array, Logical_Station* station, Scan* scan, Ext_Configure_Scan* conf_scan_msg) {
  int bit_stream_mask = 0;
  // search for the channels that should be active

  if (cjd && array && station && scan) {
    // we need 3 pieces of information about the channels: sign/magnitude, sideband and bbc, we can find these respectively in Channel (from Scan), Channel_Reference (from CJD) and Logical_Channel (from Logical_Station).
    // we start our journey with a Logical_Array and Logical_Station

    // for all telescopes that map to this station
    // get the channels to be correlated for the telescope
    for (int telescope = 0; telescope < array->get_nr_of_telescopes(); telescope++) {
      CompString key(array->get_telescope_name(telescope));
      StoreObject& object = (cjd->get_telescopes())->get(key); 
      if (object.valid() && ((Telescope&)object).get_station_id() == station->get_station_ID()) {
	for (int channel = 0; channel < array->get_nr_of_channels(); channel++) {
	  Log_Channel_El* log_chan_el = array->get_logical_channel(array->get_telescope_name(telescope), array->get_channel_name(channel));
	  if (log_chan_el) {
	    // find the referenced logical channel in the logical station structure
	    Logical_Channels_Block* block = station->get_log_chan_block_containing_ROT((scan->get_start_TOT() + scan->get_stop_TOT()) / 2);
	    if (block && log_chan_el->get_log_channel_nr() < block->get_nr_channels()) {
	      Logical_Channel log_chan = block->get_logical_channel(log_chan_el->get_log_channel_nr());
	      if (log_chan.get_physical_bbc()) { // if the channel has an unknown bbc, we cant continue anyway
		Channel_Reference* chan_ref = cjd->get_channel(log_chan.get_channel_reference());
		if (chan_ref) {
		  // so now we have the Logical_Channel and Channel_Reference, all we need is a Channel
		  Record_Format* rf = scan->get_record_format();
		  int channels_per_track = 1;
		  if (rf) {
		    channels_per_track = (int)ceil(0.9/rf->get_fan_out_factor());
		  }
		  else {
		    cout << "ERROR: Scan has no record format" << endl;
		  }
		  Channel channels[4];
		  for(int track = 0; track < 70; track++) {
		    scan->get_channel_track_link(track, channels);
		    for (int chan = 0; chan < channels_per_track; chan++) {
		      if (channels[chan].get_chan_ref() == log_chan.get_channel_reference()) {
			// we have found a triple Channel_Reference, Logical_Channel, Channel that match
			int bit_stream = (channels[chan].get_sign_magnitude() == sign? 0 : 1) +
			  (chan_ref->get_sideband() == upper? 0 : 16) + 
			  (log_chan.get_physical_bbc() - 1) * 2;
			bit_stream_mask |= (1 << bit_stream);

			if (conf_scan_msg) {
                          //cout << "reference " << log_chan.get_channel_reference() << ": channel " << channel << " bbc " << log_chan.get_physical_bbc() << (channels[chan].get_sign_magnitude() == sign? " sign " : " magnitude ") << (chan_ref->get_sideband() == upper? " upper " : " lower ") << endl;

			  conf_scan_msg->crossbar_map[channel * 2 + (channels[chan].get_sign_magnitude() == sign? 0 : 1)] = bit_stream; // map the bbc on the right channel
			}
		      } // the scan channel and logical station channel match
		    } // for all channels represented by track
		  } // for all possible tracks
		} // channel refrence found
		else {
		  cout << "Channel Reference not found for channel " << log_chan.get_channel_reference();
		}
	      } // bbc valid
	      else {
		cout << "Invalid bbc in channel nr " << log_chan_el->get_log_channel_nr() << endl;
	      }
	    } // block found and channel within bounds
	    else {
	      cout << "Cannot find block containing ROT " << Short_Time_Value((scan->get_start_TOT() + scan->get_stop_TOT()) / 2).get_rot_string() << " or channel index " << log_chan_el->get_log_channel_nr() << " out of bound" << endl;
	    }
	  }
	  else {
	    cout << "Cannot find log chan el for telescope " << array->get_telescope_name(telescope) << " and channel " << array->get_channel_name(channel) << endl;
	  }
	} // for all channels in array
      } // if telescope matches the station
    } // for all telescopes in array
  } // if logical array found
  return bit_stream_mask;
}
