#include <su_control/du_status.h>
#include <su_control/ndbg_mark5b.h>

using namespace std;

Mark5B_Status::Mark5B_Status() {
  _bank_enquired = -1;
  _last_status = 0;
}

string Mark5B_Status::get_mounted_vsn() const {
  if (_bank_enquired >= 0 &&  _bank_enquired < MAX_NR_BANKS) {
    return _mounted_vsn[_bank_enquired];
  }
  for (int i = 0; i < MAX_NR_BANKS; i++) {
    if (_mounted_vsn[i] != "") return _mounted_vsn[i];
  }
  return "";
}

string Mark5B_Status::get_mounted_vsn(int bank) const {
  if (bank >= 0 && bank < MAX_NR_BANKS) {
    return _mounted_vsn[bank];
  }
  NDBG(LOGSTREAM, WARNING|CONTROL, "Bank out of range in get vsn: " << bank << endl);
  return "";
}

void Mark5B_Status::set_mounted_vsn(const string& vsn, int bank) {
  if (bank < MAX_NR_BANKS) {
    _mounted_vsn[bank] = vsn;
  }
  else {
    NDBG(LOGSTREAM, WARNING|CONTROL, "Bank out of range in set vsn: " << bank << endl);
  }
}

bool Mark5B_Status::enquire_vsn(const std::string& vsn) {
  _bank_enquired = -1;
  //disk bar codes have to match or wildcard
  if (vsn == "") {
    _bank_enquired = MAX_NR_BANKS; // (ab)using this value for a wildcard enquire
    return true;
  }
  else {
    for (int bank = 0; bank < MAX_NR_BANKS; bank++) {
      if (vsn ==  _mounted_vsn[bank]) {
	_bank_enquired = bank;
	return true;
      }
    }
  }
  return false;
}

void Mark5B_Status::process_mounted_msg(const Mount_Change& msg) {
  for (int i = 0; i < MAX_NR_BANKS; i++) {
    string bank(msg.vsn[i],8);
    bank = (bank != "LABEL NO"?bank:"");
    if (i == _bank_enquired && bank != _mounted_vsn[i]) {
      NDBG(LOGSTREAM,LOG|CONTROL,"VSN enquired dismounted" << endl);
      _bank_enquired = -1;
    }
    _mounted_vsn[i] = bank;
  }
}

void Mark5B_Status::process_m5_status(const M5_Status& msg) {
  _last_status = msg.status;
}

U32 Mark5B_Status::get_status_word() {
  return _last_status;
}
