#include <su_control/scan_iterator.h>
#include <su_control/ndbg_mark5b.h>

using namespace std;

Scan_Iterator::Scan_Iterator() {
  _current_scan = NULL;
  _scans = NULL;
  _current_station = NULL;
  _current_medium = NULL;
}

bool Scan_Iterator::set_cjd(Corr_Job_Descr* cjd, const string& mounted_vsn) {
  _current_station = NULL;
  _current_medium = NULL;
  
  Forward_List_Iterator station_iter(cjd->get_logical_stations());
  if(station_iter.reset() && station_iter.current().isType() == LOGICAL_STATION_ID) {
    do {
      _current_station = &((Logical_Station &) station_iter.current());
      Forward_List_Iterator medium_iter(_current_station->get_recorded_media());
      if( _current_station->get_recorded_media()->count() > 0 && medium_iter.reset()) {
	while(mounted_vsn != ((Medium &)medium_iter.current()).get_medium_ID() && medium_iter.next()) { //find medium with barcode == mounted vsn
	  NDBG(LOGSTREAM,CONTROL|DEBUG, "comparing " << mounted_vsn << " to " << ((Medium &)medium_iter.current()).get_medium_ID() << " station " << _current_station->get_station_ID() << endl);
	}
	NDBG(LOGSTREAM,CONTROL|DEBUG, "comparing " << mounted_vsn << " to " << ((Medium &)medium_iter.current()).get_medium_ID() << " station " << _current_station->get_station_ID() << endl);	
	if(mounted_vsn == ((Medium &)medium_iter.current()).get_medium_ID()) {
	  NDBG(LOGSTREAM,CONTROL|DEBUG,"succes" << endl);
	  _current_medium = &((Medium &) medium_iter.current());
	}
      }
      else {
	NDBG(LOGSTREAM,CONTROL|DEBUG,"no media for station " << _current_station->get_station_ID() << endl);

      }
    } while (station_iter.next() && !_current_medium);
  }
  else { 
    NDBG(LOGSTREAM,WARNING|CONTROL,"set_cjd() failed to reset iterator" << endl);
  }
    
  if (_current_station == NULL || _current_medium == NULL) {
    NDBG(LOGSTREAM,WARNING|CONTROL,"set_cjd() finds no current station or medium." << endl);
    return false;
  }
  NDBG(LOGSTREAM,INFO|CONTROL,"set_cjd() current_station " << _current_station->get_station_ID() << endl);
  NDBG(LOGSTREAM,INFO|CONTROL,"set_cjd() current_medium " << _current_medium->get_medium_ID() << endl);
  
  _scans = _current_medium->get_scan_list();
  _scans->iterator().reset();
  _current_scan = &((Scan &) _scans->iterator().current());
  if (mounted_vsn.substr(3,5) != "-eVLBI") {
    while(_current_scan->get_stop_TOT() < cjd->get_sub_job_starting_ROT() && _scans->iterator().next()) {
      _current_scan = &((Scan &) _scans->iterator().current());
    }
    if(_current_scan->get_stop_TOT() < cjd->get_sub_job_starting_ROT()) {
      NDBG(LOGSTREAM,WARNING|CONTROL,"set_cjd() finds scan stop_TOT <= subjob start ROT" << endl);
      return false;
    }
  }
  set_end_and_next_scan();
  NDBG(LOGSTREAM,LOG|CONTROL,
       "current scan start = " << Short_Time_Value(_current_scan->get_start_TOT()).get_rot_string() << endl << 
       "current scan  stop = " << Short_Time_Value(_current_scan->get_stop_TOT()).get_rot_string() << endl << 
       "sub job start = " << Short_Time_Value(cjd->get_sub_job_starting_ROT()).get_rot_string() << endl <<
       "sub job  stop = " << Short_Time_Value(cjd->get_sub_job_ending_ROT()).get_rot_string() << endl);
  
  return true;
}

Scan* Scan_Iterator::get_current_scan() {
  return _current_scan;
}

Scan* Scan_Iterator::peak_next_scan() {
  return _next_scan;
}

Scan* Scan_Iterator::get_end_scan() {
  return _end_scan;
}

void Scan_Iterator::set_end_and_next_scan() {
  _end_scan = _current_scan;
  bool next;
  while((next = _scans->iterator().next()) && ((Scan&)_scans->iterator().current()).get_start_TOT() <= _end_scan->get_stop_TOT()) { // merge gap-less scans
    _end_scan = &((Scan &) _scans->iterator().current());
  }  
  if (next) {
    _next_scan = &((Scan &) _scans->iterator().current());
  }
  else {
    _next_scan = NULL;
  }    
}

void Scan_Iterator::goto_next_scan() {
  _current_scan = _next_scan;
  set_end_and_next_scan();
}

Logical_Station* Scan_Iterator::get_containing_station() {
  return _current_station;
}

Medium* Scan_Iterator::get_containing_medium() {
  return _current_medium;
}

