#include <su_control/mark5b_connection.h>
#include <string>

using namespace std;

extern int mark5b_debug_exclude;

Mark5B_Connection conn;

//#define GEO

//#define CONFIG

void check_msgs() {
  Mount_Change* mount_change = conn.get_message<Mount_Change>();
  if (mount_change) {
    cout << "bank A " << string(mount_change->vsn[0],8) << " bank B " << string(mount_change->vsn[1],8) << endl;
  }
  M5_Status* stat = conn.get_message<M5_Status>();
  if (stat) {
      cout << "stats " << hex << stat->status << endl;
  }    
}

int main(int argc, char** argv) {
  const int BOCF = 64;

  //mark5b_debug_exclude = CONNECTION;
  conn.connect_socket("markv8");
  int su_id = 8;
  int job_id = 65535;
  int realtime_mode = 0;

  Ext_Initialize_Mk5 init_msg(job_id);
  init_msg.cf_length = (U32)32e6 / BOCF;
  init_msg.bocf_stretch = 2; // means a bocf frame of 240 sysclicks
  init_msg.su_id = su_id;
  init_msg.send_mounts = 1; // send all mount changes
  init_msg.realtime = realtime_mode;
  init_msg.packet_size = 9000;
  conn.send_message(init_msg);
  Mount_Change* mount_change = NULL;
  struct timeval wait;
  while (!mount_change) {
    wait.tv_sec = 1;
    wait.tv_usec = 0;
    cout << "retreived " << conn.retreive_messages(wait,su_id) << " messages" << endl;
    mount_change = conn.get_message<Mount_Change>();
  }
  cout << "bank A " << string(mount_change->vsn[0],8) << " bank B " << string(mount_change->vsn[1],8) << endl;

  init_msg.cf_length = (U32)32e6 / BOCF;
  init_msg.bocf_stretch = 2; // means a bocf frame of 240 sysclicks
  init_msg.su_id = su_id;
  init_msg.send_mounts = 1; // send all mount changes
  init_msg.realtime = realtime_mode;
  init_msg.packet_size = 9000;
  conn.send_message(init_msg);

  mount_change = NULL;
  while (!mount_change) {
    wait.tv_sec = 1;
    wait.tv_usec = 0;
    cout << "retreived " << conn.retreive_messages(wait,su_id) << " messages" << endl;
    mount_change = conn.get_message<Mount_Change>();
  }
  cout << "bank A " << string(mount_change->vsn[0],8) << " bank B " << string(mount_change->vsn[1],8) << endl;

  /*
  int julian_year = 2007 + 4799;//(header_contents->df_header[1] >> 28) & 0xf;
  int julian_begin_of_year = julian_year * 365 + julian_year / 4 - julian_year / 100 + julian_year / 400 - 31739; // julian day number for the beginning of the observation year
  int julian_day_number = 100 * (header_contents->df_header[2] >> 28 & 0xf) 
                         + 10 * (header_contents->df_header[2] >> 24 & 0xf) 
                         +      (header_contents->df_header[2] >> 20 & 0xf); // actually % 1000
  int day_of_year = (julian_day_number - julian_begin_of_year % 1000 + 1000) % 1000;
  int seconds_in_day = 10000 * (header_contents->df_header[2] >> 16 & 0xf)
                      + 1000 * (header_contents->df_header[2] >> 12 & 0xf)
                      +  100 * (header_contents->df_header[2] >>  8 & 0xf)
                      +   10 * (header_contents->df_header[2] >>  4 & 0xf)
                      +        (header_contents->df_header[2]       & 0xf);
  cout << "jy\t" << julian_year << endl 
       << "jboy\t" << julian_begin_of_year << endl 
       << "jdn\t" << julian_day_number << endl
       << "doy\t" << day_of_year << endl
       << "sid\t" << seconds_in_day << endl;
  */
#ifdef CONFIG

  Ext_Send_M5_Status req(job_id);
  req.interval = 1;
  conn.send_message(req);

  wait.tv_sec = 1;
  wait.tv_usec = 0;
  if (conn.retreive_messages(wait,su_id) < 0) {
    return 1;
  }
  check_msgs();

  Ext_Configure_Scan conf_scan(job_id);
  conf_scan.byte_num = 0;

  struct timeval tod;
  struct timezone tzp;
  gettimeofday(&tod, &tzp);//get secs since 1/1/70
  time_t t((long)(tod.tv_sec));//cast to type time_t
  tm* gmt = gmtime(&t);//convert to UT(GMT) - for struct of tm see manpage of ctime
  conf_scan.start_rot = 32e6 * (gmt->tm_yday*24*60*60 + gmt->tm_hour*60*60 + gmt->tm_min*60 + gmt->tm_sec + 70);

  cout << "start_rot " <<  conf_scan.start_rot << endl;
  conf_scan.end_rot = conf_scan.start_rot + 120 * 32e6;
  conf_scan.ss_invalid_code = 0x11223344; // standard values (according to mail from Roger Cappallo)
  conf_scan.dim_invalid_code = 0x11223344;
  for (int i = 0; i < 32; i++) {
    conf_scan.crossbar_map[i] = i;
  }
  const int rate = 16;
  conf_scan.sample_rate = rate;
  conf_scan.output_rate = rate;
  conf_scan.num_bitstreams = 32;
  conf_scan.bits_sample = 2;
  strlcpy(conf_scan.scan_label, "test1", sizeof(conf_scan.scan_label));

  Ext_Model_Data model(job_id);
  model.rot_interval[0] = conf_scan.start_rot - 60 * 32e6;
  model.rot_interval[1] = conf_scan.end_rot + 60 * 32e6;
  for (int i = 0; i < 6; i++) {
    model.delay[i] = 0;
    for (int j= 0; j < 16; j++) {
      model.phase[j][i] = 0;
    }
  }
  model.sidebands = 0xffff;
  model.nchan = 16;
  model.oversampling = 1;
  model.k_div = 1;

  conn.send_message(model);

  wait.tv_sec = 1;
  wait.tv_usec = 0;
  if (conn.retreive_messages(wait,su_id) < 0) {
    return 1;
  }
  check_msgs();

  conn.send_message(conf_scan);

  wait.tv_sec = 1;
  wait.tv_usec = 0;
  if (conn.retreive_messages(wait,su_id) < 0) {
    return 1;
  }
  check_msgs();

  while (true) {
    wait.tv_sec = 1;
    wait.tv_usec = 0;
    if (conn.retreive_messages(wait,su_id) < 0) {
      return 1;
    }
    check_msgs();
  }

#else
  Ext_Find_Next_Header find_header(job_id);
  find_header.byte_num = 567101469420ll;
  Header_Contents* header_contents;
  do {
    conn.send_message(find_header);
    header_contents = NULL;
    while (!header_contents) {
      wait.tv_sec = 1;
      wait.tv_usec = 0;
      conn.retreive_messages(wait,su_id);
      check_msgs();
      header_contents = conn.get_message<Header_Contents>();
      find_header.byte_num = header_contents->byte_num + 100 * 2504*4;
    }
    cout << "bytenum " << header_contents->byte_num << hex << " header " << header_contents->df_header[0] << " " << header_contents->df_header[1] << " " << header_contents->df_header[2] << " " << header_contents->df_header[3] << dec;
    int julian_year = 2007 + 4799;//(header_contents->df_header[1] >> 28) & 0xf;
    int julian_begin_of_year = julian_year * 365 + julian_year / 4 - julian_year / 100 + julian_year / 400 - 31739; // julian day number for the beginning of the observation year
    int julian_day_number = 100 * (header_contents->df_header[2] >> 28 & 0xf) 
      + 10 * (header_contents->df_header[2] >> 24 & 0xf) 
      +      (header_contents->df_header[2] >> 20 & 0xf); // actually % 1000
    int day_of_year = (julian_day_number - julian_begin_of_year % 1000 + 1000) % 1000;
    int seconds_in_day = 10000 * (header_contents->df_header[2] >> 16 & 0xf)
      + 1000 * (header_contents->df_header[2] >> 12 & 0xf)
      +  100 * (header_contents->df_header[2] >>  8 & 0xf)
      +   10 * (header_contents->df_header[2] >>  4 & 0xf)
      +        (header_contents->df_header[2]       & 0xf);
    cout << " ROT " << day_of_year << "/" << seconds_in_day/(60*60) << ":" << seconds_in_day/60%60 << ":" << seconds_in_day%60 << endl;
  } while (header_contents->byte_num >= 0);


#endif

  return 0;
}
