//transp.C

//SU_CONTROL application - transparent command channel
//JIVE, May 1996

//_________________________________________________________________________

//for RCS

//$Header: /export/home/jive_cvs/JCCS/CVS_Repository/su_control/source/transp.C,v 3.6 2008/04/09 11:44:31 jive_cc Exp $

//$Locker:  $

//$Log: transp.C,v $
//Revision 3.6  2008/04/09 11:44:31  jive_cc
//BE: first version of mark5b control, also added the possibility to use the spare mk5b for production
//
//Revision 3.5  2007/10/05 11:28:06  jive_cc
//BE: station reconnect, also contains (1) auto bank switching (during configuration only), (2) reduced evlbi startup delay, (3) evlbi params in database are queried on-the-fly, (4) data handler DTD gathering using dummy data, (5) automatic data status monitor statup (for evlbi experiments), (6) unfinished spacecraft tracking mode
//
//Revision 3.4  2005/12/23 13:16:56  verkout
//HV: Phew. This is the big thing. Merged all developments with both the port to Linux and the mods needed for a byte-order safe COF
//
//Revision 3.2.4.1  2005/10/25 15:02:12  verkout
//HV: Merge with main branch. May need fixes in order to compile on jaw0/HP-UX
//
//Revision 3.3  2005/09/30 12:31:50  jive_cc
//Arpad: e-VLBI enabled version
//
//Revision 3.2  2004/06/25 13:21:49  verkout
//gnu3.2 version: many many MANY way too many changes
//
//Revision 3.1  2003/05/15 11:46:36  jive_cc
//Arpad: merged MarkV version into main
//
//Revision 3.0.2.1  2003/01/13 15:09:27  jive_cc
//*** empty log message ***
//
//Revision 3.0  2002/04/17 14:45:47  jive_cc
//Included gnu branch
//
//Revision 1.13.4.1  2002/01/07 15:51:41  jive_cc
//Friso and Arpad: many many MANY pointers were explicitely set to zero after being deleted to avoid memory problems caused by double deletions
//
//Revision 1.13  1999/08/30 17:13:38  jive_cc
//minor changes, note the Connection_ID copy constructor disn't!
//
//Revision 1.12  1999/08/30 09:46:43  jive_cc
//back to start footage once only
//
//Revision 1.11  1999/03/08 17:18:49  pwm
//debug get_connection()
//
//Revision 1.10  1999/03/04 17:18:12  pwm
//fixed dangling connection
//
//Revision 1.9  1999/01/29 16:07:17  pwm
//Integrated peaking branch into root
//
//Revision 1.8  1999/01/08 13:11:47  jive_cc
//In get_connection(): check connection for ready_to_write.
//
//Revision 1.7.2.2  1999/01/19 15:44:09  pwm
//added location_broker
//
//Revision 1.7.2.1  1998/11/19 15:21:28  pwm
//change in SU_Control_Status msg
//
//Revision 1.7  1998/09/16 18:35:49  pwm
//temp? hack to pass set_ROT msgs
//
//Revision 1.6  1998/08/26 20:00:47  jive_cc
//added quit()
//
//Revision 1.5  1998/07/01 12:25:07  pwm
//many minor corrections, updated transmit_tape_status() as reqd
//
//Revision 1.4  1998/01/26 20:19:03  pwm
//Added service manager
//
//Revision 1.3  1997/11/07 15:30:56  pwm
//added Service_Manager to try it out
//
//Revision 1.2  97/06/09  18:30:44  18:30:44  pwm (Paul William Maguire)
//changes mainly reflecting changes in ccc_msg
//
//Revision 1.1  97/04/03  15:20:32  15:20:32  pwm (Paul William Maguire)
//Initial revision
//

//___________________________________________________________________________

#include <su_control/transp.h>

using namespace std;
Storeable_Conn::Storeable_Conn(Connection *conn, Connection_ID& connid)
   : the_conn(conn)
{
  the_connid = connid;
}

Transparent_channel::Transparent_channel(Thread_ID my_id, Thread_ID parent, Thread_Master *owner, \
					 SU_Command_Channel *comm1, unsigned int suid_code)
   		:Thread(my_id, parent, owner),
		 su_id(suid_code)
{
   su_conn = 0;
   su_controller = 0;
   su_cmd = comm1;
   init = false;
   service = 0;
   transp_mgr = 0;
}

Transparent_channel::~Transparent_channel()
{
  cout << "~Transparent_channel" << endl;
}

void Transparent_channel::set_local_time(void)
{
   //set local time
   
   ieee_dbl sysclks;
   
   gettimeofday(&tod, &tzp);
     
   time_t t((long)(tod.tv_sec));
   
   tm *temp = gmtime(&t);
   sysclks = SYSTEM_RATE * (temp->tm_yday*24*60*60 + temp->tm_hour*60*60 + temp->tm_min*60 + temp->tm_sec);
   
   local_time.set_sysclks(sysclks);
   local_time.set_base_date(temp->tm_year < 90 ? 2000 + temp->tm_year : 1900 + temp->tm_year);
}

      
int Transparent_channel::run()
{
   //receive and deal with messages
   
   if(init)
   {
      this->check_connections();
      this->check_manager();
      return 0;
   }
   else return 1;
}

bool Transparent_channel::initialise(SU_Controller *suc, SU_Connection *comm3)
{
   Connection *target;
   
   su_conn = comm3;
   su_controller = suc;
   
   //connections from whatever - SU_CONTROL == server
   Service_ID *transparent_channel = location_if.get_service_id(suc_trans_servtypes[su_id]);
   
   //create the required service - in this case a server is required
   if(transparent_channel &&
      (service = new Service_Server(*transparent_channel)) &&
      (transp_mgr = new Service_Manager(*get_controller(),
					get_id(),
					*service,
					transp_interface)))
   {
      service->create_listener(Connection_ID::unix_sock);
      transp_interface.set_receive(true); //ie handle incoming messages
      transp_interface.set_server_handling(true);
      get_controller() -> add_thread(transp_mgr);
      location_if.register_server(suc_trans_servtypes[su_id], su_id);
      init = true;
   }
   else
   {
      //transmit error message to PROCESSOR_CONTROL
      //error code?
      
      target = su_cmd->get_connection(PROCESSOR_CONTROL);
      this->transmit_error(target, su_cmd->get_PC_Handle_ID(), SU_Control_Error_Codes::Transparent_Channel_Exception,
			   CCC_Action_Request::Init_SU_Control);
   }

   return init;
 
}

int Transparent_channel::shutdown()
{
   return 0;
}

int Transparent_channel::quit()
{
   location_if.remove_server(suc_trans_servtypes[su_id], su_id);
   this->set_stat(Thread::terminating);
   transp_interface.set_quit(true);
   return 0;
}

void Transparent_channel::check_connections(void)
{
  // search the list of connections for broken ones
  CompInt *key = 0;
  StoreObject  *obj = 0;
  Connection* tmp_conn = 0;
  while(tmp_conn = service->poll(tmp_conn, Service_Client::exception))
  {
    service->disconnect(*(tmp_conn->identity()));
  }

  //how to remove the appropriate obj from connections ????
}


Connection* Transparent_channel::get_connection(Msg_Handler_ID& destination)
{
   Connection *connection;
   Connection_ID *conn_id = 0;
   CompInt *key = 0;
   StoreObject  *obj = 0;
   bool good = false;
   
   if(key = new CompInt(destination.ip_and_obj()))
   {
      obj =  &(connections.get(*key));
      if(obj->valid())
      {
	 connection = service->find_connection(((Storeable_Conn *)obj)->the_connid);
	 if(connection && connection->ready_to_write())good = true;
	 else delete connections.remove(*key);
      }
   }
   delete key; key = 0;
   
   if(good) return connection;
   else return 0;
}

	
void Transparent_channel::transmit_error(Connection* target, Msg_Handler_ID dest_ID, unsigned int error_code,
					 unsigned int command_code, unsigned int ref_id)
{
   //transmit error message to target
   this->set_local_time();

   error_msg.set_error_code(error_code);
   error_msg.set_local_time(local_time);
   error_msg.set_time_stamp(time_stamp);
   error_msg.set_su_id(su_controller->get_suid_code());
   error_msg.set_job_id(su_controller->get_job_id());
   error_msg.set_sender_id(su_controller->get_SUC_ID());
   error_msg.set_receiver_id(dest_ID);
   error_msg.set_action_code(command_code);
   error_msg.set_ref_id(ref_id);
   if(target != 0) error_msg.transmit(*target);
}


void Transparent_channel::check_manager()
{
  Incoming_Message_Info* info;             // Incoming message/connection tuple
  Connection* tempconn;
  CCC_Msg* ccc_msg;
  Action_Status* msg;
  Command_Message *cmd_msg, *cmd;
  unsigned int action_code, msg_id, cmd_code;
  Connection* target;

  for (info = transp_interface.get_incoming_iter().first();
       info;
       info = transp_interface.get_incoming_iter().advance())
  {
    tempconn = info->get_conn();
    ccc_msg = (CCC_Msg*) info->get_msg();
    //get action_code
    action_code = (unsigned int)ccc_msg->get_action_code();
    //get _msg_id
    msg_id = (unsigned int)ccc_msg->get_msg_id();


    // Take the message info off the 'incoming' list to stop this message
    // being handled again.
    transp_interface.get_incoming_iter().drop();
    
    if(ccc_msg->get_message_type() == SU_Message_Types::Action_Status)
    {
       //at this point not interested in action status
       delete ccc_msg;
       delete info;
       ccc_msg = 0;
       info = 0;
       continue;
    }
    
    //temporary
    if(action_code == SU_Action_Values::Set_ROT)
    {
      if (su_conn) {
	target = su_conn->get_connection(S_U);
	if(target != 0 )
	  {
	    ccc_msg->transmit(*target);
	  }
	delete ccc_msg;
	delete info;
	ccc_msg = 0;
	info = 0;
	continue;
      }
    }
    
    //if command message
    if(action_code > SU_Action_Masks::Command_Message)
    {
       //cast object to cmd_msg
       cmd_msg = (Command_Message*) ccc_msg;
    }
    else
    {
       delete ccc_msg;
       delete info;
       ccc_msg = 0;
       info = 0;   
       continue;
    }
    
    //now obtain destination of message
    Msg_Handler_ID destination = cmd_msg->get_receiver_id();
    //get source
    Msg_Handler_ID source = cmd_msg->get_sender_id();
    //get cmd_code
    cmd_code = cmd_msg->get_command_code();
    
    
    if(! (su_controller->initiated())) 
    {
       //form error action status message and transmit to originator - error code? 
       this->transmit_error(tempconn, source, SU_Control_Error_Codes::Initialised_Exception,
			    action_code, msg_id);
       delete ccc_msg;
       delete info;
       ccc_msg = 0;
       info = 0; 
       continue; //go back to beginning of loop
    }
    
    //if(destination == SU || DPU || cmd_code == Flush_Queue)
    if(action_code == SU_Action_Values::Station_Unit ||
       cmd_code == SU_Command_Queue_Commands::Flush_Queue)
    {
       //transmit command to SU if ... or flush queue command
      if (su_conn) {
	target = su_conn->get_connection(S_U);
	if(target != 0 )
	  {
	    cmd_msg->transmit(*target);
	    //save conn
	    CompInt* key = 0;
	    Storeable_Conn *obj = 0;
	    
	    if((key = new CompInt(source.ip_and_obj())) &&
	       (connections.get(*key)).valid())
	      {
		//ie if we already know about this connection
		delete ccc_msg;
		delete info;
		delete key;
		ccc_msg = 0;
		info = 0;
		key = 0;
		continue;
	      }
	    
	    if((obj = new Storeable_Conn(tempconn, *(tempconn->identity()))) &&
	       (connections.add(key,obj)));
	    else
	      {
		delete key; key = 0;
		delete obj; obj = 0;
		this->transmit_error(tempconn, source, SU_Control_Error_Codes::Memory_Allocation_Exception,
				     action_code, msg_id);
	      }
	    delete ccc_msg;
	    delete info;
	    ccc_msg = 0;
	    info = 0;
	    continue;
	  }
	else
	  {
	    //return error message to originator - error code?
	    this->transmit_error(tempconn, source, SU_Control_Error_Codes::SU_Connection_Exception,
				 action_code, msg_id);
	    delete ccc_msg;
	    delete info;
	    ccc_msg = 0;
	    info = 0;
	    continue;
	  }
      }
    }
    
    switch (cmd_code) {
       case (CCC_Action_Request::Enquire_SU_Control) :
	 {
	   //form and transmit SU Control Status msg - have to query su_controller for parameters
	   
	   unsigned int status;
	   if (su_controller->use_mark5) status = su_controller->transmit_disk_status();
	   else status = su_controller->transmit_tape_status();
	   
	   if(status == -1)//no conn to sum
	     {
	       status = 0;
	       if(su_controller->is_mounted())
		 {
		   status += SU_Control_SU_Status_Codes::Medium_Mounted;
		 }

	       if(su_controller->get_allocation())
		 {
		   status +=  SU_Control_SU_Status_Codes::SU_Allocated;
		 }
	     }
	   int_32 head_positions[4];
	   su_controller->get_head_positions(head_positions);
	   
	   char medium_bar_code[MAX_NR_BANKS][MEDIUM_ID_LEN];
	   for (int bank = 0; bank < MAX_NR_BANKS; bank++) {
	     strncpy(medium_bar_code[bank],su_controller->get_barcode(bank),MEDIUM_ID_LEN);
	   }
	   if(msg = new SU_Control_Status(su_controller->get_suid_code(),
					  medium_bar_code,
					  DPUType,
					  su_controller->get_DPU_id(),
					  status,
					  su_controller->get_allocation() ? su_controller->get_job_id() : 0,
					  su_controller->num_head_stacks,
					  head_positions))
	     {
	       this->set_local_time();
	       msg->set_sender_id(su_controller->get_SUC_ID());
	       msg->set_receiver_id(source);
	       msg->set_ref_id(msg_id);
	       msg->set_local_time(local_time);  
	       msg->set_time_stamp(time_stamp);
	       msg->transmit(*tempconn);
	       delete msg;
	       msg = 0;
	     }
	   else
	     {
	       this->transmit_error(tempconn, source, SU_Control_Error_Codes::Memory_Allocation_Exception,
				    action_code, msg_id);
	     }
	   delete ccc_msg;
	   delete info;
	   ccc_msg = 0;
	   info = 0;
	   
	   break;
	 }//end of case
    case (CCC_Action_Request::Resync):
      //form reset SUIM command code and transmit to SU
	       
	  if(cmd = new Command_Message(SU_Station_Unit_Cmds::Reset_SUIM))
	  {
	    if (su_conn) {
	      //do we do anything with the Resynch msg_id?
	      this->set_local_time();
	      
	      cmd->set_local_time(local_time);  
	      cmd->set_time_stamp(time_stamp);
	      cmd->set_obey_ROT(cmd_msg->get_obey_ROT());
	      cmd->set_end_ROT(cmd_msg->get_end_ROT());
	      cmd->set_rpt_intrvl(cmd_msg->get_rpt_intrvl());
	      cmd->set_flags(cmd_msg->get_flags());
	      cmd->set_sender_id(su_controller->get_SUC_ID());
	      cmd->set_receiver_id(su_conn->get_SUID());
	      //transmit cmd to SU
	      target = su_conn->get_connection(S_U);
	      if(target != 0 )
		{
		  cmd->transmit(*target);
		  delete cmd;
		  cmd = 0;
		}
	      //or if transmit fails transmit error to originator
	      else 
		{
		  this->transmit_error(tempconn, source, SU_Control_Error_Codes::SU_Connection_Exception,
				       action_code, msg_id);
		  delete cmd;
		  cmd = 0;
		}
	    }
	  }
	  else
	  {
	    this->transmit_error(tempconn, source, SU_Control_Error_Codes::Memory_Allocation_Exception,
				 action_code, msg_id);
	  }
	  delete ccc_msg;
	  delete info;
	  ccc_msg = 0;
	  info = 0;
	  break;
	    
       default:
	  //transmit error message to originator
	  this->transmit_error(tempconn, source, SU_Control_Error_Codes::Transparent_Command_Exception,
			       action_code, msg_id);
	  delete ccc_msg;
	  delete info;
	  ccc_msg = 0;
	  info = 0;  
	  break;
    }//end switch
  }//end for
}//end method
