//init_proc_p.C

//SU_CONTROL application - private implementation of init_proc

//JIVE, November 1997

//_________________________________________________________________________

#include <su_control/init_proc.h>
#include <su_control/mark5b_control.h>
#include <iomanip>
#include <jccs_util/nint.h>
#include <herbert/machine.h>
#include <sys/time.h>
#include <sstream>
#include <data_handler/PhaseCalDataRecord.h>
#include <proc_job/defs.h>
#include <su_control/parse_mk5.h>
#include <bitset>
#include <fstream>

using namespace std;

//extern int h_errno; // From gethostbyname() for herror() --- needed for socket comm
//extern void herror(const char * s); // Needed on HP-UX --- needed for socket comm

bool ignore_next_servo_error = true;
ieee_dbl disk_servo_error = 0;
ieee_dbl temp_servo_error = 0;


#define DELETE_ALL delete aux_table; aux_table = 0; delete table1; table1 = 0; delete table2; \
   table2 = 0; delete table3; table3 = 0; delete table4; table4 = 0; delete table5; table5 = 0; \
   delete table6; table6 = 0; delete gen_tab1; gen_tab1 = 0; delete gen_tab2; gen_tab2 = 0; \
   delete gen_tab3; gen_tab3 = 0; delete gen_tab4; gen_tab4 = 0; delete gen_tab5; gen_tab5 = 0; \
   delete gen_tab6; gen_tab6 = 0; delete cmd; cmd = 0

#define DELETE_CMD delete cmd; cmd = 0; aux_table = 0; table1 = 0; table2 = 0; table3 = 0; table4 = 0; \
   table5 = 0; table6 = 0; gen_tab1 = 0; gen_tab2 = 0; gen_tab3 = 0; gen_tab4 = 0; gen_tab5 = 0; gen_tab5 = 0


string Init_Proc::form_net2disk_command(ieee_dbl rot) { // return "open:scan_ID", where if rot >= 0 : scan_ID is the name of the first scan  with end ROT > rot, else scan_ID is current_scan->get_scan_ID
  cout << "forming net2disk command for rot " << rot << endl;
  string command = "open:";
  if (rot < 0) {
    if (current_scan) {
      command += current_scan->get_scan_ID();
    }
    else {
      command += "unknown";
    }
  }
  else {
    if (current_medium) {
      Ordered_List* scans = current_medium->get_scan_list();
      scans->iterator().reset();
      Scan* current_scan = &((Scan &) scans->iterator().current());
      while(current_scan->get_stop_TOT() <= rot && scans->iterator().next()) {
	current_scan = &((Scan &) scans->iterator().current());
      }
      command += current_scan->get_scan_ID();
    }
    else {
      command += "unknown";
    }
  }
  cout << "command = " << command << endl;
  return command;
}

void Init_Proc::set_local_time(void)
{
   //set local time

   ieee_dbl sysclks;

   ::gettimeofday(&tod, &tzp);
 
   time_t t((time_t)(tod.tv_sec));

   tm *temp = ::gmtime(&t);
   sysclks = SYSTEM_RATE * 
	   			(temp->tm_yday*24*60*60 +
				 temp->tm_hour*60*60 +
				 temp->tm_min*60 + 
				 temp->tm_sec);

   local_time.set_sysclks(sysclks);
   // y2k windowing hack... valid until 2089...
   local_time.set_base_date( ((temp->tm_year<90)?
			   				  (2000 + temp->tm_year):
							  (1900 + temp->tm_year)) );
}

void Init_Proc::do_the_mask(const Record_Format *record_format)
{
   Record_Mode recording_mode = record_format->get_record_mode();
   //enum Record_Mode {Mark3A,Mark4,Mark5A,Vlba,S2,K4};
   //enum Tape_Format {MkIIIA = 1, MkIIIAx, MkIV, MkIVA, VLB, MkVA};
	 
   switch (recording_mode) {
      case (Mark3A) :
	 the_tape_format = MkIIIA;
	 the_mask[0] = 0;//TEMP ! 0xffffffff;// = all bits set;
	 the_mask[1] = 0;//TEMP ! 0xffff00ff;// = 11111111111111110000000011111111;
	 break;
	     
      case (Mark4) :
	 the_tape_format = MkIV;
	 the_mask[0] = 0;//0xffffffff;//headstack positions - all bits set to 1
	 the_mask[1] = 0;//0xff300000; // = 11111111001100000000000000000000;
	 break;
	    
      case (Vlba) :
	 the_tape_format = VLB;
	 the_mask[0] = 0;//0x00ffffff; // = 00000000111111111111111111111111;
	 the_mask[1] = 0;//0xfff00000; // = 11111111111100000000000000000000;
	 break;
   }
}

//________________________________________________________________________________

bool Init_Proc::do_config_arrays(Scan *scan, int &nr_tracks)
{
  int i, j, k;//counters
  the_sidebands = 0;
  static int index = 0;
  int logical_channel_nrs[NUM_CHANNELS];
  ieee_dbl time_slot = scan->get_start_TOT();
  //current_scan->get_start_TOT() + 
  //   (current_scan->get_stop_TOT() - current_scan->get_start_TOT());
  int xbc_l_index = 0, xbc_u_index = 32;//index for bottom and top end of dim_x_bar_configuration[NUM_TRACKS]
  unsigned int the_bit_pattern = 0; 
  int phases = (int)scan->get_barrel_rolling()->get_nr_of_periods();
  //returns an int_32 max value 16
  //int recorder_ID = current_medium->get_recorder()->get_recorder_ID();
  int num_chans_ref;//number of channels per channel_reference
  
  //muxmode for this scan can either be 4, 2, 1, 0.5 or 0.25
  int num_chans;//1, 1, 1, 2, 4 respectively
  
  //    Channel channels[4], *chans_ptr;
  Channel *chans_ptr;
  
  if(muxmode >= 1)
    {
      num_chans = 1;
    }
  else
    {
      if(muxmode < 0.4)
        {
	  num_chans = 4;
        }
      else
        {
	  num_chans = 2;
        }
    }
  
  
  //reinitialise the various arrays, 
  for(i = 0; i < NUM_TRACKS; ++i)
    {
      dim_x_bar_configuration[i] = 80;
      the_active_tracks[i] = 0;
      for(j = 0; j < NUM_CHANNELS; ++j)
        {
	  the_aux_data[j][i][0] = 0;
	  the_aux_data[j][i][1] = 0;
	  the_cross_bar_config[j][i] = 0;
        }
    }
  
  for(i = 0;  i < NUM_CHANNELS; ++i)
    {
      for(j = 0; j < 9; ++j)
        {
	  mux_table[i][j] = -1;
        }
    }
  
  for(i = 0;  i < NUM_CHANNELS; ++i)
    {
      output_mapping[i] = 0xFFFFFFFF;
      log_chan_ref_table[i] = std::string();
      for(j = 0; j < 73; ++j)
        {
	  for(k = 0; k < 2; ++k)
	    {
	      log_chan_track_link[i][j][k] = unknown;
	    }
        }
    }
  
  nr_tracks = 0;//re_initialisation
  
  //first for the_aux_data[*][*][0]
  this->do_aux_data(scan);
  
  int tracks[4];
  int vex_head = -1;

  for(j = 0; j < 70; ++j)
    {
      scan->get_tracks(j, tracks);
      
      for(k = 0; k < 4; ++k)
	{
	  if (tracks[k] <  36 && vex_head != 2) {vex_head = 1; if (!use_mark5) break;} //tape has only one headstack
	  if (tracks[k] <  36 && vex_head == 2) {vex_head = 3; break;}
	  if (tracks[k] >= 36 && vex_head != 1) {vex_head = 2; if (!use_mark5) break;} 
	  if (tracks[k] >= 36 && vex_head == 1) {vex_head = 3; break;}
	  break;
	}
      if (vex_head == 3 || ((vex_head == 1 || vex_head == 2) && !use_mark5)) break;
    }
  


  //now find out what headstacks connect to what DIMs
  SU_Conns_Query *query = (SU_Conns_Query *) mystore.get_query_object(SU_CONNS);
  //set retrieval condition & criterion
  query->set_su_id(the_suid_code);
  query->set_su_id_cond(Table_Column::eq);
  query->set_dim01_dita_cond(Table_Column::any);
  //do query
  
  Iter_List<Query_Object *> *list = mystore.get_multi_object(query);
  
  //check contents of list
  
  if(! list)
    {
      cout << "no list! " << endl;
      if(persistent_error == 0)
	{
	  //no matches found
	  cout << "No matches" << endl;
	}
      else
	{
	  //query failed 
	  cout << "Query failed" << endl;
	}
      delete query; query = 0;
      return false;
    }
  
  Forward_List_Iter<Query_Object *> iter(list);
    
  SU_Conns_Query *result = (SU_Conns_Query *) iter.first(); 
  
  
  char dim01_dita[4];
  
  strncpy(dim01_dita, result->get_dim01_dita(), sizeof(dim01_dita));
  
  delete query; query = 0;
  
  uint dim0_disk = dim01_dita[0] - '0';
  uint dim1_disk = dim01_dita[1] - '0';
  uint dim0_tape = dim01_dita[2] - '0';
  uint dim1_tape = dim01_dita[3] - '0';
  
  //Error conditions first:
  
  if((use_mark5&&(dim0_disk==0&&dim1_disk==0))||(!use_mark5&&(dim0_tape==0&&dim1_tape==0)))
	 {
		cout << "Error: no disk/tape connected to DIMs" << endl;
		return false;
	 }
  if(use_mark5&&dim0_disk!=vex_head&&dim1_disk!=vex_head&&vex_head!=3)
	 {
		cout << "Error: requested headstack (disk) not connected to either DIM" << endl;
		return false;
	 }
  if(!use_mark5&&dim0_tape!=0&&dim1_tape!=0)
	 {
		cout << "Error: two headstacks (tape) connected. Physically impossible" << endl;
		return false;
	 }
  if(use_mark5&&((dim0_disk==0||dim1_disk==0)&&vex_head==3))
	 {
		cout << "Error: 2 headstacks requested (disk); only one or none connected" << endl;
		return false;
	 }
  if((use_mark5&&(dim0_disk==dim1_disk))||(!use_mark5&&((dim0_tape==1||dim0_tape==2)&&(dim1_tape==1||dim1_tape==2))))
	 {
		cout << "Error: single disk or tape output connected to both DIMs" << endl;
		return false;
	 }
  if((dim0_disk!=0&&dim0_tape!=0)||(dim1_disk!=0&&dim1_tape!=0))
    {
      cout << "Error: disk AND tape output connected to same DIM?" << endl;
      return false;
    }  
  
  
  cout << "Init_Proc::do_config_arrays() - INFO" << endl;
  cout << "--use_mark5 = " << use_mark5 << endl;
  cout << "--dim0_disk,dim1_disk = " << dim0_disk << "," << dim1_disk << endl;
  cout << "--dim0_tape,dim1_tape = " << dim0_tape << "," << dim1_tape << endl;
  cout << "--vex_head = " << vex_head << endl;
  if((use_mark5&&((dim0_disk==1&&dim1_disk==0)||(dim0_disk==0&&dim1_disk==2)||(dim0_disk==1&&dim1_disk==2)))
	  ||(!use_mark5&&((dim0_tape==1||dim0_tape==2)&&dim1_tape==0&&vex_head==1))
	  ||(!use_mark5&&((dim1_tape==1||dim1_tape==2)&&dim0_tape==0&&vex_head==2))) 
	 swap_head = false;
  else 
	 swap_head = true;


  cout << "swap_head = " << swap_head << endl;
  
  
  int iswap;
  
  
  
  //construct logical_channel/track link table - also stores if a particular channel is part
  //of the current scan, which tracks are sign or magnitude and upper or lower sideband
  
  
  for(i = 0; i < 70; ++i) //for each track 0 to 69
    {
      if (swap_head)
	{
	  if (i < 36) iswap = i + 36;
	  else iswap = i - 36; 
	}
      else iswap = i;
      
      Channel channels[4];
      if(scan->get_channel_track_link(i, channels))
        {	 
	  chans_ptr = channels;
	  
	  //look in each channel object for the chan_ref
	  for(j = 0; j < num_chans; ++j)
	    {
	      string channel_ref = chans_ptr->get_chan_ref();
	      //find logical_channel_nrs
	      num_chans_ref = current_station->get_logical_channel_nrs(channel_ref, time_slot, logical_channel_nrs);
	      if(num_chans_ref == -1)
		{
		  //time_slot is incorrect
		  cerr << "Error in init_proc::do_arrays. time_slot incorrect\n";
		  return false;
		}
	      else
		{
		  //look in logical_channel_nrs for elements != -1
		  for(k = 0; k < NUM_CHANNELS; ++k)
		    {
		      if(logical_channel_nrs[k] != -1)
			{
			  log_chan_track_link[logical_channel_nrs[k]][72][0] = active;
			  log_chan_track_link[logical_channel_nrs[k]][72][1] = active;
			  //put ref into appropriate place in log_chan_ref_table
			  log_chan_ref_table[ logical_channel_nrs[k] ] = channel_ref;
			  //assign the appropriate sign/magnitude info in the array
			  chans_ptr->get_sign_magnitude() == sign ? 
			    log_chan_track_link[logical_channel_nrs[k]][iswap][0] = sine : 
			    log_chan_track_link[logical_channel_nrs[k]][iswap][0] = mag;
			  //also need to acquire the USB/LSB bit
			  Channel_Reference* chr_tmp = cjd->get_channel(channel_ref);
			  if (chr_tmp)
			    {
			      Sideband sb_tmp = chr_tmp->get_sideband();
			      if (sb_tmp == lower)
				{
				  log_chan_track_link[logical_channel_nrs[k]][iswap][1] = lsb;
				}
			      else
				{
				  log_chan_track_link[logical_channel_nrs[k]][iswap][1] = usb;
				}
			    }
			}
		    }
		}
	      ++chans_ptr;
	    }
        }
    }
  
  //start filling in 0 phase of tables
  
  for(i = 0; i < 16; ++i)
    {
      //is ith logical_channel part of this scan?
      if(log_chan_track_link[i][72][0] == active)
        {
	  //next question : muxmode
	  
	  if(muxmode >= 1) //ie 1, 2, or 4 tracks per channel
	    {
	      if(!this->do_mux_table(i, scan, xbc_l_index, xbc_u_index, nr_tracks))
		{
		  //Error!
		  cout << "do_mux_table not called?" << endl;
		  return false;
		}
	    }
	  
	  if(muxmode < 1) //ie 2, or 4 channels per track
	    {
	      //Not sure if following actually works !!!
	      if(!this->do_q_1_mux_table(i, scan, xbc_l_index, xbc_u_index, nr_tracks))
		{
		  //Error!
		  return false;
		}
	    }
        }//end if(log_chan_track_link[i][72][0] == active)
    }//end for(i = 0; i < 16; ) for each logical channel
  
  for(i = 0; i < 16; ++i)
    {
      //now that the logical to physical channel mapping has been defined 
      //do the_sidebands for the load polys aux data
      
      //is logical_channel i part of this scan?
      if (log_chan_track_link[i][72][0] == active)
        {
	  //if so is it lower sideband
	  for(j = 0; j < 70; ++j)
	    {
	      if (log_chan_track_link[i][j][1] == lsb)
		{
		  the_sidebands |= 1<<output_mapping[i];
		  break;//out of inner for
		}
	    }
        } 
    }
  
  // If an external crm shuffle is in place, we need to also shuffle
  // the sideband bits
  if (external_crm_shuffle) {
    unsigned short int new_sidebands = 0;
    for (i=0; i<16; i++) {
      if (crm_shuffle[i]!=-1 && (the_sidebands & 1<<crm_shuffle[i]))
	new_sidebands |= 1<<i;
    }
    the_sidebands = new_sidebands;
  }
  
  //fill in the 0 phase of the crm x_bar_config table
  
  if(!this->do_the_cross_bar_config_table())
    {
      //Error!
      return false;
    }
  
  //now barrel rolling must be taken into account - first determine if barrel rolling is active
  //by looking at phases (if > 1 then barrel rolling active)
  
  if(phases > 1)
    {
      //barrel roll
      char table[NUM_CHANNELS][NUM_TRACKS];//barrel roll pattern
      char tr0;//representing the physical track # at 0 phase
      char tri;//representing the physical track # at i phase
      unsigned int the_old_track_bits, the_new_track_bits;
      
      scan->get_barrel_rolling()->get_cross_bar_config(table);
      
      //deal with crm x_bar_config table the_cross_bar_config[NUM_CHANNELS][NUM_TRACKS]
      //the_cross_bar_config[0][j] contains a (char) dim_xbc_index or -1
      //dim_x_bar_configuration[dim_xbc_index] contains the corresponding physical (char) track #
      
      //to barrel roll the_cross_bar_config[i][j] , for each j, if(the_cross_bar_config[0][j] != -1),
      //look in  dim_x_bar_configuration[(int) the_cross_bar_config[0][j]] for the 0 phase track #.
      
      //look in table[0][k] for the 0 phase track #, then
      //look in table[i][k] for the i phase track #
      //next look in dim_x_bar_configuration[y] for the i phase track #, and enter y into
      //the_cross_bar_config[i][j] ... argh!!!!!!!!!!!!
      
      
      int dim_xbc_index;
      
      for(i = 1; i < phases; ++i)
        {
	  for(j = 0; j < NUM_TRACKS; ++j)
	    {
	      dim_xbc_index = (int)the_cross_bar_config[0][j];
	      
	      tr0 = dim_x_bar_configuration[dim_xbc_index];
	      if(tr0 == 80) continue;//track not active
	      
	      for(k = 0; k < NUM_TRACKS; ++k)
		{
		  if(table[0][k] == tr0) break;//out of for loop
		}
	      
	      if(table[0][k] != tr0 || ((int)table[0][k]) == -1)
		{
		  //got a problem 
		  cerr << "\nProblem in barrel rolling in do_config_arrays!\n";
		  return false;
		}
	      
	      tri = table[i][k]; 
	      if((int)tri == -1)
		{
		  //shouldnt happen
		  cerr << "\nProblem in barrel rolling in do_config_arrays!\n";
		  return false;
		}
	      
	      for(k = 0; k < NUM_TRACKS; ++k)
		{
		  if(dim_x_bar_configuration[k] == tri) break;
		}
	      
	      if(dim_x_bar_configuration[k] == tri)
		{
		  the_cross_bar_config[i][j] = k;
		}
	      else
		{
		  //there is a problem
		  cerr << "Can't find tri in do_config_arrays!\n";
		  return false;
		}
	    }
	}
      ///////////////////////////////////////////////////////////////////////////////////////
      //to barallel roll the_aux_data[i][j][*], look in dim_x_bar_configuration[j] for a char
      //tr0 (or 80). if 80 continue. 
      
      //Next look in table[0][k] for tr0.  
      //Look in dim_x_bar_configuration[k] for tri (index k)
      //the basic aux_data to go into the_aux_data[i][j][*] is the_aux_data[0][k][*]
      //corrected for physical track#
      
      /////////////////////////////////////////////////////////////////////////////////////////
      
      //now deal with the_aux_data[NUM_CHANNELS][NUM_TRACKS][2] - which is even more complex in some ways!
      //the track no's will vary upon barrel rolling so requiring updating of that value
      //which is different for each recording mode
      
      //to barrel roll the aux_data[i][j][*], look in the_cross_bar_config[i][j] for tr0.
      //tr0 is the index whereby aux_data[i][j][*] = aux_data[0][tr0][*]
      //However, will need to correct the value of aux_data[0][k][1] for physical track #
      
      //the "new" track number tri is dim_x_bar_configuration[j] - if 80 track inactive
      //the "old" track number tr0 can be found by looking in the_cross_bar_config[0][j] for index l, 
      //and looking in dim_x_bar_configuration[l] for tr0
      
      
      
      
      for(i = 1; i < phases; ++i)
	{
	  for(j = 0; j < NUM_TRACKS; ++j)
	    {
	      tri = dim_x_bar_configuration[j];
	      
	      if(tri == 80) continue;//track not active
	      
	      tr0 = dim_x_bar_configuration[(int)the_cross_bar_config[0][j]];
	      
	      the_aux_data[i][j][0] = the_aux_data[0][(int)tr0][0];
	      
	      if(the_tape_format == MkIV)//mk IV
		{
		  //now change the bit pattern
		  the_bit_pattern = the_aux_data[0][(int)tr0][1];
		  
		  the_bit_pattern &= ~((this->get_bcd_rep((int)tr0)<<24));
		  the_bit_pattern |= (this->get_bcd_rep((int)tri))<<24;
		  
		  //new_bit_pattern = old_bit_pattern - the_old_track_number + the_new_track_number
		  
		  the_aux_data[i][j][1] = the_bit_pattern;
		}
	      
	      
	      if(the_tape_format == VLB)//VLBA
		{	       
		  the_new_track_bits = this->vlba_group_track_rep((int)tri);
		  the_old_track_bits = this->vlba_group_track_rep((int)tr0);
		  
		  //as the above are full representations we can use normal addition and subtraction
		  the_bit_pattern = the_aux_data[0][(int)tr0][1] - the_old_track_bits + the_new_track_bits;
		  
		  the_aux_data[i][j][1] = the_bit_pattern; 
		}
	      
	      if(the_tape_format == MkIIIA)//Mk III
		{
		  //The MK IV memo 230 states the aux data fields are common across all tracks, 
		  //and it cant barrel roll so the contents of the_aux_data[NUM_CHANNELS][NUM_TRACKS][*] must be the same 
		  //for * == 0 and * ==1.
		  
		  the_aux_data[i][j][1] = the_aux_data[0][(int)tr0][1];
		}//end if Mk III
	      
	    }//end for each track == j
	}//end for phase ==i
      
      //what happens if phases < 16 ? - 
      if(phases < NUM_CHANNELS)
	{
	  for(i = phases; i < NUM_CHANNELS; ++i)
	    {
	      for(j = 0; j < NUM_TRACKS; ++j)
		{
		  the_cross_bar_config[i][j] = the_cross_bar_config[i - phases][j];
		  the_aux_data[i][j][0] = the_aux_data[i - phases][j][0];
		  the_aux_data[i][j][1] = the_aux_data[i - phases][j][1];
		}
	    }
	}      
    }//end if barrel rolling
  else
    {
      //barrel rolling is not active
      
      for(i = 1; i < NUM_CHANNELS; ++i)
	{
	  for(j = 0; j < NUM_TRACKS; ++j)
	    {
	      the_cross_bar_config[i][j] = the_cross_bar_config[0][j];
	      the_aux_data[i][j][0] = the_aux_data[0][j][0];
	      the_aux_data[i][j][1] = the_aux_data[0][j][1];
	    }
	}  
    }
  
  return true;
}

//______________________________________________________________________________

void Init_Proc::do_aux_data(Scan *scan)
{
  unsigned int the_bit_pattern = 0;
  int recorder_ID = current_medium->get_recorder()->get_recorder_ID();
  int i, j;
  
  if(the_tape_format == MkIIIA)//Mk III
    {
      //the aux data field can be written as 16 hex characters - vvwxwxyzyzFFaabb
      //where vv indicates fwd or reverse passes,wxyz indicates positions of headstacks, 
      //aa indicatess status and error flags, and bb the hardware ID
      //get tape direction from Scan
      if(scan->get_tape_direction() == forward)
	{
	  //vv == FF
	  the_bit_pattern |= 0xff000000;
	}
      else
	{
	  //vv = FE
	  the_bit_pattern |= 0xfe000000;
	}
      
      //now the head offsets - only 1 used for writing
      
      //get the head offsets
      int head_offset = (int)(current_medium->get_recorder()->get_head_offset(0));//micron units
      //an int is big enough
      
      unsigned int wxyz_bits;
      
      if(head_offset < 0)
	{
	  wxyz_bits = this->get_bcd_rep(abs(head_offset) + 4000);
	}
      else
	{
	  wxyz_bits = this->get_bcd_rep(head_offset);
	}
      
      the_bit_pattern |= (wxyz_bits>>8)<<16;//= vvwx
      the_bit_pattern |= (wxyz_bits>>8)<<8;//= vvwxwx
      the_bit_pattern |= (wxyz_bits&0xf);//= vvwxwxyz
      
      for(i = 0; i < NUM_CHANNELS; ++i)
	{
	  for (j = 0; j <NUM_TRACKS; ++j)
	    {
	      the_aux_data[i][j][0] = the_bit_pattern;//vvwxwxyz
	    }
	}       
      
      //get hardwareID bits - ie bb and add to FF****
      the_bit_pattern = 0;//reinitialised
      the_bit_pattern |= (wxyz_bits&0xf)<<24;//=yz000000
      the_bit_pattern |= 0xff0000;//=yzFF0000
      the_bit_pattern |= recorder_ID;//=yzFF00bb
      
      
      for(i = 0; i < NUM_CHANNELS; ++i)
	{
	  for (j = 0; j <NUM_TRACKS; ++j)
	    {
	      the_aux_data[i][j][1] = the_bit_pattern;//yzFFaabb
	    }
	}
      the_bit_pattern = 0;//reinitialised
    }//end if Mk III
  
  if(the_tape_format == MkIV)//mk IV
    {
      the_bit_pattern = 0;//reinitialised
      
      //the aux data comprises 16 hex characters (64 bits)  of the form aaaabbbbrrssttuu where :
      //aaaa = nominal headstack 1 position in microns 0000 - 3999 + micron positions (bcd encoded)
      //					       4000 - 7999 - micron positions (bcd encoded) which
      //represent positions of 0 to -3999 microns respectively.
      //bbbb = nominal headstack 2 position in microns (encoded as above)
      //rr bits 7-6 = headstack number (0 - 3), bits 5-0 track number
      //ss = Data_ID - bit 5 = sign (0) or mag (1) bit; bit 4 = Uppersideband(USB = 0) or lowersideband(LSB =1)
      //tt = formatter status
      //uu systemID 
      
      //get the head offsets
      int head0_offset = (int)(current_medium->get_recorder()->get_head_offset(0));//micron units
      //an int is big enough
      
      if(head0_offset < 0)
	{
	  the_bit_pattern |= this->get_bcd_rep(abs(head0_offset) + 4000)<<16;
	}
      else
	{
	  the_bit_pattern |= this->get_bcd_rep(head0_offset)<<16;
	}
      
      int head1_offset = (int)(current_medium->get_recorder()->get_head_offset(1));//micron units
      
      if(head1_offset < 0)
	{
	  the_bit_pattern |= this->get_bcd_rep(abs(head1_offset) + 4000);
	}
      else
	{
	  the_bit_pattern |= this->get_bcd_rep(head1_offset);
	}
      
      for(i = 0; i < NUM_CHANNELS; ++i)
	{
	  for (j = 0; j <NUM_TRACKS; ++j)
	    {
	      the_aux_data[i][j][0] = the_bit_pattern;
	    }
	}
      the_bit_pattern = 0;//reinitialised
    }//end if Mk IV
  
  if(the_tape_format == VLB)//VLBA
    {
      
      //The aux data comprises eight bytes (64 bits) organised as 16 BCD digits SSRHPPPP TGBSCD00
      //with TGBSCD bit reversed - where SS = station code, R = recorder No, H = headstack No,
      //P = position sign (0 = + , 8 = -), PPP = position (100's, 10's, 1's), T = track (0 - 7),
      //G = group (0 - 4), B = bit (0 : mag, 1 : sign), S = sideband (0 : lower, 1 : upper),
      //C = converter (0 - 7), D = DAR No 0 - 1 (currently only single DAR), 0, 0.
      
      unsigned int bit_pattern1 = 0; 
      unsigned int bit_pattern2 = 0; 
      
      //get the recorder No 
      bit_pattern1 |= (this->get_bcd_rep(recorder_ID))<<20;
      bit_pattern2 |= (this->get_bcd_rep(recorder_ID))<<20;
      
      //get the recorder headstack position      
      //vlba uses only a single headstack designated 0 
      
      int head_offset1 = (int)(current_medium->get_recorder()->get_head_offset(0));//micron units
      //first determine its sign bit
      if(head_offset1 < 0)
	{
	  bit_pattern1 |= 0x8<<12;
	}
      
      //convert to absolute value of head_ofset
      head_offset1 = abs(head_offset1);
      
      int head_offset2 = (int)(current_medium->get_recorder()->get_head_offset(1));//micron units
      //first determine its sign bit
      if(head_offset2 < 0)
	{
	  bit_pattern2 |= 0x8<<12;
	}
      
      //convert to absolute value of head_ofset
      head_offset2 = abs(head_offset2);
      
      //next position bits : extends over 12 bits 0 -> 11
      bit_pattern2 |= (this->get_bcd_rep(head_offset2));
      
      for(i = 0; i < NUM_CHANNELS; ++i)
	{
	  for (j = 0; j < 36; ++j)
	    {
	      the_aux_data[i][j][0] = bit_pattern1;//SSRHPPPP
	    }
	  for (j = 36; j <NUM_TRACKS; ++j)
	    {
	      the_aux_data[i][j][0] = bit_pattern2;//SSRHPPPP
	    }
	}//end for
      bit_pattern1 = 0;//reinitialised
      bit_pattern2 = 0;//reinitialised
    }//end if vlba
}

//___________________________________________________________________________________

bool Init_Proc::do_mux_table(const int chan_nr,  Scan *scan, int &xbc_l_index, int &xbc_u_index, int &nr_tracks)
{
   //for 1 bit mode, look in log_chan_track_link[chan_nr][#][0] == sine for #
   //query current_scan scan->get_tracks(#, int[4]) for sine tracks (because of ordering )
   //if log_chan_track_link[chan_nr][int[*]][0] == sine, assign int[*] to the lowest available TRM I/P.
   //and assign this number to mux_table[chan_nr][* + 1].
   
   //if 2 bit the procedure is repeated for mag, except that chan_nr = chan_nr + 1, 
   //and (mux_table[chan_nr] [*+5]) is required
   
   //note also if(muxmode > 3 && num_bits > 1) then only alternate rows in the mux_table may be used


   int i, j, k, tracks[4];
   int tracks_per_channel = (int)muxmode;
   unsigned int the_bit_pattern = 0;
   
   //1 bit mode == sine
   lc_trk_lnk bit = sine;
   
   //if(muxmode > 3) //ie 4 tracks per channel
   unsigned int index;
   if(muxmode > 3 && num_bits > 1) index = chan_nr * 2;
   else index = chan_nr;
   
   if(index > 15)
   {
      cout << "SU_Control Error : index too large! Too many logical Channels." << endl;
      return false;
   }
   

   cout << "Init_Proc::do_mux_table() - INFO" << endl;
   cout << "---swap_head = " << swap_head << endl;
   cout << "---tracks_per_channel = " << tracks_per_channel << endl;
   cout << "---index = " << index << endl;
   for(i = 0; i < num_bits; ++i)
     {
       for(j = 0; j < 70; ++j)
	 {
	   if(log_chan_track_link[chan_nr][j][0] == bit)
	     {
	       if (swap_head)
		 {
		   if (j < 36) scan->get_tracks(j+36, tracks);
		   else scan->get_tracks(j-36, tracks);
		 }
	       else
		 scan->get_tracks(j, tracks);
	       
	       for(k = 0; k < tracks_per_channel; ++k)
		 {
		   cout << "---i,j,k = " << i << "," << j << "," << k << endl;
		   if (swap_head)
		     {			  
		       if (tracks[k] < 36) tracks[k] += 36;
		       else tracks[k] -= 36;
		     }
		   cout << "---tracks[k] = " << tracks[k] << endl;
		   if(log_chan_track_link[chan_nr][tracks[k]][0] == bit)
		     {
		       //chan_index effectively defines the logical to physical channel mapping
		       mux_table[index][0] = index;
		       output_mapping[chan_nr] = index;
		       
		       if(tracks[k] < 36)
			 {
			   // headstack 1 in VEX file; tracks now go into DIM 0 (lower)
			   the_bit_pattern = 0;//reinitalise
			   dim_x_bar_configuration[xbc_l_index] = tracks[k];
			   cout << "---vex hs1->dim 0; dim_x_bar_configuration[" << xbc_l_index << "] = " << tracks[k] << endl;
			   the_active_tracks[xbc_l_index] = 1;
			   if(bit == sine) mux_table[index][1 + k] = xbc_l_index;
			   else mux_table[index][5 + k] = xbc_l_index;
			   ++nr_tracks;
			   ++xbc_l_index;
			 }
		       else
			 {
			   // headstack 2 in VEX file; tracks now go into DIM 1 (uppper)
			   the_bit_pattern = 0;//reinitalise
			   if(the_tape_format == MkIV)//only MK IV uses more than one headstack at the moment
			     {
			       //first the headstack No = 1
			       the_bit_pattern |= 1<<30;
			     }
			   else
			     {
			       //first the headstack No = 1
			       the_bit_pattern |= 1<<16;
			     }
			   dim_x_bar_configuration[xbc_u_index] = tracks[k];
			   cout << "---vex hs2->dim 1; dim_x_bar_configuration[" << xbc_u_index << "] = " << tracks[k] << endl;
			   the_active_tracks[xbc_u_index] = 1;
			   if(bit == sine) mux_table[index][1 + k] = xbc_u_index;
			   else mux_table[index][5 + k] = xbc_u_index;
			   ++nr_tracks;
			   ++xbc_u_index;
			 }
		       
		       if(the_tape_format == MkIV)//if MK IV then
			 {
			   //next the track No
			   the_bit_pattern |= (this->get_bcd_rep(tracks[k]))<<24;
			   //next the sign/mag bit
			   if(bit == mag) the_bit_pattern |= 1<<21;
			   
			   //also need to acquire the USB/LSB bit
			   if(log_chan_track_link[chan_nr][tracks[k]][1] == lsb) the_bit_pattern |= 1<<20;
			   
			   if(tracks[k] < 36)
			     {
			       the_aux_data[0][xbc_l_index - 1][1] = the_bit_pattern;//index has already been incremented
			     }
			   else
			     {
			       the_aux_data[0][xbc_u_index - 1][1] = the_bit_pattern;//index has already been incremented
			     }
			   the_bit_pattern = 0;//reinitialised
			 }//end Mk IV
		       
		       if(the_tape_format == VLB)//VLBA
			 {
			   //get track/groups
			   the_bit_pattern = this->vlba_group_track_rep(tracks[k]);
			   //next the sign/mag bit
			   if(bit == sine) the_bit_pattern |= 1<<23;
			   
			   //also need to acquire the USB/LSB bit
			   if(log_chan_track_link[chan_nr][tracks[k]][1] == usb) the_bit_pattern |= 1<<19;
			   
			   if(tracks[k] < 36)
			     {
			       the_aux_data[0][xbc_l_index - 1][1] = the_bit_pattern;//index has already been incremented
			     }
			   else
			     {
			       the_aux_data[0][xbc_u_index - 1][1] = the_bit_pattern;//index has already been incremented
			     }
			   the_bit_pattern = 0;//reinitialised
			 }//end if vlba
		     }//end of inner if(log_chan_track_link[chan_nr][tracks[k]][0] == bit)
		   else
		     {
		       cout << "problem in do_mux_table" << endl;
		       return false;
		     }
		 }//end of for tracks_per_channel
	       bit = mag;
	       break;//out of for j
	     }//end if(log_chan_track_link[chan_nr][j][0] == bit)
	 }//end of for j
     }//end for num_bits ie sine dealt with first
   
   
   return true;
}

//___________________________________________________________________________

bool Init_Proc::do_q_1_mux_table(const int chan_nr,  Scan *scan, int &xbc_l_index, int &xbc_u_index, 
				 int &nr_tracks)
{
  //if muxmode == 0.25 && num_bits == 1
  //get the physical track# (DIM O/P #) log_chan_track_link[chan_nr][track#][0] == sine
  //query scan->get_channel_track_link(track#, channels) for the four channel objects.
  //For each of these obtain its channel_ref, scan the log_chan_ref_table for the same value,
  //get its index and go to log_chan_track_link[index][track#][0] which should == sine.
  //if so go to log_chan_track_link[index][72][0] and change value to unknown, and assign this track# to
  //the lowest available TRM I/P. Assign this number to mux_table[index][1]
  
  //if num_bits == 2
  //get the physical track# (DIM O/P #) log_chan_track_link[chan_nr][track#][0] == sine
  //query scan->get_channel_track_link(track#, channels) for the four channel objects.
  //For each of these obtain its channel_ref, scan the log_chan_ref_table for the same value,
  //get its index and go to log_chan_track_link[index][track#][0] which should == sine.
  //if(track# < 36) assign this track# to the xbc_l_index TRM I/P and Assign xbc_l_index 
  //to mux_table[index][1] (&& mux_table[index][0] = index)
  //else assign this track# to the xbc_u_index TRM I/P and Assign xbc_u_index 
  //to mux_table[index][1] 
  
  //next get the physical track# (DIM O/P #) log_chan_track_link[chan_nr][track#][0] == mag
  //query scan->get_channel_track_link(track#, channels) for the four channel objects.
  //For each of these obtain its channel_ref, scan the log_chan_ref_table for the same value,
  //get its index and go to log_chan_track_link[index][track#][0] which should == mag.
  //if so go to log_chan_track_link[index][72][0] and change value to unknown
  //if(track# < 36) assign this track# to the xbc_l_index TRM I/P and Assign xbc_l_index 
  //to mux_table[index][1] (&& mux_table[index][0] = index)
  //else assign this track# to the xbc_u_index TRM I/P and Assign xbc_u_index 
  //to mux_table[index][5] (&& mux_table[index][0] = index)
  
  //if muxmode == 0.5 the procedure is the same except there will be only 2 channel objects 
  
  unsigned int the_bit_pattern = 0;
  
  //1 bit mode == sine
  lc_trk_lnk bit = sine;
  int mux_index = 1;
  
  int channels_index;
  if(muxmode < 0.5)
    {
      channels_index = 4;
    }
  else
    {
      channels_index = 2;
    }
  
  for (int i=0; i<num_bits; ++i)
    {
      for (int j=0; j<70; ++j)
	{
	  if (log_chan_track_link[chan_nr][j][0] == bit)
	    {
	      Channel channels[4];
	      scan->get_channel_track_link(j, channels);
	      
	      for (int k=0; k<channels_index; ++k)
		{
		  string channel_ref = channels[k].get_chan_ref();
		  //scan the log_chan_ref_table for the same value
		  int l;
		  for (l=0; l<16; ++l)
		    {
		      if( log_chan_ref_table[l]==channel_ref ) break;
		    }
		  if( log_chan_ref_table[l]==channel_ref )
		    {
		      if (log_chan_track_link[l][j][0] == bit)
			{
			  //if(j < 36) assign j to the xbc_l_index TRM I/P and Assign xbc_l_index 
			  //to mux_table[index][1] (&& mux_table[index][0] = index)
			  if (j < 36)
			    {
			      dim_x_bar_configuration[xbc_l_index] = j;
			      mux_table[l][mux_index] = xbc_l_index;
			      the_active_tracks[xbc_l_index] = 1;
			      ++nr_tracks;
			      ++xbc_l_index;
			    }
			  else
			    {
			      dim_x_bar_configuration[xbc_u_index] = j;
			      mux_table[l][mux_index] = xbc_u_index;
			      the_active_tracks[xbc_u_index] = 1;
			      ++nr_tracks;
			      ++xbc_u_index;
			    }
			  if (num_bits < 2 || bit == mag)
			    {
			      mux_table[l][0] = l;
			      log_chan_track_link[l][72][0] = unknown;
			      //l effectively defines the logical to physical channel mapping
			      output_mapping[l] = l;
			    }
			  
			  if (the_tape_format == MkIV)//if MK IV then
			    {
			      //first the headstack No 
			      if (j > 35)
				{
				  the_bit_pattern |= 1<<30;
				  //next the track No
				  the_bit_pattern |= (this->get_bcd_rep(j - 36))<<24;
				}
			      else
				{
				  //next the track No only
				  the_bit_pattern |= (this->get_bcd_rep(j))<<24;
				}
			      //next the sign/mag bit
			      if (bit == mag) the_bit_pattern |= 1<<21;
			      //also need to acquire the USB/LSB bit
			      if (log_chan_track_link[l][j][1] == lsb) the_bit_pattern |= 1<<20;
			      
			      if (j < 36) the_aux_data[0][xbc_l_index - 1][1] = the_bit_pattern;
			      else the_aux_data[0][xbc_u_index - 1][1] = the_bit_pattern;
			      the_bit_pattern = 0;//reinitialised
			    }//end Mk IV
			  
			  if (the_tape_format == VLB)//VLBA
			    {
			      //the headstack No = 0
			      //get track/groups
			      the_bit_pattern = this->vlba_group_track_rep(j);
			      //next the sign/mag bit
			      if (bit == sine) the_bit_pattern |= 1<<23;
			      //also need to acquire the USB/LSB bit
			      if (log_chan_track_link[l][j][1] == usb) the_bit_pattern |= 1<<19;
			      
			      if (j < 36) the_aux_data[0][xbc_l_index - 1][1] = the_bit_pattern;
			      else the_aux_data[0][xbc_u_index - 1][1] = the_bit_pattern;
			      the_bit_pattern = 0;//reinitialised
			    }//end if vlba
			}
		      else
			{
			  cout << "SU_Control ERROR : problem in log_chan_track_link table" << endl;
			  return false;
			}
		    }
		  else
		    {
		      cout << "SU_Control ERROR : problem in log_chan_ref_table" << endl;
		      return false;
		    }
		}
	      break;//out of inner j for loop
	    }
	}
      bit = mag;
      mux_index = 5;
    }
  return true;
}

//_______________________________________________________________

bool Init_Proc::do_the_cross_bar_config_table(void)
{
   
   //modified from BA's code crmxbar.c
/*
 * generates configuration data for the SU CRM from high(er)-level
 * description of relationship of channels to tracks
 *
 * Reference: P+G Module Software Addresses for the SU, Appendix A
 *
 * This whole thing is based on 5 tables. An inspection of the
 * Appendices shows the 5 tables which can be boiled down to 1 table and
 * some rules about how to use the table.
 *------------------------------------------------
 * Table 1.  XBAR Assignments  (see A1)
 *
 *  channel              xbar outputs for
 *  number           sign              magnitude
 *    00        00  01  02  03      04  05  06  07
 *    01        06  07  04  05      02  03  00  01
 *    02        08  09  10  11      12  13  14  15
 *    03        14  15  12  13      10  11  08  09
 *    04        16  17  18  19      20  21  22  23
 *    05        22  23  20  21      18  19  16  17
 *    06        24  25  26  27      28  29  30  31
 *    07        30  31  28  29      26  27  24  25
 *    08        32  33  34  35      36  37  38  39
 *    09        38  39  36  37      34  35  32  33
 *    10        40  41  42  43      44  45  46  47
 *    11        46  46  44  45      42  43  40  41
 *    12        48  49  50  51      52  53  54  55
 *    13        54  55  52  53      50  51  48  49
 *    14        56  57  58  59      60  61  62  63
 *    15        62  63  60  61      58  59  56  57
 *-------------------------------------------------------------------
 * Rules:
 *
 *  1. Start with a xbar-state aray containing 64 zeros.
 *  2. Ignore magnitude bits in Table 1 for 1-bit data.
 *  3. For channel C and the nth track for that channel, look up the
 *     xbar output in Table 1 and write the actual track number into
 *     the xbar-state array in that position.
 *  4. Do (3) first for for signs and then, if appropriate, magnitudes.
 *  5. MUX = 4:1 Cannot have 16 channels in 2-bit mode: not enough tracks
 *               Can use a mix of odd and even channels provided only
 *               odd or even is used in [0,1], [2,3], etc. pairs.
 *  6. MUX 2:1 and 1:1 have no other restrictions
 *  7. MUX 1:2 and 1:4 Each Channel should be specified in the input (mux_table)
 *			See Assumptions below
 *--------------------------------------------------------------------
 * Assumptions: the high-level info comes in a table of the form
 *
 *              channel#  tr#  tr#  tr#  tr# ..    (up to 8 tr#)
 *              channel#  tr#  tr#  tr#  tr# ..    (up to 8 tr#)
 *              etc.
 *
 *              In 2-bit mode, sign tracks first then magnitude
 *
 *              For MUX 1:n, a track is repeated for more than
 *              one channel. Such channels must be contiguous, start
 *              with an even channel (1:2) or a doubly even channel (1:4)
 *              and tracks should appear in the order in which time
 *              multiplexing occurs.
 *
 *              The track order on a line MUST match the MUX
 *              mode. "Surplus" tracks on a line are ignored.
 *              Otherwise missing tracks default to zero.
 *-------------------------------------------------------------------
 */

  int numt, chan, tr, mux_table_value;
    
  if(muxmode > 3) numt = 4;
  else if(muxmode > 1.5) numt = 2;
  else numt = 1;   

  int  table1[16][8] = {{  0,   1,   2,   3,   4,   5,   6,   7},
			{  6,   7,   4,   5,   2,   3,   0,   1},
			{  8,   9,  10,  11,  12,  13,  14,  15},
			{ 14,  15,  12,  13,  10,  11,   8,   9},
			{ 16,  17,  18,  19,  20,  21,  22,  23},
			{ 22,  23,  20,  21,  18,  19,  16,  17},
			{ 24,  25,  26,  27,  28,  29,  30,  31},
			{ 30,  31,  28,  29,  26,  27,  24,  25},
			{ 32,  33,  34,  35,  36,  37,  38,  39},
			{ 38,  39,  36,  37,  34,  35,  32,  33},
			{ 40,  41,  42,  43,  44,  45,  46,  47},
			{ 46,  47,  44,  45,  42,  43,  40,  41},
			{ 48,  49,  50,  51,  52,  53,  54,  55},
			{ 54,  55,  52,  53,  50,  51,  48,  49},
			{ 56,  57,  58,  59,  60,  61,  62,  63},
			{ 62,  63,  60,  61,  58,  59,  56,  57}};

  if (!external_crm_shuffle) {
    for (chan = 0; chan < 16; ++chan) { //channel loop 
      if(mux_table[chan][0] == chan) { //only do channels used 
	for (tr = 0; tr < numt; ++tr) { // track loop, sign 
	  mux_table_value = mux_table[chan][tr + 1];

	  if(mux_table_value == -1) {
	    cout << "SU_Control ERROR : problem in " 
		 << "do_the_cross_bar_config_table()" << endl;
	    return false;
	  }

	  the_cross_bar_config[0][table1[chan][tr]] = mux_table_value;

	  if(num_bits > 1) { // have to do magnitude 
	    mux_table_value = mux_table[chan][tr + 5];
	    if(mux_table_value == -1) {
	      cout << "SU_Control ERROR : problem in " 
		   << "do_the_cross_bar_config_table()" << endl;
	      return false;
	    }
	    the_cross_bar_config[0][table1[chan][tr + 4]] = mux_table_value;
	  }
	    //} else {
	    ////just copy the sign bit into the equivalent mag bit
	    //the_cross_bar_config[0][table1[chan][tr + 4]] = mux_table[chan][tr + 1];
	    //}
	}
      }
    }
  } else { // override and probably duplicate the crm settings
    int realchan;
    //cout << "Shuffling CRM:" << endl;
    for (chan = 0; chan < 16; ++chan) {//channel loop 
      
      realchan = crm_shuffle[chan];
      //cout << "  " << chan << " -> " << realchan << endl;
      if (realchan==-1) continue; // Skip this channel

      if (mux_table[realchan][0] == realchan) { //only do channels used 
	for (tr = 0; tr < numt; ++tr) { // track loop, sign 

	  mux_table_value = mux_table[realchan][tr + 1];
	  if (mux_table_value == -1) {
	    cout << "SU_Control ERROR : problem in " 
		 << "do_the_cross_bar_config_table()" << endl;
	    return false;
	  }
	  the_cross_bar_config[0][table1[chan][tr]] = mux_table_value;
	  if(num_bits > 1) { // have to do magnitude 
	    mux_table_value = mux_table[realchan][tr + 5];
	    if(mux_table_value == -1) {
	      cout << "SU_Control ERROR : problem in " 
		   << "do_the_cross_bar_config_table()" << endl;
	      return false;
	    }
	    the_cross_bar_config[0][table1[chan][tr + 4]] = mux_table_value;
	  }
	}
	//} else {
      }
    } //end for each channel
  } 

  return true;
}

//__________________________________________________________________________
// If the envirnonment variable "EXTERNAL_CONFIG" is set, override some of the
// automatic settings such as SU configuration poarameters and crossbar settings
// Also reads the "feature_bits" string from the CJD 

void Init_Proc::do_external_config() {
  char *cfg_file, *char_ptr;
  char line[200];
  FILE *su_cfg;
  int status, len, i, d;
  int found = 0;
  int failed = 0;

  // Read the feature bits string
  string feature_bits = cjd->get_feature_bits();

  //  // Not needed here, left as example
  //  int corr_head = -1;
  //  if (feature_bits.length() > 1)
  //    {
  //      // The actual head being correlated
  //      if (isdigit(feature_bits[1]))
  //	{
  //	  corr_head = feature_bits[1] - '0'; // ASCII to integer
  //	  if (corr_head > 1)
  //	    {
  //	      corr_head = -1;
  //	      cout << "Warning: Feature bit Active Head too large (" << corr_head << ")" << endl;
  //	    }
  //	}
  //      else
  //	{
  //	  cout << "Warning: Bad value (" << corr_head << ") for Active Head feature bit [1]" << endl;
  //	}
  //    }

  external_dim_xbar = false;
  external_trm_active = false;
  external_suim_pg = false;
  external_crm_shuffle = false;
  force_bits = -1;
  //header_multiplier = 1;
  
  if ((cfg_file = getenv("EXTERNAL_CONFIG"))) {

    cerr << "Using external file \"" << cfg_file<< "\" for SU config" << endl;
     
    su_cfg = fopen(cfg_file, "r");
    if (su_cfg == NULL) {

      cerr << "Cannot open external config file " << cfg_file <<
	"(ignoring)" << endl;
      perror("");
      return;
    }
      
    while (!failed && (fgets(line,200,su_cfg)!=NULL)) {
      // fgets includes a trailing \n
      len = strlen(line);
      if (line[len-1]=='\n') {
	len--;
	line[len] = '\0';     
      }

      // Remove comment characters
      char_ptr = strchr(line,'#');
      if (char_ptr != NULL) {
	*char_ptr = '\0';
	len = strlen(line); 
      }
      // No point testing blank lines
      if (len == 0) continue;

      // Now use strtok to break up the string and suck in the 
      // data we want
      char_ptr = strtok(line, " \t");
      if (char_ptr==NULL) continue; // Ignore blank lines

      // DIM settings
      if (strcasecmp(char_ptr,"DIM")==0) {
	char_ptr = strtok(NULL, " \t");
	if (char_ptr==NULL) {
	  cerr << "Error: Blank \"DIM\" line. Ignoring rest of config file" 
	       << endl;
	  failed = 1;
	  continue;
	}
	// Re-initialise the table entries
	for(i = 0; i<NUM_TRACKS; i++)
	  //old dim_x_bar_configuration[i] = 80;
	  dim_xbar_override[i] = 80;
	     
	for (i=0; i<NUM_TRACKS && char_ptr!=NULL; i++, 
	       char_ptr=strtok(NULL, " \t")) {
	  status = sscanf(char_ptr, "%d", &d);
	  if (status!=1) {
	    cerr << "Warning: Failed to match \"" << char_ptr << "\"" << endl;
	    d = 80;
	  }
	  dim_xbar_override[i] = d;
	}
	external_dim_xbar = true;
	found = 1;
      }

      // TRM active setting
      else if (strcasecmp(char_ptr,"TRM")==0) {
	char_ptr = strtok(NULL, " \t");
	if (char_ptr==NULL) {
	  cerr << "Blank \"TRM\" line. Ignoring rest of config file" 
	       << endl;
	  failed = 1;
	  continue;
	}
	
	// By default don't change the active tracks
	for(i = 0; i<NUM_TRACKS; i++)
	  trm_active_override[i] = -1;

	if (strcasecmp(char_ptr,"EVEN")==0) {         // Turn OFF even tracks
	  for(i = 0; i < NUM_TRACKS; i+=2) 
	    trm_active_override[i] = 0;
	  cout << "Turning OFF even TRM tracks" << endl;
	} else if (strcasecmp(char_ptr,"ODD")==0) {   // Turn OFF odd tracks
	  for(i = 1; i < NUM_TRACKS; i+=2) 
	    trm_active_override[i] = 0;
	  cout << "Turning OFF odd TRM tracks" << endl;
	} else if (strcasecmp(char_ptr,"FOUR0")==0) {   // Turn OFF groups of four
	  for(i = 0; i+3 < NUM_TRACKS;i+=4) {
	    trm_active_override[i++] = 0;
	    trm_active_override[i++] = 0;
	    trm_active_override[i++] = 0;
	    trm_active_override[i++] = 0;
	  }
	  cout << "Turning OFF groups of four TRM tracks (starting at 0)" 
	       << endl;
	} else if (strcasecmp(char_ptr,"FOUR1")==0) {  // Turn OFF groups of four
	  for(i = 4; i+3 < NUM_TRACKS;i+=4) {
	    trm_active_override[i++] = 0;
	    trm_active_override[i++] = 0;
	    trm_active_override[i++] = 0;
	    trm_active_override[i++] = 0;
	  }
	  cout << "Turning OFF groups of four TRM tracks (starting at 4)" 
	       << endl;
	} else {
	  // First turn off ALL tracks
	  for(i = 0; i<NUM_TRACKS; i++)
	    trm_active_override[i] = 0;
	  
	  // -1 for default, 1 to enable, -1 to disable
	  for (i=0; i<NUM_TRACKS && char_ptr!=NULL; 
	       i++, char_ptr=strtok(NULL, " \t")) {
	    status = sscanf(char_ptr, "%d", &d);
	    if (status!=1) {
	      cerr << "Warning: Failed to match \"" << char_ptr << "\"" << endl;
	      d = 0;
	    }
	    if (d>1) d = 1;
	    if (d<-1) d = -1;
	    trm_active_override[i] = d;
	  }
	  cout << "Turning OFF selected TRM tracks" << endl;
	}
	found = 1;
	external_trm_active = true;
      }

      // SUIM Pulsar gating 
      else if (strcasecmp(char_ptr,"SUIM_PG")==0) {
	cout << "Warning: Setting Pulsar gating mode" << endl;
	char_ptr = strtok(NULL, " \t");
	if (char_ptr==NULL) {
	  cerr << "Blank \"SUIM_PG\" line. Ignoring rest of config file" 
	       << endl;
	  failed = 1;
	  continue;
	}
	// Re-initialise the table entries
	for (i = 0; i<17; i++)
	  suim_pg[i] = 0;
	     
	for (i=0; i<=17 && char_ptr!=NULL; i++, // Grab the numbers one by one
	       char_ptr=strtok(NULL, " \t")) {
	  status = sscanf(char_ptr, "%x", &d);
	  if (status != 1) {
	    cerr << "Error: Failed to match \"" << char_ptr << "\"" << endl;
	    d = 0;;
	  }
	  suim_pg[i] = d;
	}
	found = 1;
	external_suim_pg = true;
      }

      // CRM override
      else if (strcasecmp(char_ptr,"CRM_OVERRIDE")==0) {
	char_ptr = strtok(NULL, " \t");
	if (char_ptr==NULL) {
	  cerr << "Error: Blank \"CRM_OVERRIDE\" line" << endl;
	  failed = 1;
	  continue;
	}
	     
	for (i=0; i<NUM_CHANNELS && char_ptr!=NULL; i++, 
	       char_ptr=strtok(NULL, " \t")) {
	  status = sscanf(char_ptr, "%d", &d);
	  if (status!=1) {
	    cerr << "Warning: Failed to match \"" << char_ptr << "\"" << endl;
	    d = -1;
	  } else if (d<-1 || d>=NUM_CHANNELS) {
	    cerr << "Warning: Channel " << d << " is out of range" << endl;
	    d = -1;
	  }
	  crm_shuffle[i] = d;
	}
	for (;i<NUM_CHANNELS; i++)  // Default is channel unused
	  crm_shuffle[i] = -1;

	external_crm_shuffle = true;
	found = 1;

	cout << "**CRM_SHUFFLE: ";
	for (i=0;i<NUM_CHANNELS; i++) 
	  cout << crm_shuffle[i] << " ";
	cout << endl;

      }

      // SUIM bitmode setting
      else if (strcasecmp(char_ptr,"FORCE_BITS")==0) {
	char_ptr = strtok(NULL, " \t");
	if (char_ptr==NULL) {
	  cerr << "Blank \"FORCE_BITS\" line. Ignoring rest of config file" 
	       << endl;
	  failed = 1;
	  continue;
	}
	
	if (strcasecmp(char_ptr,"1BIT")==0          // Force 1 bit mode
	    || strcasecmp(char_ptr,"1")==0) {
	  force_bits = 1;
	  found = 1;

	} else if (strcasecmp(char_ptr,"2BIT")==0   // Force 2 bit mode
	    || strcasecmp(char_ptr,"2")==0) {
	  force_bits = 2;
	  found = 1;
	} else {
	  cerr << "Warning: Do not recognize force_bits option " << char_ptr
	       << endl;
	}
      }

      
      // Header multiplier (for oversampling)
      //else if (strcasecmp(char_ptr,"HEADER_MULTIPLIER")==0) {
      //char_ptr = strtok(NULL, " \t");
      //if (char_ptr==NULL) {
      //	  cerr << "Blank \"HEADER_MULTIPLIER\". Ignoring rest of config file" 
      //	       << endl;
      //	  failed = 1;
      //	  continue;
      //	}
      //	status = sscanf(char_ptr, "%d", &header_multiplier);
      //	if (status!=1) {
      //	  cerr << "Warning: Failed to match \"" << char_ptr << "\"" << endl;
      //	} else {
      //	  cout << "**Warning: Setting header multiplier to " 
      //	       <<  header_multiplier << endl;
      //	  found = 1;
      //	  
      //	}
      //      }


      // BOCF rate
      //if (strcasecmp(char_ptr,"BOCF_RATE")==0) {
      //  char_ptr = strtok(NULL, " \t");
      //  if (char_ptr==NULL) {
      //    cerr << "Blank \"BOCF_RATE\" line. Ignoring reset of config file" 
      //	    << endl;
      //    failed = 1;
      //    continue;
      //  }
      //  sscanf(char_ptr, "%d", &BOCF_rate);
      //  cerr << "Warning: Using BOCF rate of " << BOCF_rate << endl;
      //  found = 1;
      //}

    }
    fclose(su_cfg);
    if (!found) 
      cerr << "Hit EOF without finding any entries in " << cfg_file << endl;
  
  }
  return;
}


//__________________________________________________________________________

bool Init_Proc::construct_tables(Scan *scan, Table_Data *&table1, 
				 Table_Data *&table2, Table_Data *&table3, 
				 Table_Data *&table4, Table_Data *&table5, 
				 Table_Data *&table6) 
{
   int_16 the_data_field[5];
   int i = 0;//counter

   int BOCF_rate = cjd->get_bocf_rate();
   if (itsRecirculationFactor > 0)
     {
       cout << "Init_Proc::construct_tables() - recirculation factor = " << itsRecirculationFactor << endl;
       BOCF_rate /= itsRecirculationFactor;
     }
   cout << "Using BOCF rate of "<< BOCF_rate << endl;

   // Maybe some values have been overridden externally. Copy either
   // the automatic values or the external values into a new array

   char dim_xbar_values[NUM_TRACKS]; // Copy of dim_x_bar_configuration
   char ac_tracks[NUM_TRACKS]; // Copy of the_active_tracks

   // Maybe DIM override
   if (external_dim_xbar) {
     for(i=0; i<NUM_TRACKS; i++) 
       dim_xbar_values[i] = dim_xbar_override[i];
   } else {
     for(i=0; i<NUM_TRACKS; i++) 
       dim_xbar_values[i] = dim_x_bar_configuration[i];
   }

   // Maybe the TRM active override
   if (external_trm_active) {
     // -1 for default. 0 disabled, 1 enabled
     for(i = 0; i<NUM_TRACKS; i++) {
       if (trm_active_override[i]==-1) 
	 ac_tracks[i] = the_active_tracks[i];
       else 
	 ac_tracks[i] = trm_active_override[i];
     }
   } else {
     for(i = 0; i<NUM_TRACKS; i++) 
       ac_tracks[i] = the_active_tracks[i];
   }

   //now construct the various tables

   if(!(table1 = new DIM_Cross_bar_Switch_Table(dim_xbar_values)))
   {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
	    
	/* data_fields - persumably will be obtained from the mythical hardware db :
	        following taken from ~bryana/SUTEST/TST1-1/CFGTRM9.DAT
		0x0FFF      //RxParityThr    TRM config data which needs to be sorted out with KH
		0x0012      //Rxcontrol   was 2   sort 960119 92
		0x0030      //Txcontrol 
		0x0000      //FrameValidityMask 6    C
		0x0024      //Clockrecovery 
		
	   the_active_tracks[NUM_TRACKS], 1 element for each DIM O/P (TRM I/P) : 1 == active, 0 == inactive
	*/
	 
	 
   the_data_field[0] = RX_PARITY_THR;
   //RxParityThr : defines a threshold of parity errors at which to flag an error
	 
   the_data_field[1] = 0;
   the_data_field[1] |= 1<<1;//bit 1 (enable Rx)
   the_data_field[1] |= 1<<4;//bit 4 (12 bit CRC)
   if(the_tape_format == VLB) the_data_field[1] |= 1<<2;
   if(scan->get_record_format()->get_data_modulation() == on) 
        the_data_field[1] |= 1<<7;//see MkIV memo 230, para 2.1.4
   //Rxcontrol : bits 1 (enable Rx) and 4 (12 bit CRC) set true by default?
	 
   the_data_field[2] = 0;
   the_data_field[2] |= 1<<4;//bit 4 (Tx valid)
   the_data_field[2] |= 1<<5;//bit 5 (enable Tx cf DEcember visit to JIVE)
   //Txcontrol : bits 0, 3, 4 are obsolete, 5 & 6 are controlled by the TRM micro(?)
	 
   the_data_field[3] = 0;
   //if(invalid_if_CRC_error) the_data_field[3] |= 1<<0;
   the_data_field[3] |= 1<<1;//invalid_if_RxParityThr_exceeded 
   //if(invalid_if_lost_sync) the_data_field[3] |= 1<<2;
   //if(invalid_if_unexpected_sync) the_data_field[3] |= 1<<3;
   //FrameValidityMask : invalid_if_RxParityThr_exceeded by default?
	 
   switch ((int)(16000000.0 * muxmode/ (fs * speedup) + 0.01)) {
      case 1: the_data_field[4] = 0x0044; break;
      case 2: the_data_field[4] = 0x0024; break;
      case 4: the_data_field[4] = 0x0064; break;
      case 8: the_data_field[4] = 0x0014; break;
      default: 
	 //something wrong - generate an error?
	cout << "Init_Proc::construct_tables() - something wrong: invalid case "
	     << (int)(16000000.0 * muxmode/ (fs * speedup) + 0.01) << endl;
	cout << "    muxmode, fs, speedup = " << muxmode
	     << ", " << fs << ", " << speedup << endl;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	 return false;
   }
   
   //Clockrecovery 
	 
   if(!(table2 = new TRM_Configuration_Table(ac_tracks, the_data_field)))
   {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
		
	 
   for(i = 0; i < NUM_CHANNELS; ++i) the_frames_per_barrel_roll_ph[i] = (int)
      scan->get_barrel_rolling()->get_frames_per_phase(); //phases? !NUM_CHANNELS

   // Obsolete table
   //if(!(table3 = new TRM_Aux_Data_Check_Table(the_active_tracks, the_frames_per_barrel_roll_ph,
   //				      the_aux_data, the_mask)))
   //{
   //  this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
   //  return false;
   //}
	 
   unsigned short int the_misc_pars;

   float CC = fs * speedup / 1e+06;
   the_misc_pars = this->get_f(CC / muxmode, CC) + this->get_mux_demux(num_bits);
   short int the_frames_per_sec = (short int)(fs/(20000 * muxmode) + 0.01);
	 
   if(!(table4 = new CRM_Configuration_Table(THE_CROSS_TRACK_PARITY,
					     the_misc_pars,
					     the_frames_per_sec,
					     THE_CLOCKS_BETWEEN_TRACK_FRAMES,
					     the_frames_per_barrel_roll_ph,
					     the_cross_bar_config)))
   {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
												
	 /*	 
	 following taken from ~bryana/SUTEST/TST1-1/CFGDCM.DAT
	 # first line       // data down from here
	 #0x0007A120             // RotIntRate for 16 ticks  8 meg clock
	 #
	 0x00000004             // SYSCLK's  per STN_CLK.
	 0x00F0                 // header length
	 0x0001                 // header multiplier   (changed from 1 to 0 30/5/96)
	 0x003d0810             // Cframe length 2 frames per second ( - header ?)
	 0.00000000000000000e+0 // BOCF Offset
	 0x0010             // nchans
	 0x0002             // k_div
	 8.00000000000000000e+06    // fs
	 #
	 */ 

   // Optionally read the RotIntRate from the feature bit string
   unsigned int RotIntRate = 16;  // Default 16 interupts per second
   string feature_bits = cjd->get_feature_bits(4);
   if (!feature_bits.empty())
     {
       if (isalnum(feature_bits[0]))
	 {
	   RotIntRate = strtoul(feature_bits.c_str(), (char **)NULL, 36);
	   if (RotIntRate==0)
	     {
	       RotIntRate=16;
	       cout << "Warning: Bad value ("<< RotIntRate << ") for RotIntRate feature bit" << endl;
	     }
	   else
	     {
	       cout << "Warning: Using RotIntRate of " << RotIntRate << endl;
	     }
	 }
       else
	 {
	   cout << "Warning: Bad value ("<< feature_bits << ") for RotIntRate feature bit" << endl;
	 }
     }
   
   RotIntRate = (unsigned int)(fs * speedup / RotIntRate);
   //unsigned int SYSCLKS_per_STN_CLK = (unsigned int)(SYSTEM_RATE / (fs * speedup)); // CJP 090701
   unsigned int SYSCLKS_per_STN_CLK = (unsigned int)(SYSTEM_RATE / fs); 

   int header_multiplier;
   // Read values from the feature bits string
   string featureBits = cjd->get_feature_bits(5);
   if (!featureBits.empty())
     {
       cout << "Correlating oversampled data using *all* samples"<< endl;
       header_multiplier = 1;
     }
   else
     {
       header_multiplier = oversampling_factor;
     }
   unsigned int c_frame_len = (unsigned int)((fs * speedup)/BOCF_rate - 240 * header_multiplier);
   cout << "fs " << fs << " BOCF_rate " << BOCF_rate << " header_multiplier " << header_multiplier << " => c_frame_len " << c_frame_len << endl;
   
   unsigned int output_rate = (int)(fs * speedup / 1e6);
   ieee_dbl delta = 0.0;
   switch (output_rate) 
   {
      case (2) :  
        delta = 0; 
        cerr << "Warning: BOCF_OFFSET untested at 2 MHz output rate" << endl;
        break;
      case (4) :  delta = 16; break;
      case (8) :  delta = 64; break;
      case (16) : delta = 80; break;
      case (32) : delta = 96; break;
      default: 
       //something wrong - generate an error?
       cerr << "Unknown Output data rate (" << output_rate << ")" << endl;
       this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
       return false;
   }
   
   //   ieee_dbl bocf_offset = SYSTEM_RATE - delta;
   ieee_dbl bocf_offset = delta;
   
   if(!(table5 = new  Delay_Memory_Config_Table(RotIntRate,
						SYSCLKS_per_STN_CLK,
						0x00F0, // header length
						header_multiplier,
						c_frame_len, 
						bocf_offset)))
   {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
	 
   int   the_initial_tape_direction;

   int the_su_array_code;

   the_su_array_code = cjd->get_corr_job_ID();//for now!!!!
   cout << "Configuring with SU array code " << the_su_array_code << endl;
	 
   if(scan->get_tape_direction() == forward) the_initial_tape_direction = 1;
   else the_initial_tape_direction = 0;

   int   the_number_of_heads = 1;//from db?

   unsigned int temp = (unsigned int)(((fs * speedup * speedup) / (muxmode * bpi * 80)) + 0.5);
   ieee_dbl the_backup_TOT = (9.0/8.0) * temp * SYSTEM_RATE;
   //see email from Steve dated 12/12/97
   
   temp = (unsigned int)((9.0 * fs / (8.0 * muxmode * bpi)) + 0.5);
   ieee_dbl the_TOT_per_foot = 12.0 * SYSTEM_RATE / temp;
   
   cerr << "Using a formatter error of " << formatter_error << endl;
   
   if(!(table6 = new SU_Misc_Cmds_Table(the_su_array_code,
					the_tape_format,
					CLOCK_RATE,
					dpu_accel * 100,
					the_initial_tape_direction, 
					the_nominal_tape_speed*100,
					the_number_of_heads,
					dpu_type,
					speedup,
					formatter_error*SYSTEM_RATE,
					TAPE_SERVO_INTERVAL,
					TAPE_SERVO_MAX_SLEW_TIME,
					bpi, 
					SCAN_DATA_INTERVAL*speedup, 
					the_backup_TOT,
					the_TOT_per_foot)))
   {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   return true;
}

//___________________________________________________________________________

void Init_Proc::clear_queues(void)
{
   //delete any messages in queues
   
   while (cmd_iter->first())//while there is still an element in the list
   {
      delete (cmd_iter->first()->get_msg());//does not own msg
      delete (cmd_queue.remove(cmd_iter));
   }
   
   while (response_iter->first())//while there is still an element in the list
   {
      delete (response_iter->first()->get_msg());//does not own msg
      delete (response_queue.remove(response_iter));
   }
}

//_________________________________________________________________________

bool Init_Proc::in_response_queue(const unsigned int action_status)const
{
   //check response_queue for message with action_status
  
   CCC_Msg *ccc_msg;
   Action_Status *msg;
   
   if(response_iter->first() == 0) return false;//position iterator to first element if existant
   
   do{
      ccc_msg = (CCC_Msg *)((Incoming_Message_Info *)response_iter->contents())->get_msg();
      if(ccc_msg->get_message_type() == SU_Message_Types::Action_Status)
      {
	 msg = (Action_Status *) ccc_msg;
         if(msg->get_action_status() == action_status) return true;
      }
   }while (response_iter->advance());
   return false;
}

//________________________________________________________________________________

bool Init_Proc::data_in_response_queue(void)const
{
   //check response_queue for data message
   
   CCC_Msg *ccc_msg;
   
   if(response_iter->first() == 0) return false;//position iterator to first element if existant
   
   //check if ccc_msg is a data message - check action_code
   
   do{
      ccc_msg = (CCC_Msg *)((Incoming_Message_Info *)response_iter->contents())->get_msg();
      if(ccc_msg->get_message_type() == SU_Message_Types::Action_Request &&
	 ccc_msg->get_action_code() == SU_Action_Values::SU_Response)//must be a Data Message?
      {
	 return true;
      }	 
   }while (response_iter->advance());
   return false;
}

//________________________________________________________________________

void Init_Proc::do_exit_to_mounted(unsigned int error_code, unsigned int action_code, unsigned int ref_id)
{
  if (use_mark5) 
    {
      if (!eVLBI) disk_unit.stop_play();
      else
	{
	  disk_unit.in2net("off");
	  closeEVLBIout(false);
	  disk_unit.in2net("disconnect");
	}
    }
  else this->start_dpu(false);//stop dpu
  this->transmit_error(error_code, action_code, ref_id);
  allocated = false;
  su_cmd->close_medium_handler_conns();
  current_state = mounted;
  if (use_mark5) this->transmit_disk_status(SU_Control_SU_Status_Codes::Error, error_code);
  else this->transmit_tape_status(SU_Control_SU_Status_Codes::Error, error_code);
   
#ifdef DEBUG 
   cout << "Some problem - " << errors.get_error_text(error_code) << endl
	<< "return to mounted state."<< endl;
#endif
}

//______________________________________________________________________________

void Init_Proc::transmit_error(unsigned int error_code, unsigned int action_code, unsigned int ref_id)
{
   //the spec of this msg also has error parameters - not yet implemented.
   
   Connection *target;
 
   error_msg.set_error_code(error_code);
   error_msg.set_job_id(id_of_job);
   error_msg.set_action_code(action_code);
   error_msg.set_su_id(the_suid_code);
   error_msg.set_sender_id(SU_CONTROL_ID);
   error_msg.set_receiver_id(su_cmd->get_PC_Handle_ID());
   error_msg.set_ref_id(ref_id);
   
   
   this->set_local_time();
       
   error_msg.set_local_time(local_time);  
   error_msg.set_time_stamp(time_stamp); 
   
   target = su_cmd->get_connection(PROCESSOR_CONTROL);
   if(target) error_msg.transmit(*target);
   
   su_cmd->transmit_medium_handlers(&error_msg);
}   

//______________________________________________________________________________

int Init_Proc::transmit_tape_status(unsigned int status, unsigned int error_code)
{
   Action_Status *msg;
   Connection *target;
   char tape_bar_code[MAX_NR_BANKS][bar_code_len], temp[bar_code_len];
   unsigned int dpu_id = the_suid_code;
   int i;//counter
   char station_id[16][TELESCOPE_ID_LEN];
   for(i = 0; i < 16; ++i) station_id[i][0] = '\0';
   unsigned int stations = 0;//# of 
   
   tape_bar_code[1][0] = 0;
   if (current_station)
     {
       stations = current_station->get_nr_of_telescopes();
       for (i=1; i<stations+1; ++i)
	 {
	   string tel_id = current_station->get_nth_telescope_name(i);
	   if (tel_id.empty()) break; //error
	   
	   //then what? we need two letter codes
	   CompString site_name(tel_id);
	   StoreObject& telescope = cjd->get_telescopes()->get(site_name);
	   if (!telescope.valid()) break; //error

	   string tmpstr = ((Telescope &) telescope).get_site_ID();
	   tmpstr.copy(station_id[i-1],string::npos);
	   station_id[i-1][tmpstr.length()] = 0;
	 }
     }
   
   if(su_configuration) dpu_id = (unsigned int)su_configuration->get_DPU_id();
   
   /*
   this->transmit_tape_status(SU_Control_SU_Status_Codes::Parity_Errors);

   struct SU_Control_SU_Status_Codes 
   {
     enum Values
     {
      Medium_Mounted               = 0x00000001,
      SU_Allocated                 = 0x00000002,
      Configuring_SU               = 0x00000004,
      Tape_Positioning_Footage     = 0x00000008,
      Peaking_Heads                = 0x00000010,
      Medium_Waiting               = 0x00000020,
      Medium_Running               = 0x00000040,
      Finished                     = 0x00000080,
      Tape_Positioning_TOT         = 0x00000100,
      Tape_Moving_Forward          = 0x00000200,
      Tape_Moving_Reverse          = 0x00000400,
      Tape_Moving_Fast             = 0x00000800,
      Parity_Errors                = 0x00001000,
      Servo_Error                  = 0x00002000,
      DPU_Not_Present              = 0x00004000,
      Error                        = 0x00008000,
      Tape_Packing                 = 0x00010000,
      Medium_Processing            = 0x00020000,
      Tape_PrePass                 = 0x00040000,
      Heads_Peaked                 = 0x00080000
     };
   };
   */
   
   if(error_code == SU_Control_Error_Codes::SU_Connection_Exception) 
      status |= SU_Control_SU_Status_Codes::DPU_Not_Present;
   
   switch (current_state)
   {
      case (configure_dpu) :
      case (configure_su) :
      case (form_pcal_tables) :
	 status |= SU_Control_SU_Status_Codes::Configuring_SU;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (pos_to_footage) :
	 status |= SU_Control_SU_Status_Codes::Tape_Positioning_Footage;
	 break;
      case (peak_heads) :
	 status |= SU_Control_SU_Status_Codes::Peaking_Heads;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;	 
      case (heads_peaked):
	 status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (pos_to_footage_2) :
	 status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status |= SU_Control_SU_Status_Codes::Tape_Positioning_Footage;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (positioning1) :
	 status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status |= SU_Control_SU_Status_Codes::Tape_Positioning_TOT;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (wait_for_ROT) :
      case (wait_job_starting) :
      case (configure_su2) :
      case (get_polynomials) :
	 status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status += SU_Control_SU_Status_Codes::Medium_Waiting;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (running) :
	 if(peaking_thread == 0) status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status |= SU_Control_SU_Status_Codes::Medium_Running;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
   }
   
   if(initialised && su_status->get_medium_mounted_status())
   {
      status |= SU_Control_SU_Status_Codes::Medium_Mounted;
      su_status->get_medium_bar_code(temp);
      strncpy(tape_bar_code[0], temp, 8);
      tape_bar_code[0][8] = '\0';
   }
   else
   {
      for(i = 0; i < bar_code_len; ++i) tape_bar_code[0][i] = 0;
   }
   
   if(allocated)
   {
      status |=  SU_Control_SU_Status_Codes::SU_Allocated;
   }
   
   int_32 head_positions[4];
   int num_heads = 0;
   su_status->get_head_positions(head_positions);
   if(num_head_stacks == 0)
   {
      for(i = 0; i < 4; ++i) if(head_positions[i] != 0) ++num_head_stacks;
   }
   
   if(msg = new SU_Control_SU_Status(the_suid_code,
				     tape_bar_code,
				     dpu_type,//dpu_type_code,
				     dpu_id,
				     allocated ? id_of_job : 0,
				     status,
				     error_code,
				     stations,
				     station_id,
				     num_head_stacks,
				     head_positions,
				     use_mark5,
				     eVLBI))

   {
	this->set_local_time();
       
   	msg->set_local_time(local_time);  
   	msg->set_time_stamp(time_stamp); 
        msg->set_action_code(SU_Action_Values::Unsolicited);
	msg->set_sender_id(SU_CONTROL_ID);
     	msg->set_receiver_id(su_cmd->get_SUM_Handle_ID());
        target = su_cmd->get_connection(SU_MONITOR);
	if(target)
	{
	   msg->transmit(*target);
	   delete msg;
	   msg = 0;
	   return status;
	}
	delete msg;
	msg = 0;
   }
      
   return -1;
}
//______________________________________________________________________________

void Init_Proc::get_du_parameters(Object_Store& mystore, Disk_Unit& disk_unit, bool eVLBI, int the_suid_code) {
  //get associated disk unit IP address from store based on su_ID
  SU_Params_Query *query = (SU_Params_Query *) mystore.get_query_object(SU_PARAMS);
  //set retrieval condition & criterion
  query->set_su_id(the_suid_code);
  query->set_su_id_cond(Table_Column::eq);
  query->set_assoc_du_ip_cond(Table_Column::any);
  query->set_assoc_ext_du_ip_cond(Table_Column::any);
  query->set_assoc_e_station_name_cond(Table_Column::any);

  //do query
      
  Iter_List<Query_Object *> *list = mystore.get_multi_object(query);
      
  //check contents of list
      
  if(! list)
    {
      cout << "no list! " << endl;
      if(persistent_error == 0)
	{
	  //no matches found
	  cout << "No matches" << endl;
	}
      else
	{
	  //query failed 
	  cout << "Query failed" << endl;
	}
      delete query; query = 0;
      return;
    }

      
  Forward_List_Iter<Query_Object *> iter(list);
      
  SU_Params_Query *result = (SU_Params_Query *) iter.first(); 
      
  char du_ip[50];
  char ext_du_ip[50];
  char e_station_name[50];

  strcpy(du_ip,result->get_assoc_du_ip());
  if (disk_unit.merlin_slave) {
    // we will use the mounted e_station to form a VSN for this SU
    char temp[MAX_NR_BANKS][MEDIUM_ID_LEN];
    strlcpy(&temp[0][0],result->get_assoc_e_station_name(), 3);		
    strcat(&temp[0][0],"-slave");
    for (int bank = 1; bank < MAX_NR_BANKS; bank++) {
      temp[bank][0] = '\0';
    }
    disk_unit.du_status.set_VSN(temp,0);
  }

  if (eVLBI)
    {
      strcpy(e_station_name,result->get_assoc_e_station_name());
      strcpy(ext_du_ip,result->get_assoc_ext_du_ip());
    }

  delete query; query = 0;
      
  disk_unit.du_status.set_ip(du_ip);

  if (eVLBI)
    {
      disk_unit.du_status.set_e_station_name(e_station_name);
      disk_unit.du_status.set_ext_ip(ext_du_ip);

      //create fake vsn, will have to agree with fake vsn in vex file
      char temp[MAX_NR_BANKS][MEDIUM_ID_LEN];
      strcpy(temp[0],disk_unit.du_status.get_e_station_name());		
      strncat(temp[0],"-eVLBI",7);
      for (int bank = 1; bank < MAX_NR_BANKS; bank++) {
	temp[bank][0] = '\0';
      }
      disk_unit.du_status.set_VSN(temp,0);

	  	  
      //now get evlbi station unit IP address from store based on station name
      eVLBI_Params_Query *e_query = (eVLBI_Params_Query *) mystore.get_query_object(EVLBI_PARAMS);
      //set retrieval condition & criterion
      e_query->set_e_station_name(disk_unit.du_status.get_e_station_name());
      e_query->set_e_station_name_cond(Table_Column::eq);
      e_query->set_e_station_control_ip_cond(Table_Column::any);
      e_query->set_sockbuf_cond(Table_Column::any);
      e_query->set_workbuf_cond(Table_Column::any);
      e_query->set_protocol_cond(Table_Column::any);
      e_query->set_mtu_cond(Table_Column::any);
      e_query->set_connectivity_cond(Table_Column::any);
      e_query->set_ipd_cond(Table_Column::any);
      e_query->set_trackmask_cond(Table_Column::any);

      //do query
	  
      Iter_List<Query_Object *> *e_list = mystore.get_multi_object(e_query);
	  
      //check contents of list
	  
      if(! e_list)
	{
	  cout << "no e_list! " << endl;
	  if(persistent_error == 0)
	    {
	      //no matches found
	      cout << "No matches" << endl;
	    }
	  else
	    {
	      //query failed 
	      cout << "e_Query failed" << endl;
	    }
	  delete e_query; e_query = 0;
	  return;
	}
	  
      Forward_List_Iter<Query_Object *> e_iter(e_list);
	  
      eVLBI_Params_Query *e_result = (eVLBI_Params_Query *) e_iter.first(); 

      char e_station_control_ip[50];
      char protocol[8];
      int_32 workbuf;
      int_32 sockbuf;
      int_32 mtu;
      int_32 connectivity;
      int_32 ipd;
      uint64 trackmask;
	  
      strcpy(e_station_control_ip,e_result->get_e_station_control_ip());
      strcpy(protocol,e_result->get_protocol());
      workbuf = (int_32)e_result->get_workbuf();
      sockbuf = (int_32)e_result->get_sockbuf();
      mtu = (int_32)e_result->get_mtu();
      connectivity = (int_32)e_result->get_connectivity();
      ipd = (int_32)e_result->get_ipd();
      trackmask = (uint64)e_result->get_trackmask();

      if (disk_unit.du_status.get_e_station_control_ip() != string(e_station_control_ip)) cout << "e_station_control_ip: " << e_station_control_ip << endl;
      if (disk_unit.du_status.get_workbuf_size() != workbuf) cout << "workbuf: " << workbuf << endl;
      if (disk_unit.du_status.get_sockbuf_size() != sockbuf) cout << "sockbuf: " << sockbuf << endl;
      if (disk_unit.du_status.get_protocol() != string(protocol)) cout << "protocol: " << protocol << endl;
      if (disk_unit.du_status.get_mtu_size() != mtu) cout << "mtu: " << mtu << endl;
      if (disk_unit.du_status.get_connectivity() != connectivity) cout << "connectivity: " << connectivity << endl;
      if (disk_unit.du_status.get_ipd() != ipd) cout << "ipd: " << ipd << endl;
      if (disk_unit.du_status.get_trackmask() != trackmask) cout << "trackmask: 0x" << hex << trackmask << dec << endl;

      disk_unit.du_status.set_e_station_control_ip(e_station_control_ip);
      disk_unit.du_status.set_protocol(protocol);
      disk_unit.du_status.set_sockbuf_size(sockbuf);
      disk_unit.du_status.set_workbuf_size(workbuf);
      disk_unit.du_status.set_mtu_size(mtu);
      disk_unit.du_status.set_connectivity(connectivity);
      disk_unit.du_status.set_ipd(ipd);
      disk_unit.du_status.set_trackmask(trackmask);
    

      delete e_query; e_query = 0;
    }
}

int Init_Proc::transmit_disk_status(unsigned int status, unsigned int error_code)
{
  get_du_parameters(mystore,disk_unit,eVLBI,the_suid_code);

  //Ignore dpu_id, it is set to 0 (always). Get bar_code from disk. Have to change 
  //SU_Control_Status_Codes to include disk systems, and su_control to expect disks.
  Action_Status *msg;
  Connection *target;
  char disk_bar_code[MAX_NR_BANKS][bar_code_len], temp[bar_code_len];
  unsigned int dpu_id = the_suid_code;
  int i;//counter
  char station_id[16][TELESCOPE_ID_LEN];
  for(i = 0; i < 16; ++i) station_id[i][0] = '\0';
   unsigned int stations = 0;//# of 
   
   if (current_station)
     {
       stations = current_station->get_nr_of_telescopes();
       for (i=1; i<stations+1; ++i)
	 {
	   string tel_id = current_station->get_nth_telescope_name(i);
	   if (tel_id.empty()) break; //error
	   
	   //then what? we need two letter codes
	   CompString site_name(tel_id);
	   StoreObject& telescope = cjd->get_telescopes()->get(site_name);
	   if (!telescope.valid()) break; //error

	   string tmpstr = ((Telescope &) telescope).get_site_ID();
	   tmpstr.copy(station_id[i-1],string::npos);
	   station_id[i-1][tmpstr.length()] = 0;
	 }
     }
   
   if(su_configuration) dpu_id = (unsigned int)su_configuration->get_DPU_id();
   for (int bank = 0; bank < MAX_NR_BANKS; bank++) {
     strncpy(disk_bar_code[bank],disk_unit.du_status.get_VSN(bank),bar_code_len);
   }
   
   /*
   this->transmit_tape_status(SU_Control_SU_Status_Codes::Parity_Errors);

   struct SU_Control_SU_Status_Codes 
   {
     enum Values
     {
      Medium_Mounted               = 0x00000001,
      SU_Allocated                 = 0x00000002,
      Configuring_SU               = 0x00000004,
      Tape_Positioning_Footage     = 0x00000008,
      Peaking_Heads                = 0x00000010,
      Medium_Waiting               = 0x00000020,
      Medium_Running               = 0x00000040,
      Finished                     = 0x00000080,
      Tape_Positioning_TOT         = 0x00000100,
      Tape_Moving_Forward          = 0x00000200,
      Tape_Moving_Reverse          = 0x00000400,
      Tape_Moving_Fast             = 0x00000800,
      Parity_Errors                = 0x00001000,
      Servo_Error                  = 0x00002000,
      DPU_Not_Present              = 0x00004000,
      Error                        = 0x00008000,
      Tape_Packing                 = 0x00010000,
      Medium_Processing            = 0x00020000,
      Tape_PrePass                 = 0x00040000,
      Heads_Peaked                 = 0x00080000
     };
   };
   */
   
   if(error_code == SU_Control_Error_Codes::SU_Connection_Exception) 
      status |= SU_Control_SU_Status_Codes::DPU_Not_Present;
   
   switch (current_state)
   {
      case (configure_su) :
      case (form_pcal_tables) :
	 status |= SU_Control_SU_Status_Codes::Configuring_SU;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (positioning1) :
	 status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status |= SU_Control_SU_Status_Codes::Tape_Positioning_TOT;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (wait_for_ROT) :
      case (wait_job_starting) :
      case (configure_su2) :
      case (get_polynomials) :
	 status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status += SU_Control_SU_Status_Codes::Medium_Waiting;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
      case (running) :
	 if(peaking_thread == 0) status |= SU_Control_SU_Status_Codes::Heads_Peaked;
	 status |= SU_Control_SU_Status_Codes::Medium_Running;
	 status |= SU_Control_SU_Status_Codes::Medium_Processing;
	 break;
   }
   
   if(initialised && su_status->get_medium_mounted_status())
   {
      status |= SU_Control_SU_Status_Codes::Medium_Mounted;
   }
   
   if(allocated)
   {
      status |=  SU_Control_SU_Status_Codes::SU_Allocated;
   }
   
   //   int_32 head_positions[4];
   //   int num_heads = 0;
   //   su_status->get_head_positions(head_positions);
   //   if(num_head_stacks == 0)
   //   {
   //      for(i = 0; i < 4; ++i) if(head_positions[i] != 0) ++num_head_stacks;
   //   }

   //fake it all
   
   int_32 head_positions[4];
   for(i = 1; i < 4; ++i) head_positions[i] = 1;
   head_positions[0] = 1;
   num_head_stacks = 1;


   if(msg = new SU_Control_SU_Status(the_suid_code,
				     disk_bar_code,
				     dpu_type,//dpu_type_code,
				     dpu_id,
				     allocated ? id_of_job : 0,
				     status,
				     error_code,
				     stations,
				     station_id,
				     num_head_stacks,
				     head_positions,
				     use_mark5,
				     eVLBI))

   {
	this->set_local_time();
       
   	msg->set_local_time(local_time);  
   	msg->set_time_stamp(time_stamp); 
        msg->set_action_code(SU_Action_Values::Unsolicited);
	msg->set_sender_id(SU_CONTROL_ID);
     	msg->set_receiver_id(su_cmd->get_SUM_Handle_ID());
        target = su_cmd->get_connection(SU_MONITOR);
	if(target)
	{
	   msg->transmit(*target);
	   delete msg;
	   msg = 0;
	   return status;
	}
	delete msg;
	msg = 0;
   }
      
   return -1;
}

//____________________________________________________________________________________

bool Init_Proc::transmit_to_SU(CCC_Msg *cccmsg)
{
   if (time(0) < next_SU_command_time) 
     { // a reset_SU_peripherals command has been sent, wait at least 15 seconds before next command
       sleep(next_SU_command_time - time(0));
     }
   sleep(1);
   Connection* target;
   
   target = su_conn->get_connection(S_U);
   if(target )
   {
      cccmsg->transmit(*target);
      //reset timer
      timer.reset(timer_key);
      return true;
   }
   else
   {
      return false;
   }
}

//_________________________________________________________________________________

bool Init_Proc::transmit_data_reqs(Scan *scan, ieee_dbl start)
{
   Command_Message *cmd;
   Aux_Data *aux_table = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   ieee_dbl interval = DATA_REQ_INTERVAL * SYSTEM_RATE;
   ieee_dbl servo_interval = SERVO_REQ_INTERVAL * SYSTEM_RATE * speedup; 
   ieee_dbl end_TOT = 0;
   ieee_dbl current_scan_start = start;
   
   scan->get_stop_TOT() <= sub_job_ending_ROT ? end_TOT = scan->get_stop_TOT() :
      end_TOT = sub_job_ending_ROT;
   
   // FIXME: Request Phase Calibration Data here if this function ever
   // gets used again.

   //send Request Sampler Stats cmd to SU 
   if(!(cmd = form_command(SU_Station_Unit_Cmds::Req_Sampler_Stats)))
   {
       this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
       return false;
   }
   
   //set obey ROT, End ROT and repeat interval
   cmd->set_obey_ROT(current_scan_start + (interval * 0.9));//stagger
   cmd->set_end_ROT(end_TOT);
   cmd->set_rpt_intrvl(interval);
   cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
		  SU_Action_Request_Flags::Obey_ROT_Set +
		  SU_Action_Request_Flags::End_ROT_Set);
   
   if(!(this->transmit_to_SU(cmd)))
   {
      this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
      delete cmd;
      cmd = 0;
      return false;
   }
   delete cmd; cmd = 0;

   //send Request Servo Error cmd to SU 

   if (use_mark5)
     {
       if(!(cmd = form_command(SU_Station_Unit_Cmds::Req_Servo_Error)))
	 {
	   this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	   return false;
	 }
       
       //set obey ROT, End ROT and repeat interval
       cmd->set_obey_ROT(current_scan_start + (interval * 0.9));//stagger
       cmd->set_end_ROT(end_TOT);
       cmd->set_action_code(SU_Action_Values::Station_Unit);
       cmd->set_rpt_intrvl(servo_interval);
       cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
		      SU_Action_Request_Flags::Obey_ROT_Set +
		      SU_Action_Request_Flags::End_ROT_Set);
       
       if(!(this->transmit_to_SU(cmd)))
	 {
	   this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	   delete cmd;
	   cmd = 0;
	   return false;
	 }
       delete cmd; cmd = 0;
     }

   //next send average statistics message
   
   if(!(cmd = form_command(SU_Station_Unit_Cmds::Req_Av_Stats)))
   {
       this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
       return false;
   }
   
   //set obey ROT, End ROT and repeat interval
   cmd->set_obey_ROT(current_scan_start + (interval * 0.5));//stagger
   cmd->set_end_ROT(end_TOT);
   cmd->set_rpt_intrvl(interval);
   cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
		  SU_Action_Request_Flags::Obey_ROT_Set +
		  SU_Action_Request_Flags::End_ROT_Set);
   
   if(!(this->transmit_to_SU(cmd)))
   {
      this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
      delete cmd;
      cmd = 0;
      return false;
   }
   delete cmd; cmd = 0;
   return true;
}

//____________________________________________________________________________

bool Init_Proc::resume_clock_msgs(void)
{
   //send msg to rot-clock to resume messages
	 
	  Msg_Handler_ID *target_handle = new Msg_Handler_ID();//temporary
	  
	  Resume_Clk_Msgs_Request *scmr = 0;
	  
	  if(!(scmr = new Resume_Clk_Msgs_Request(SU_CONTROL_ID, *target_handle, cjd->get_corr_job_ID(), the_suid_code)))
	    {
	      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	      return false;
	    }
	  
	  this->set_local_time();
       
	  scmr->set_local_time(local_time);  
	  scmr->set_time_stamp(time_stamp);
	  
	  Connection *target = su_cmd->get_connection(ROT_CLOCK);

	  if(target)
	  {
	     scmr->transmit(*target);
	     clk_msgs_disabled = false;
	     delete scmr;
	     scmr = 0;
	     return true;
	  }
	  else
	  {
	     delete scmr;
	     scmr = 0;
	     this->do_exit_to_mounted(SU_Control_Error_Codes::ROT_Clock_Conn_Exception);
	     return false;
	  }
}

//______________________________________________________________________________________

bool Init_Proc::suspend_clock_msgs(void)
{
   Msg_Handler_ID *target_handle = new Msg_Handler_ID();//temporary
	  
	  Suspend_Clk_Msgs_Request *scmr = 0;
	  
	  if(!(scmr = new Suspend_Clk_Msgs_Request(SU_CONTROL_ID, *target_handle, cjd->get_corr_job_ID(), 
						   the_suid_code)))
	    {
	      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	      return false;
	    }
	  
	  this->set_local_time();
       
	  scmr->set_local_time(local_time);  
	  scmr->set_time_stamp(time_stamp);
	  
	  Connection *target = su_cmd->get_connection(ROT_CLOCK);

	  if(target)
	  {
	     scmr->transmit(*target);
	     delete scmr;
	     scmr = 0;
	     clk_msgs_disabled = true;
	     //sleep(5);
	     return true;
	  }
	  else
	  {
	     delete scmr;
	     scmr = 0;
	     this->do_exit_to_mounted(SU_Control_Error_Codes::ROT_Clock_Conn_Exception);
	     return false;
	  }
}

//__________________________________________________________________________________

bool Init_Proc::send_load_poly_cmd(Command_Message *cmd, ieee_dbl rot, bool update_rot)
{
   Table_Data *table1 = 0;
   Load_Polynomials_Aux *aux_table = 0;
   General_CCC_Table *gen_tab1 = 0;
   CompDouble end_key(DBL_MAX);
   
   int i, j, intervals = 0;
   ieee_dbl the_splines[9][194];//this comes to a size of 15520
   ieee_dbl *delay_polys;
   ieee_dbl *phase_polys;
   ieee_dbl last_End = 0; // End time of second last poly end to be used as 
                          // obey time of subsequent load polynomial messages.
   //unsigned short int k_div = (unsigned short int)(log2(SYSTEM_RATE / (bandwidth*2)));
   //unsigned short int the_nchan = 0x0010;

   for(i = 0; i < 9; ++i)
     {
       for(j = 0; j < 194; ++j)
	 {
	   the_splines[i][j] = 0.0;
	 }
     }

   Polys *p_polys = this->get_polys();
   if(p_polys == 0) return false;
   Ordered_List *polys = p_polys->get_ordered_polys();

   //first get aux data 
   CompDouble key((double) rot);
   StoreObject& obj = load_polys_aux->get_eq_or_less(key);
   
   if(!obj.valid())
   {
     cout << "ERROR: in send_load_poly_cmd() - invalid StoreObject for key "
	  << (double)rot << endl;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Polynomials_Error);
      return false;
   }

   //now get first poly_set
   Poly_set *poly_set = 0;
   CompDouble key2((double) rot);
   poly_set = &((Poly_set &)(polys->get(key2, end_key)));
   
   if(!poly_set->valid())
   {
     cout << "ERROR: in send_load_poly_cmd() - invalid poly_set for key "
	  << (double)rot << endl;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Polynomials_Error);
      return false;
   }  
   
   ieee_dbl begin_ROT_old = begin_ROT;
   ieee_dbl end_ROT_old = end_ROT;

   begin_ROT = poly_set->get_begin_rot();
   end_ROT = poly_set->get_end_rot();
   
   if((aux_table = new Load_Polynomials_Aux(((Storeable_Aux_Data &)obj).get_data())))
   {
      do{
	 if(intervals > 0)//ie if already round while loop check if aux data the same
	 {
	   last_End = end_ROT; // Don't need to worry about initialisation because if
	                       // this loop isn't done many times, a further load 
	                       // polynomial message won't be sent (I think).
 
	    begin_ROT = poly_set->get_begin_rot();
	    end_ROT = poly_set->get_end_rot();
	    CompDouble key_2((double) begin_ROT);
	    StoreObject& obj_2 = load_polys_aux->get_eq_or_less(key_2);
	    
	    if(!obj_2.valid() || &obj != &obj_2)
	    {
	       //really want the real key value as the obey rot of the next load polys cmd!
	       poly_obey_ROT = key_2.get_double();//THIS DISN'T WORK!!!
	       cout << "New Aux Table reqd at ROT = " << poly_obey_ROT << endl;
	       break;//out of do loop

	    }//end if(!obj_2.valid() || &obj != &obj_2)
	 }//end if intervals
	 
	 
	 the_splines[intervals][0] = begin_ROT;
	 the_splines[intervals][1] = end_ROT;
	 
	 delay_polys = poly_set->get_delay_polynomials();
	 phase_polys = poly_set->get_phase_polynomials();
       
	 //have to take into account logical to physical channel mapping
	 for(i = 0; i < NUM_CHANNELS; ++i) {
	   for(j = 0; j < 6; ++j) {
	     if(output_mapping[i] == 0xFFFFFFFF) {
	       ++delay_polys;
	       ++phase_polys;
	     } else {
	       the_splines[intervals][output_mapping[i] * 6 + j + 2] = *delay_polys++ + (j==0?poly_offset:0);//[i][j];
	       the_splines[intervals][output_mapping[i] * 6 + j + 98] = *phase_polys++;//[i][j];
	     }
	   }
	 }

	 if (external_crm_shuffle) {

	   int realchan;
	   ieee_dbl new_splines[192];

	   for(i=0; i<NUM_CHANNELS; i++) {
	     realchan = crm_shuffle[i];
	     for (j=0; j<6; j++) {
	       if (realchan!=-1) {
		 new_splines[i*6 + j] = the_splines[intervals][realchan*6 + j + 2];
		 new_splines[i*6 + j + 96] = the_splines[intervals][realchan*6 + j + 98];
	       } else {
		 new_splines[i*6 + j] = 0.0;
		 new_splines[i*6 + j + 96] = 0.0;
	       }
	     }
	   }
	   for (j=0;j<192;j++)
	     the_splines[intervals][j+2] = new_splines[j];
	 }

	 ++intervals;
	 //CJP 24/10 poly_obey_ROT = begin_ROT;
	 poly_obey_ROT = last_End; // Next load polynomial message should only start when
	                           // second last from this message complete
	 CompDouble key3(end_ROT + 1);
	 //get next poly_set - key2 has had the next comparable copied into it by previous call
	 poly_set = &((Poly_set &)(polys->get(key3, end_key)));
       
      }while(poly_set->valid() &&
	     end_ROT < sub_job_ending_ROT &&
	     intervals < 9);
   }
   else
   {
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   
   if((table1 = new Load_Polynomials_Table(intervals, the_splines)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::Delay_Phase_Poly, table1, aux_table)) &&
      cmd->add_table(gen_tab1))
   {
     if (!update_rot) 
       {
	 begin_ROT = begin_ROT_old;
	 end_ROT = end_ROT_old;
       }
     if(this->transmit_to_SU(cmd))
       {
	 cout << "# Poly sets loaded = " << intervals << endl;
	 return true;
       }
     else
       {
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
       }
   }
   else
   {
      delete table1; table1 = 0;
      delete aux_table; aux_table = 0;
      delete gen_tab1; gen_tab1 = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }

   return false;
}
//___________________________________________________________________-

bool Init_Proc::send_alarm_request(unsigned int parameter, ieee_dbl obey_rot)
{
  cout << "sending alarm request " << parameter << " rot " << obey_rot << endl;
   //create a default Msg_Handler_ID for the target ie ROT_Clock
   Msg_Handler_ID *target_handle = new Msg_Handler_ID();//temporary
		  
   Set_Alarm_Request *acm = 0;

   if(!(acm = new Set_Alarm_Request(SU_CONTROL_ID, *target_handle, cjd->get_corr_job_ID(), parameter)))
     {
       this->do_abort();
       this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
       return false;
     }

   acm->set_obey_ROT(obey_rot);
   acm->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
   this->set_local_time();
   acm->set_local_time(local_time);
   acm->set_time_stamp(time_stamp);
			
   Connection *target = su_cmd->get_connection(ROT_CLOCK);

   if(target)
   {
      acm->transmit(*target);
      delete acm;
      acm = 0;
      return true;
   }
   else
   {
      delete acm;
      acm = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::ROT_Clock_Conn_Exception);
      return false;
   }
}

//___________________________________________________________________

bool Init_Proc::send_req_tpe_frm_hdr(const int track_nr, int &counter, unsigned int &msg_id)
{
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   cmd = this->form_command(SU_Station_Unit_Cmds::Req_Medium_Frme_Hdr);
   
   if(!cmd)
   {
      this->do_abort();
;      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   
   msg_id = (unsigned int)cmd->get_msg_id();//Request_medium_frame_header_id
   
   for(int i = 0; i < 3; ++i)
   {
      if((table1 = new Req_Medium_Frame_Header_Table(track_nr + i)) &&
	 (gen_tab1 = new General_CCC_Table(SU_Table_Types::Medium_Frame_Hd_Req, table1, 0)) && 
	 cmd->add_table(gen_tab1))
      {
	 if(this->transmit_to_SU(cmd))
	 {
	    ++counter;
	    delete (cmd->remove_table(SU_Table_Types::Medium_Frame_Hd_Req));
	    table1 = 0;
	    gen_tab1 = 0;
	 }
	 else
	 {
	    delete cmd; 
	    cmd = 0;
	    table1 = 0;
	    gen_tab1 = 0;
	    this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	    return false;
	 }
      }
      else
      {
	 delete gen_tab1;
	 gen_tab1 = 0;
	 delete table1;
	 table1 = 0;
	 delete cmd; 
	 cmd = 0;
	 this->do_abort();
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	 return false;
      }
   }
   
   delete cmd;
   cmd = 0;
   table1 = 0;
   gen_tab1 = 0;
   
   return true;
}

//______________________________________________________________________________

bool Init_Proc::send_set_ROT(ieee_dbl rot)
{
   Set_ROT_Clock_Message *srcm = 0;
   Long_Time_Value *ltv_rot = 0;
   Clock_Message_Entry *cme = 0;

   ieee_dbl rate = SYSTEM_RATE * speedup;//variable
   
   int the_su_array_code;
   the_su_array_code = cjd->get_corr_job_ID();//for now!!!!
	
   if((ltv_rot = new Long_Time_Value(rot, (int)cjd->get_obs_TOT_start().get_base_date())) &&
      (cme = new Clock_Message_Entry(the_su_array_code, *ltv_rot, rate, the_suid_code)) &&
      (srcm = new Set_ROT_Clock_Message(SU_CONTROL_ID, su_conn->get_SUID(), &cme, 1)))
   {
      //time stamped etc?
      if(!(this->transmit_to_SU(srcm)))
      {
	 delete srcm; srcm = 0;
	 ltv_rot = 0; cme = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
      else
      {
	 delete srcm; srcm = 0; ltv_rot = 0; cme = 0;
	 return true;
      }
   }
   else
   {
      delete ltv_rot; ltv_rot = 0;
      delete cme; cme = 0;
      delete srcm; srcm = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
}

//___________________________________________________________________

bool Init_Proc::configure_suim(void)
{
   //now configure the suim
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   int_16 Tick_Generator = 0;
	 
   switch ((int)(SYSTEM_RATE / (fs * speedup) + 0.01))
   {
      case 1 : Tick_Generator = 0x01; break;
      case 2 : Tick_Generator = 0x09; break;
      case 4 : Tick_Generator = 0x11; break;
      case 8 : Tick_Generator = 0x19; break;
      case 16 : Tick_Generator = 0x21; break;
   }
   
   int_16 SUIM_config_array[64];
   for(int i = 0; i < 64; ++i)
   {
      SUIM_config_array[i] = 0;
   }
   //are all 0 - flushes suim
	    
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Configure_SUIM)) &&
      (table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 //delete (cmd->remove_table(SU_Table_Types::SUIM_Settings));
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      } 
   }	
   else
   {
      delete gen_tab1;
      gen_tab1 = 0;
      delete table1;
      table1 = 0;
      delete cmd;
      cmd = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   
   SUIM_config_array[0] = 0x0003;
   SUIM_config_array[2] = Tick_Generator;
   SUIM_config_array[3] = 0x0002;
   SUIM_config_array[5] = 0x0003;
   //rest are all 0 - values copied from test program at JIVE
	    
   if((table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      } 
   }	
   else
   {
      delete gen_tab1;
      gen_tab1 = 0;
      delete table1;
      table1 = 0;
      delete cmd;
      cmd = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
}

//_____________________________________________________________________
bool Init_Proc::configure_suim_2(void)
{
   //now configure the suim
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   int_16 Tick_Generator = 0;
   static int_16 tick_gen = 0;
	 
   switch ((int)(SYSTEM_RATE / (fs * speedup) + 0.01))
   {
     case 1 : Tick_Generator = 0x00; break;
     case 2 : Tick_Generator = 0x08; break;
     case 4 : Tick_Generator = 0x10; break;
     case 8 : Tick_Generator = 0x18; break;
     case 16 : Tick_Generator = 0x20; break;
      default :
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
         cout << "Init_Proc::configure_suim_2 - bad Tick_Generator" << endl;
	 return false;
   }
   
   tick_gen = Tick_Generator;
      
   int_16 SUIM_config_array[64];

   // Mask for 1 ot 2 bit operation;
   int_16 bitmode = 0x000;
   if (num_bits==2) {
     bitmode = 0x400;
   }
   
   // Allow 1/2 bit configuration to be overwritten for system testing
   if (force_bits>0) {
     if (force_bits == 2) {
       bitmode = 0x400;
       cerr << "WARNING: Forcing 2 bit suim configuration" << endl;
     } else if (force_bits == 1) {
       bitmode = 0x000;
       cerr << "WARNING: Forcing 1 bit suim configuration" << endl;
     }
   }

   for(int i = 0; i < 64; ++i)
   {
      SUIM_config_array[i] = 0;
   }
   
   ///980917fmo start: more output
     SUIM_config_array[62] = 1;
     SUIM_config_array[63] = 1;
   ///980917fmo end: more output
   
   //are all 0 - flushes suim
      if((cmd = this->form_command(SU_Station_Unit_Cmds::Configure_SUIM)) &&
      (table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete (cmd->remove_table(SU_Table_Types::SUIM_Settings));
	 sleep(1);
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      } 
   }	
   else
   {
      delete gen_tab1;
      gen_tab1 = 0;
      delete table1;
      table1 = 0;
      delete cmd;
      cmd = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   
      ///980917fmo start: insert extra message
     SUIM_config_array[0] = 0x5b1;
     SUIM_config_array[1] = 1;
     if((cmd = this->form_command(SU_Station_Unit_Cmds::Configure_SUIM)) &&
	(table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
	(gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
	cmd->add_table(gen_tab1))
       {
	 if(this->transmit_to_SU(cmd))
	   {
	     delete (cmd->remove_table(SU_Table_Types::SUIM_Settings));
	     sleep(1);
	   }
	 else
	   {
	     delete cmd;
	     cmd = 0;
	     this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	     return false;
	   } 
       }	
     else
       {
	 delete gen_tab1;
	 gen_tab1 = 0;
	 delete table1;
	 table1 = 0;
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	 return false;
       }
     ///980917fmo end: insert extra message
   
      //SUIM_config_array[0] = 0x251;
      //SUIM_config_array[0] = 0x461; // New value 9/7/98
      SUIM_config_array[0] = 0x5b1; // New value 13/7/98
      SUIM_config_array[1] = 1;
      SUIM_config_array[2] = Tick_Generator;
      //SUIM_config_array[3] = 3;
      SUIM_config_array[3] = 2; // Sergey value 26/6/98
      SUIM_config_array[4] = 0;
      SUIM_config_array[5] = 0x1C | bitmode;
      SUIM_config_array[6] = 0x1C | bitmode;
      SUIM_config_array[7] = 0x1C | bitmode;
      SUIM_config_array[8] = 0x1C | bitmode;
      
   
	    
   if((table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete (cmd->remove_table(SU_Table_Types::SUIM_Settings));
	 sleep(1);
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      } 
   }	
   else
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   
   SUIM_config_array[5] = 0x94 | bitmode;
   SUIM_config_array[6] = 0x94 | bitmode;
   SUIM_config_array[7] = 0x94 | bitmode;
   SUIM_config_array[8] = 0x94 | bitmode;
   
   if((table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete (cmd->remove_table(SU_Table_Types::SUIM_Settings));
	 sleep(1);
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      } 
   }	
   else
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   
   SUIM_config_array[5] = 0x9C | bitmode;
   SUIM_config_array[6] = 0x9C | bitmode;
   SUIM_config_array[7] = 0x9C | bitmode;
   SUIM_config_array[8] = 0x9C | bitmode;
   
   if((table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete (cmd->remove_table(SU_Table_Types::SUIM_Settings));
	 sleep(1);
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      } 
   }	
   else
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   
   SUIM_config_array[5] = 0x1C | bitmode | VALMAG;
   SUIM_config_array[6] = 0x1C | bitmode | VALMAG;
   SUIM_config_array[7] = 0x1C | bitmode | VALMAG;
   SUIM_config_array[8] = 0x1C | bitmode | VALMAG;
   
   // Pulsar gating hack !
   if (external_suim_pg) {
     for (int i=0; i<17; i++) {
       SUIM_config_array[i+12] = suim_pg[i];
     }
   }

   if((table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      } 
   }	
   else
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
}

//_____________________________________________________________________
bool Init_Proc::flush_suim(void)
{
   // Flush the SUIM
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   int_16 SUIM_config_array[64];
   
   for(int i = 0; i < 64; ++i)
   {
      SUIM_config_array[i] = 0;
   }
   
   //are all 0 - flushes suim
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Configure_SUIM)) &&
      (table1 = new SUIM_config_cmd_Table(SUIM_config_array)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::SUIM_Settings, table1, 0)) &&
      cmd->add_table(gen_tab1))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 sleep(1);
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   } 
   else 
   {
       delete gen_tab1;
       delete table1;
       delete cmd;
       gen_tab1 = 0;
       table1 = 0;
       cmd = 0;
       this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
       return false;
   }
}

//__________________________________________________________________________________________-
bool Init_Proc::get_dpu_footage(ieee_dbl repeat_interval)
{
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
      
   //send req footage cmd
   char command[1][dpu_cmd_len] = {"R 30\r\n\0\0\0\0\0\0\0\0\0"};
      
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::DPU_Command)) ||
      !(table1 = new DPU_Cmd_Table(1, command)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) ||
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(repeat_interval != 0)
      {
         cmd->set_rpt_intrvl(repeat_interval * SYSTEM_RATE);
	 cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set);
      }
         
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 this->do_abort();
         this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
         return false;
      }
   }
   
}

//__________________________________________________________________________________________-

Command_Message *Init_Proc::form_command(unsigned int command_code)
{
   Command_Message *cmd;

   
    if(cmd = new Command_Message(command_code)) 
    {
     this->set_local_time();
       
     cmd->set_local_time(local_time);  
     cmd->set_time_stamp(time_stamp); 
     cmd->set_sender_id(SU_CONTROL_ID);
     cmd->set_receiver_id(su_conn->get_SUID());
     return cmd;
    }
    else return 0;
}

//____________________________________________________________________________

bool Init_Proc::read_bar_code(void)
{
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   /*
   
   char cmds[2][dpu_cmd_len] = {{"W B3 0001\r\n\0\0\0\0"},
                                {"W BE 0001\r\n\0\0\0\0"}};
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::DPU_Command)) ||
      !(table1 = new DPU_Cmd_Table(2, cmds)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) ||
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      return false;
   }
   else
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
      }
      else
      {
	 delete cmd;
	 return false;
      }
   }
   */
   
   char command[6][dpu_cmd_len] = {{"R 35\r\n\0\0\0\0\0\0\0\0\0"},
      				   {"R 36\r\n\0\0\0\0\0\0\0\0\0"},
				   {"R 37\r\n\0\0\0\0\0\0\0\0\0"},
				   {"R 38\r\n\0\0\0\0\0\0\0\0\0"},
				   {"R 39\r\n\0\0\0\0\0\0\0\0\0"},
				   {"R 3A\r\n\0\0\0\0\0\0\0\0\0"}};
      
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::DPU_Command)) ||
      !(table1 = new DPU_Cmd_Table(6, command)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) ||
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      return false;
   }
   else
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 return false;
      }
   }
}
   
//______________________________________________________________________________________

bool Init_Proc::do_tape_dismounted(void)
{
   if(this->in_response_queue(SU_Action_Status::Medium_Dismounted))
   {	 
      su_status->set_medium_mounted_status(DISMOUNTED);
      su_status->set_medium_bar_code("");
      //form abort su ops cmd and transmit to su
      this->do_abort();
      allocated = false;
      //inform medium handlers
      this->do_exit_to_mounted(SU_Control_Error_Codes::Medium_Dismounted_Exception);
      #ifdef DEBUG 
	       cout << "Received Tape Dismounted. Returning to Idle state" <<endl;
      #endif
      
      //change state to idle
      current_state = idle;
      //inform su_monitor
      this->transmit_tape_status();
      return true;
   }
   else return false;
}

//___________________________________________________________________________
   
bool Init_Proc::request_tape_mounted_status(void)
{  
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   
   char command[2][dpu_cmd_len] = {{"W B3 0001\r\n\0\0\0\0"},
                                   {"W BE 0001\r\n\0\0\0\0"}};
      
      if(!(cmd = this->form_command(SU_Station_Unit_Cmds::DPU_Command)) ||
	 !(table1 = new DPU_Cmd_Table(2, command)) ||
	 !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) ||
	 !(cmd->add_table(gen_tab1)))
      {
	 delete gen_tab1;
	 delete table1;
	 delete cmd;
	 gen_tab1 = 0;
	 table1 = 0;
	 cmd = 0;
	 return false;
      }
      else
      {
	 if(this->transmit_to_SU(cmd))
	 {
	    delete cmd;
	    cmd = 0;
	    //return true;
	 }
	 else
	 {
	    delete cmd;
	    cmd = 0;
	    return false;
	 }
      }
      
      sleep(5);
      
      char commands[1][dpu_cmd_len] = {"R 73\r\n\0\0\0\0\0\0\0\0\0"};
      
      if(!(cmd = this->form_command(SU_Station_Unit_Cmds::DPU_Command)) ||
	 !(table1 = new DPU_Cmd_Table(1, commands)) ||
	 !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) ||
	 !(cmd->add_table(gen_tab1)))
      {
	 delete gen_tab1;
	 delete table1;
	 delete cmd;
	 gen_tab1 = 0;
	 table1 = 0;
	 cmd = 0;
	 return false;
      }
      else
      {
	 if(this->transmit_to_SU(cmd))
	 {
	    delete cmd;
	    cmd = 0;
	    return true;
	 }
	 else
	 {
	    delete cmd;
	    cmd = 0;
	    return false;
	 }
      }
}
//______________________________________________________________________________

bool Init_Proc::do_abort(void)
{
   sending_dummy_data = false;
   this->do_servoing(false, -1.0);//disable servoing bit
   this->send_flush_su();//flush cmd queue
  if (cjd)
   this->send_set_ROT(cjd->get_obs_TOT_end().get_sysclks());
   //form su_abort_ops cmd and send to su, exit to wait_for_abort
   if (eVLBI)
     {
       if(disk_unit.query_e_station_play_on()) disk_unit.stop_e_station_play();
       if(disk_unit.query_in2net_on()) {
	 disk_unit.in2net("off");
       }
       if(disk_unit.query_net2out_on()) disk_unit.net2out("close");
       disk_unit.in2net("disconnect");
     }
   else 
     {
       disk_unit.stop_play();
     }
   Command_Message *cmd = 0;
  
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Abort_SU_Ops)) && this->transmit_to_SU(cmd))
   {
#if DEBUG
      cout << "Sent Abort_SU_ops" << endl;
#endif
      delete cmd;
      cmd = 0;
      current_state = wait_for_abort;
      return true;
   }
   else
   {
      delete cmd;
      cmd = 0;
      current_state = wait_for_abort;
      return false;
   }
   
}

//________________________________________________________________________________

bool Init_Proc::check_timeout(float period)
{
   //return false;//temp!
   
 if(!(timer.running()))
 {
    timer.start_timer();
    return false;
 }
 else
 {
  //determine if timeout period has elapsed
  if(timer.elapsed_time(timer_key) > period) return true;
  else return false;
 }
}

//________________________________________________________________

void Init_Proc::exit_process(unsigned int error_code, unsigned int command_code)
{
   this->transmit_error(error_code, command_code);
   su_cmd->quit();
   su_conn->quit();
   this->quit();
   get_controller()->shutdown();
   //and exits
}

//____________________________________________________________________________

void Init_Proc::SU_finished(int_32 why)
{
   Action_Status *msg;
   
   if (use_mark5) 
     {
       if (!eVLBI) disk_unit.stop_play();
       else
	 {
	   disk_unit.in2net("off");
	   closeEVLBIout(false);
	   disk_unit.in2net("disconnect");
	 }
     }
   else 
     {
       this->start_dpu(false);//stop dpu
       this->do_servoing(false, -1.0);//disable CCC control of servoing
     }
   
   //form and send success action status to medium handlers
   if(msg = new SU_Control_Finished(the_suid_code, why))
   {
      msg->set_sender_id(SU_CONTROL_ID);
      msg->set_action_code(SU_Action_Values::Unsolicited);
      this->set_local_time();
      msg->set_local_time(local_time);
      msg->set_time_stamp(time_stamp);
      //transmit msg to all medium handlers
      su_cmd->transmit_medium_handlers(msg);
      delete msg;
      msg = 0;
   }
   su_cmd->close_medium_handler_conns();
   su_cmd->close_data_handler_conn();
   //check if peaking thread is running
   if(peaking_thread)
   {
      get_controller()->remove_thread(peaking_thread);               // Remove thread completely
      peaking_thread = 0;
   }
   current_state = mounted;
   allocated = false;
   if (use_mark5) this->transmit_disk_status(SU_Control_SU_Status_Codes::Finished);
   else this->transmit_tape_status(SU_Control_SU_Status_Codes::Finished);
   this->request_su_status();

   if (why == CCC_Finish_Codes::Normal_Finish) {
     cout << "SU_finished(): will send data until sub job has finished" << endl;
     sending_dummy_data = true;
   }
   else if (why == CCC_Finish_Codes::Finish_Clk_Msg) {
     cout << "SU_finished(): will reset immediately when in mounted" << endl;
   }
   else {
     cout << "SU_finished(): unknown why in SU_finished" << endl;
   }
}//end method

//_____________________________________________________________________________


unsigned int Init_Proc::get_bcd_rep(int value)const
{
   unsigned int result = 0;
   
   if(value/10 >= 1)
   {
      result |= this->get_bcd_rep(value/10)<<4;
      result |= value%10;
   }
   else
   {
      result |= value%10;
   }
   
   return result;
   
}

//__________________________________________________________________

unsigned int Init_Proc::rev_bcd(unsigned int value)const
{
   unsigned int result = 0;
   int i;
   
   for(i = 0; i < 4; ++i)
   {
      if(value&(1<<i))
      {
	 result |= 1<<(3 - i);
      }
   }
   
   return result;
}

//__________________________________________________________________________

unsigned int Init_Proc::vlba_group_track_rep(int track)const
{

   //group 0 constitute actual track Nos 3,5,7,9,11,13,15,17. (designated tracks 0 - 7 respectively)
	    
   //group 1 constitutes actual track No's 19,21,23,25,27,29,31,33 (designated 0 - 7 respectively)
	    
   //group 2				    2,4,6,8,10,12,14,16, (0 - 7 respectively)
	    
   //group 3				    18,20,22,24,26,28,30,32 (0 - 7 respectively)
	    
   //group 4				     0,1,34,35 (designated 2,0,3,1 respectively)
   //normally the group 4 'system' tracks are used for cross track parity but conceivably could be
   //used as spare data tracks -> therefore will assume they could be data tracks


   unsigned int the_bit_pattern = 0;
   
   switch (track) {
      case (0) :
	 //track and group No's
	 the_bit_pattern = 0x42<<24;//tr 2 = 0100, gp 4 = 0010
	 break;
      case (1) :
	 //track and group No's
	 the_bit_pattern = 0x02<<24;//tr 0, gp 4
	 break;
      case (34) :
	 //track and group No's
	 the_bit_pattern = 0xc2<<24;//tr 3 = 1100, gp 4
	 break;
      case (35) :
	 //track and group No's
	 the_bit_pattern = 0x82<<24;//tr 1 = 1000, gp 4
	 break;
      default :
	 if(track%2 == 0)
	 {
	    //even no tracks
	    //track and group No's
	    if(track < 18)
	    {
	       the_bit_pattern |= (this->rev_bcd(this->get_bcd_rep(track/2 -1)))<<28;//track
	       the_bit_pattern |= 0x4<<24;//gp 2
	    }
	    else 
	    {
	       the_bit_pattern |= (this->rev_bcd(this->get_bcd_rep((track - 16)/2 -1)))<<28;//track
	       the_bit_pattern |= 0xc<<24;//gp 3
	    }
	 }
	 else
	 {
	    //odd no tracks
	    //track and group No's
	    if(track < 19)
	    {
	       the_bit_pattern |= (this->rev_bcd(this->get_bcd_rep(track/2 -1)))<<28;//track
	       //gp 0
	    }
	    else 
	    {
	       the_bit_pattern |= (this->rev_bcd(this->get_bcd_rep((track - 16)/2 -1)))<<28;//track;
	       the_bit_pattern |= 0x8<<24;//gp 1
	    }
	 }
	 break;
   }//end switch
		
   return the_bit_pattern;
}

//_____________________________________________________

void Init_Proc::make_tab(int tones[4], int pcmtab[4][3200])const
     /*
      *
      * A minor difficulty is that data on tape and
      * in the SU have different conventions! On tape,
      * Mark IV Memo #133 applies: sign is inverted
      * and magnitude is inverted if sign is 0. This
      * convention is applicable to 2-bit data but not
      * with 1-bit data: the SU is designed to force the
      * magnitude bit to 1. However, for external
      * reasons, we have decided not to tell the SU
      * that it is handling 1-bit data and instead
      * make the magnitude a copy of the sign bit by
      * routing it in the CRM  cross-bar switch.
      *
      * sctab[8][15] is the sin/cos mult table. It is based
      * on the tables in Mark IV Memo #133 except for 3
      * things. Sines appear in the first 12 rows with the 
      * cosines offset by 3 to the right. The first 4
      * rows are for 2-bit use and the last 4 rows for
      * 1-bit use. The SU PCM configure message is not well
      * in tune with Memo #133. The message indexes the
      * RAM tables using magnitude-sign rather than the
      * other way round! This is handled by swapping
      * around the second and third rows in the table.
      *
      * The PCM accesses data in RAM using VMS in that
      * order as address bits. The first half of the
      * RAM is cleared to ensure non-valid data and
      * The second half of the RAM is loaded in order MS.
      * The bits in a RAM byte are:
      *
      *               b7   b6   b5   b4   b3   b2   b1   b0
      * 2-bit data   t1sM t1sL t1cM t1cL t0sM t0sL t0cM t0cL
      * 1-bit data   t3sL t3cL t2sL t2cL t1sL t1cL t0sL t0cL
      *
      * where, in tXCY, X = tone (0, 1 or 0, 1, 2, 3)
      *                 C = c(os) | s(in)  
      *                 Y = counter (LSB | MSB))
      *
      */
{
  int       sctab[8][15] = {{2,3,3,3,3,2,1,0,0,0,0,1,2,3,3},
			    {1,1,1,1,1,1,2,2,2,2,2,2,1,1,1},
			    {1,0,0,0,0,1,2,3,3,3,3,2,1,0,0},
			    {2,2,2,2,2,2,1,1,1,1,1,1,2,2,2},
			    {1,1,1,1,1,1,0,0,0,0,0,0,1,1,1},
			    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
			    {0,0,0,0,0,0,1,1,1,1,1,1,0,0,0},
			    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}};
  int       M, IX[4], addr, i, ntones, tone;
  int       offset, col;
  int       dsm, ram_data;

  switch (num_bits) {
  case 1: 
    ntones = 4;
    offset = 4;             /* into sctab */
    break;
  case 2:
    ntones = 2;
    offset = 0;
    break;
  }
  M = (int)(SYSTEM_RATE/fs);
  for (i = 0; i < ntones; i++) {
    IX[i] = (tones[i]/10000)*M*3;
  }

  for (addr = 0; addr < 3200; addr++) {          /* RAM address */
    for (dsm = 0; dsm < 4; dsm++) {              /* data MS */
      ram_data = 0;
      for (tone = ntones-1; tone >= 0; tone--) { /* tone */
	col = ((addr*IX[tone])/800) % 12;
	ram_data <<= num_bits;
	ram_data += sctab[dsm+offset][col];      /* sine */
	ram_data <<= num_bits;
	ram_data += sctab[dsm+offset][col+3];    /* cosine */
      }
      pcmtab[dsm][addr] = ram_data;
    }
  }
}

//_____________________________________________________

unsigned short int Init_Proc::get_mux_demux(int bits)
{
   unsigned short int result = 0;
   
   ieee_dbl fan_fact = current_scan->get_record_format()->get_fan_out_factor() * 4;
   
   if (fan_fact > 8) result = 0;
   else if (fan_fact > 4) result = 1;
   else if (fan_fact > 2) result = 2;
   else if (fan_fact > 1) result = 3;
   else if (fan_fact > 0) result = 4;

   if (bits==1) result += 8;
   
   return result<<4;
}

//_________________________________________________________________________________________

int Init_Proc::get_f(float TC, float CC)const
     /* 
      * Calculates the hex digit f. 
      * See Module Software addresses page 15, Table 5.4.1
      *
      * Expect TC to take values 16.0, 8.0 or 4.0 MHz 
      * and    CC to take values 32.0, 16.0, 8.0, 4.0, 2.0, 1.0
      * MHz.
      * TC is converted to 3, 2 or 1
      * CC is converted to 6, 5, 4, 3, 2 or 1 respectively
      * and then combined as 2 hex digits with TC above CC
      *
      * The index f is then looked up in table[] and returned.
      */
{
  int     f, CCI, TCI, table[16] = { 0x00, 0x15, 0x26, 0x00,
				       0x14, 0x25, 0x36, 0x35,
				       0x13, 0x24, 0x12, 0x23,
				       0x34, 0x11, 0x22, 0x33};
  TCI = (int)(TC/4.0 + 0.1);
  if (TCI > 3) 
    TCI = 3;
  CCI = (int)(CC + 0.1);

  switch (CCI) {
  case 32: CCI = 6; break;
  case 16: CCI = 5; break;
  case 8: CCI = 4; break;
  case 4: CCI = 3; break;
  case 2: CCI = 2; break;
  case 1: CCI = 1; break;
  }
  
  CCI += (TCI << 4);
  for (f = 0; f < 16; ++f)
  {
    if (table[f] == CCI)
      return f;
  }
  return 9999;
}

//___________________________________________________________________

bool Init_Proc::set_current_scan(void)
{
  Forward_List_Iterator *station_iter = 0;
  Forward_List_Iterator *medium_iter = 0;
  current_station = 0;
  current_medium = 0;
  string bar_code = disk_unit.du_status.get_VSN_active();

  if((station_iter = new Forward_List_Iterator(cjd->get_logical_stations())))
    {
      if(station_iter->reset() && station_iter->current().isType() == LOGICAL_STATION_ID)
	{
	  do
	    {
	      current_station = &((Logical_Station &) station_iter->current());
	      int   nrecmedia = current_station->get_recorded_media()->count();
	      if( nrecmedia<=0 )
		{
		  continue;
		}
	      
	      if((medium_iter = new Forward_List_Iterator(current_station->get_recorded_media())))
		{
		  if(medium_iter->reset())//&& medium_iter->current().isType() == MEDIUM_ID)
		    {
		      string buf = ((Medium &)medium_iter->current()).get_medium_ID();
		      
		      while(strcmp_mediumid(bar_code.c_str(), buf.c_str()) && medium_iter->next())
			{
			  buf = ((Medium &)medium_iter->current()).get_medium_ID();
			}
		      
		      if(!strcmp_mediumid(bar_code.c_str(), buf.c_str()))
			{
			  scans = ((Medium &) medium_iter->current()).get_scan_list();
			  current_station = &((Logical_Station &) station_iter->current());
			  current_medium = &((Medium &) medium_iter->current());
			  break;
			}
		      else
			{
			  delete medium_iter;
			  medium_iter = 0;
			}
		    }
		  else
		    {
		      //something wrong!
		      cout << "Init_Proc::set_current_scan() did not get_recorded_media()" << endl;
		      this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
		      return false;
		    }
		}
	      else
		{
		  //memory failure
		  this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
		  cout << "Init_Proc::set_current_scan() memory failure" << endl;
		  return false;
		}
	    }
	  while(station_iter->next());
	  
	  delete station_iter; station_iter = 0;
	  delete medium_iter; medium_iter = 0;
	  cout << "Init_Proc::set_current_scan() current_station " << current_station->get_station_ID() << endl;
	  cout << "Init_Proc::set_current_scan() current_medium " << current_medium->get_medium_ID()  << endl;
	  if(current_station == 0 || current_medium == 0)//problem anywhere in above do statement
	    {
	      cout << "Init_Proc::set_current_scan() finds no current station or medium." << endl;
	      this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	      return false;
	    }
	  else
	    {
	      scans->iterator().reset();//point to first scan
	      current_scan = &((Scan &) scans->iterator().current());
	      
	      //find first scan which could be part of the sub_job
	      
	      while(current_scan->get_stop_TOT() <= sub_job_start_ROT && scans->iterator().next())
		{
		  current_scan = &((Scan &) scans->iterator().current());
		}
	      
	      if(current_scan->get_stop_TOT() <= sub_job_start_ROT)
		{
		  //something wrong
		  cout << "Init_Proc::set_current_scan() finds scan stop_TOT <= subjob start_ROT" << endl;
		  this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
		  return false;
		}
	      
	      cout << "current scan start = " << current_scan->get_start_TOT() << "     " 
		   << Short_Time_Value(current_scan->get_start_TOT()).get_rot_string() 
		   << endl;
	      cout << "current scan  stop = " << current_scan->get_stop_TOT() << "     " 
		   << Short_Time_Value(current_scan->get_stop_TOT()).get_rot_string() 
		   << endl;
	      cout << "sub job start = " << sub_job_start_ROT << "           " 
		   << Short_Time_Value(sub_job_start_ROT).get_rot_string() << endl;
	      cout << "sub job  stop = " << sub_job_ending_ROT 
		   << "           " 
		   << Short_Time_Value(sub_job_ending_ROT).get_rot_string() << endl;
	      
	      // Also record subjob time in peaking log
	      peakFile.open(peakFilename, ios::app);
	      if (peakFile) 
		{
		  peakFile << "Subjob: " << 
		    Short_Time_Value(sub_job_start_ROT).get_rot_string() << 
		    "  -  " << 
		    Short_Time_Value(sub_job_ending_ROT).get_rot_string() << 
		    endl;
		  peakFile.close();
		}
	      
	      //OK - now we want to work out if the current or next scan is required
	      //if the current scan is completely within this sub_job then we want the
	      //current scan
	      
	      if(current_scan->get_start_TOT() >= sub_job_start_ROT &&
		 current_scan->get_stop_TOT() <= sub_job_ending_ROT)
		{
		  cout << "current scan completely within sub job" << endl;
		  return true;
		}
	      
	      //if the current scan end at or after sub job then we want current scan
	      
	      if(current_scan->get_stop_TOT() >= sub_job_ending_ROT)
		{
		  cout << "current scan stops after sub job" << endl;
		  return true;
		}
	      
	      //what happens if the current scan starts before sub_job_start and ends before the sub_job_stop?
	      //if whats within the sub_job <= MIN_SUB_JOB_DUR*SYSTEM_RATE then want next_scan else current
	      
	      if(current_scan->get_stop_TOT() - sub_job_start_ROT > MIN_SUB_JOB_DUR*SYSTEM_RATE)
		{
		  cout << "current_scan  > MIN_SUB_JOB_DUR" << endl;
		  return true;
		}
	      else
		{
		  if(scans->iterator().next())
		    {
		      current_scan = &((Scan &) scans->iterator().current());
		      cout <<  "current_scan  <= MIN_SUB_JOB_DUR - use next scan" << endl;
		      return true;
		    }
		  else
		    {
		      cout <<  "current_scan  <= MIN_SUB_JOB_DUR - no next scan" << endl;
		      return false;
		    }
		}
	      
	      Scan *next_scan;
	      if(scans->iterator().next())
		{
		  next_scan = &((Scan &) scans->iterator().current());
		}
	      else
		{
		  return true;
		}
	      
	      ieee_dbl current_dur = 0, next_dur = 0;
	      
	      current_dur = current_scan->get_stop_TOT() - sub_job_start_ROT;
	      
	      next_dur = sub_job_ending_ROT - next_scan->get_start_TOT();
	      
	      cout << "current scan duration = " << current_dur << endl;
	      cout << "next scan duration = " << next_dur << endl;
	      return scans->iterator().previous();
	      
	      
	      if(next_dur > current_dur)
		{
		  //then we want the next scan
		  current_scan = next_scan;
		  cout << "next scan forms larger part of sub job" << endl;
		  return true;
		}
	      else
		{
		  //then we want the current_scan
		  cout << "current scan forms larger part of sub job" << endl;
		  return scans->iterator().previous();
		}
	    }
	}
      else
	{
	  //something wrong
	  cout << "Init_Proc::set_current_scan() finds logical station without LOGICAL_STATION_ID" << endl;
	  this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	  return false;
	}
    }
  else
    {
      //memory failure
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
    }
}

//___________________________________________________________________________

bool Init_Proc::config_dpu(const int head_nr)
{
  cout << "into config_dpu" << endl;
   int hd_init_pos = (int)(current_scan->get_headgroup_offset(head_nr)) * 10;//0.1 micron units
   
   //get_headgroup_offset returns a double (for some reason) so convert to signed int
   
   const int dpu_cmd_len = 16;
   char speed[dpu_cmd_len];
   char head_1[dpu_cmd_len] = {"W 94 0000\r\n\0\0\0\0"};
   char head_2[dpu_cmd_len] = {"W 96 0000\r\n\0\0\0\0"};
   char output[dpu_cmd_len];
   char head_posn[dpu_cmd_len];
   char the_commands[5][dpu_cmd_len];
   int i;
   char hex_head_pos[5] = "0000";
   
   sprintf(speed, "W B5 %.4lX\r\n\0\0\0\0", the_nominal_tape_speed*100);
   
   //if((current_medium->get_thickness() == thin) && the_nominal_tape_speed != 1350)
   
   if(the_nominal_tape_speed >= 160)
   {
      head_1[8] = '1';
      head_2[8] = '1';
   }
   
   if(the_nominal_tape_speed >= 320)
   {
      head_1[8] = '2';
      head_2[8] = '2';
   }
   
   if(hd_init_pos < 0)
   {
      char temp[10];
      sprintf(temp, "%.4lX", hd_init_pos);
      int len = strlen(temp) -4; 
      char *ptr = &(temp[len]);
      strncpy(hex_head_pos, ptr, 4);
   }
   else sprintf(hex_head_pos, "%.4lX", hd_init_pos);
   
   strcpy(head_posn, "W C6 ");
   strcat(head_posn, hex_head_pos);
   strcat(head_posn, "\r\n\0\0\0\0");
   
   int best_track=0;    // The track closest to the best track
	
	
   cout << "init_proc_p: best_track= " << best_track << endl;
	
	if (head_nr==1) 
	  {
		 int t;
		 cout << "Init_Proc::config_dpu() - INFO Head 1:" << endl;
		 for(i = 0; i < NUM_TRACKS/2; i++) 
			{
			  t = dim_x_bar_configuration[i]; // Save typing
			  cout << "     i,t = " << i << ", " << t << endl;
			  if(t != 80 && abs(t-HEAD1_IDEAL_TRACK) 
				  < abs(best_track - HEAD1_IDEAL_TRACK)) 
				 {
					best_track = t;
				 }
			}
		 head1_best_track = best_track;
		 cout << "init_proc_p: head1_best_track= " << head1_best_track << endl;
	  } 
	else 
	  { // Head 2
		 int t;
		 cout << "Init_Proc::config_dpu() - INFO Head 2:" << endl;
		 for(i = NUM_TRACKS/2; i < NUM_TRACKS; i++) 
			{
			  t = dim_x_bar_configuration[i]; // Save typing
			  cout << "     i,t = " << i << ", " << t << endl;
			  if(t != 80 && abs(t-HEAD2_IDEAL_TRACK) 
				  < abs(best_track - HEAD2_IDEAL_TRACK)) 
				 {
					best_track = t;
				 }
			}
		 head2_best_track = best_track;
		 cout << "init_proc_p: head2_best_track= " << head2_best_track << endl;
	  }

	
	
	
	
	if (best_track==0) 
	  {
		 cout << "Didn't find ANY good tracks" << endl;
		 this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
		 return false;  
	  }
   cout << "Reference track set to "<< best_track << endl;
	
   if(head_nr == 1 )//head 1 in reality 
	  {
		 sprintf(output, "W 90 %.4lX\r\n\0\0\0\0",best_track);
	  }
   else
	  {
		 //head_nr == 2
		 sprintf(output, "W 92 %.4lX\r\n\0\0\0\0",best_track);
	  }
   
   for(i = 0; i < dpu_cmd_len; ++i)
	  {
		 the_commands[0][i] = head_1[i];
		 the_commands[1][i] = head_2[i];
		 the_commands[2][i] = output[i];
		 the_commands[3][i] = head_posn[i];
	  }
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(4, the_commands)) ||//changed from 5
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
	  {
		 delete gen_tab1;
		 delete table1;
		 delete cmd;
		 gen_tab1 = 0;
		 table1 = 0;
		 cmd = 0;
		 this->do_abort();
		 this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
		 return false;
	  }
   else
	  {
		 if(this->transmit_to_SU(cmd))
			{
			  delete cmd;
			  cmd = 0;
			}
		 else
			{
			  delete cmd;
			  cmd = 0;
			  this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
			  return false;
			}
	  }
   return true;
}


bool Init_Proc::start_dpu(bool value, ieee_dbl obey_ROT)
{
   const int dpu_cmd_len = 16;
   char dirn[dpu_cmd_len] = {"W B1 0001\r\n\0\0\0\0"};//start
   char the_commands[1][dpu_cmd_len];
   int i;
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(!value)
   {
      dirn[8] = '0';
      dirn[3] = '0';//stop
   }

   
   if(current_scan->get_tape_direction() != forward) dirn[8] = '0';//reverse
	   
   for(i = 0; i < dpu_cmd_len; ++i)
   {
      the_commands[0][i] = dirn[i];
   }
	   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(1, the_commands)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(obey_ROT>0.0)
      {
	 cmd->set_obey_ROT(obey_ROT);
	 cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
      }
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   return true;
}

bool Init_Proc::set_dpuspeed(ieee_dbl obey_ROT)
{
   const int dpu_cmd_len = 16;
   char speed[dpu_cmd_len];
   char the_commands[1][dpu_cmd_len];
   int i;
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   sprintf(speed, "W B5 %.4lX\r\n\0\0\0\0", the_nominal_tape_speed*100);
	   
   for(i = 0; i < dpu_cmd_len; ++i)
   {
      the_commands[0][i] = speed[i];
   }
	   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(1, the_commands)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      gen_tab1 = 0;
      table1 = 0;
      cmd = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(obey_ROT>0.0)
      {
	 cmd->set_obey_ROT(obey_ROT);
	 cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
      }
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   return true;
}


//______________________________________________________________

bool Init_Proc::check_alarm(void)
{
   if (!cjd) return true;
   bool ret = true;
   Command_Message *cmd = 0;//outbound

   Alarm_Clock_Message *acm = 0;
   Clock_Message_Entry *entry = 0;
	    
   //check it's for us and to load next set of polys - remove msg from queue
   acm = (Alarm_Clock_Message *) cmd_iter->contents()->get_msg();
   delete cmd_queue.remove(cmd_iter);//msg is not deleted!
   
   //iterate through its entries
   entry = acm->first_entry();
   while (entry) {
     cout << "checking alarm, found entry " << entry->get_SU_array_code() << " parameter " << entry->get_parameter() << endl;
     if((entry->get_SU_array_code() == cjd->get_corr_job_ID()))
       {
	 if (entry->get_parameter() == load_polys_flag) 
	   {
	     cout << "== load polys flag" << endl;
	     if(!(cmd = form_command(SU_Station_Unit_Cmds::Load_Polynomials)))
	       {
		 this->do_abort();
		 this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
		 ret = false;
	       }
	     else 
	       {
		 cmd->set_obey_ROT(poly_obey_ROT);
		 cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
	       }
	     //send next set of polys starting with last sent as overlap
	     if(!this->send_load_poly_cmd(cmd, end_ROT))//send third load poly cms timed to be obeyed at obey_rot
	       {
		 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
		 ret = false;
	       }
	     else 
	       {
		 //need another alarm message?
		 if(end_ROT < sub_job_ending_ROT)
		   {
		     // load_polys_flag = rand(); BE now use "load_polys_flag = ROT_Clock_Alarm_Codes + the_suid_code;" since this has been set, no need to set it again
		     if(!this->send_alarm_request(load_polys_flag, begin_ROT))
		       {
			 ret = false;
		       }
		   }//end if alarm req reqd
	       }
	       delete cmd, cmd = 0;
	   } // end if load_polys
       }//end if alarm msg for us
     entry = acm->next_entry(entry);
   } // while entry

   delete acm; acm = 0;
   return ret;
}

//_______________________________________________________________________________

bool Init_Proc::do_tape_stats(int &lock_resyncs)
{
   Action_Status *msg = 0;

   int i;//counter
   
   int_32 head_positions[4];
	       
   AvStats *avsmsg = (AvStats *)response_iter->contents()->get_msg();
   delete response_queue.remove(response_iter);//msg is not deleted!
   
   //write msg to file
   outFile.open(outFilename, ios::app);
   if(outFile)
   {
      debug_log_flag = true;
      avsmsg->print(outFile);
      debug_log_flag = false;
      outFile.close();
   }
   
   //maintain su_state
   su_status->set_SU_state(avsmsg->get_su_state());
	       
   //check if a serial link is down
   //the SUIM state (int_32) is a bit representation of the state of the SUIM
   //bit 31 if set indicates a serial link problem
	       
   if(avsmsg->get_suim_state()&(1<<31))
   {
      if(++lock_resyncs < ALLOWED_LOCK_RESYNCS)
      {
	 if(error_msg.set_link_receiver_id(0,SU_Control_Error_Codes::Receiver_Out_Of_Lock))
	 {
	    //send error msg to PROCESSOR_CONTROL
	    this->transmit_error(SU_Control_Error_Codes::Receiver_Out_Of_Lock);
	 }
	 else
	 {
	    cerr << "Link_Receiver_Out_Of_Lock\n";
	    this->do_abort();
	    this->do_exit_to_mounted(SU_Control_Error_Codes::Receiver_Out_Of_Lock);
	    return false;
	 }
      }
      else
      {
	 this->do_abort();
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Receiver_Out_Of_Lock);
	 return false;
      }
   }
	       
	       
   //record posn of headstacks in su_status
   head_positions[0] = avsmsg->get_hd0pos();
   head_positions[1] = avsmsg->get_hd1pos();
   head_positions[2] = avsmsg->get_hd2pos();
   head_positions[3] = avsmsg->get_hd3pos();
   su_status->set_head_positions(head_positions);
   num_head_stacks = (unsigned int)avsmsg->get_num_hdstks();
   
   
   //avsmsg->get_cur_tot() is not within the sub_job request a medium_frame header to verify tot
   // or tot is not incrementing (set to -1 at start of job)
   Short_Time_Value current_tot = avsmsg->get_cur_tot();
   if((current_tot <  sub_job_start_ROT || current_tot > sub_job_ending_ROT
       || current_tot <= last_tot)
        && (peaking_thread == 0))
   {
     if (current_tot <= last_tot) 
       cerr<< "Current TOT not incrementing, initiate head peaking" << endl;

      int nom_pos = (int)(current_scan->get_headgroup_offset(0)) * 10;//0.1 micron units
      cout << "Init_Proc_p::do_tape_stats(): nom_pos = " << nom_pos << endl;
      ///050218fmo: assume always: first head for tapes (vex_head = 1, logical_head_nr = 0)
      int logical_head_nr = 0;
      int head_nr = 1;
      if (swap_head) head_nr = 2;
      Peak_Head *temp_obj = new Peak_Head(get_controller()->new_thread_id(), 0, get_controller(), this, 
					  nom_pos, 0, head_nr, logical_head_nr, false, false);
      //ie do not rewind tape and initial_state = check_headers
      if(temp_obj->send_req_tpe_frm_hdr())
      {
	 #ifdef DEBUG 
	        cout << "req_tpe_frm_hdr_cmds sent. Entering check_headers state" <<endl;
	 #endif
	 //add obj to thread master
	 peaking_thread = get_controller()->add_thread(temp_obj);
	 if(peaking_thread == 0)
	 {
	   cerr << "Error getting controller for thread" << endl;
	    this->do_abort();
	    this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	    return false;
	 }
	 this->do_auto_tracking(0, false);
	 this->transmit_tape_status();
      }
   }
   last_tot = current_tot; // Remember for next time around
		
   //maintain current footage attained
   su_status->set_DPU_footage(avsmsg->get_cur_ftage());
   
   char tape_bar_code[bar_code_len];
   su_status->get_medium_bar_code(tape_bar_code);
		
   char* av_stats_track_map = avsmsg->get_active_tracks();
   err_cnt_struct* av_stats_track_errs = avsmsg->get_errsbytrack();

   //the stastics are processed in some TBD way and sent on to all MEDIUM_HANDLERS
   
   //determine if parity error rate is above a threshold for any track
   unsigned int frames, par_errs;
   static bool parity_flag = false;
   
   for(i = 0; i < NUM_TRACKS; ++i)
   {
      if(av_stats_track_map[i] == 1)//ie if active track
      {
	 frames = (unsigned int)av_stats_track_errs[i].get_frames();
	 par_errs = (unsigned int)av_stats_track_errs[i].get_parity_errs();
	 if(par_errs >= (frames * 2500 / PARITY_ERROR_THRESHOLD) && (!parity_flag))
	 {
	    this->transmit_tape_status(SU_Control_SU_Status_Codes::Parity_Errors);
	    parity_flag = true;
	    break;//out of for loop
	 }
      }
   }
   
   if(parity_flag && i == NUM_TRACKS)
   {
      this->transmit_tape_status();
      parity_flag = false;
   }
   
   ieee_dbl servo_error = avsmsg->get_cur_servo_err();
   static bool servo_flag = false;
   
   if(servo_error > SERVO_ERROR_THRESHOLD)
   {
      this->transmit_tape_status(SU_Control_SU_Status_Codes::Servo_Error);
      servo_flag = true;
   }
   else
   {
      if(servo_flag)
      {
         this->transmit_tape_status();
         servo_flag = false;
      }
   }

   cout << "Init_Proc::do_tape_stats: servo_error = " << servo_error << endl;

		
   //They are also sent on to the MEDIUM_HANDLERS and DATA_HANDLER as a SU_Control_Medium_Stats
		
   if(msg = new SU_Control_Medium_Stats(avsmsg->get_su_id(),
				      tape_bar_code,
				      avsmsg->get_num_hdstks(),
				      head_positions[0],
				      head_positions[1],
				      head_positions[2],
				      head_positions[3],
				      avsmsg->get_cur_ftage(),
				      avsmsg->get_cur_tot(),
				      avsmsg->get_cur_servo_err()))
   {
      //also need to supply other parameters to SU_Control_Medium_Stats AS as follows
      char* suc_stats_track_map = ((SU_Control_Medium_Stats *)msg)->get_active_tracks();
      err_cnt_struct* suc_stats_track_errs = ((SU_Control_Medium_Stats *)msg)->get_errsbytrack();
      
      int track_nr = 0;
      
      //need to map tracks(TRM I/P #) to tracks(on tape)!
      for(i = 0; i < NUM_TRACKS; ++i)
      {
	 if(av_stats_track_map[i] == 1)//ie this trm is active
	 {
	    //get the actual track # - its in char dim_x_bar_configuration[i]
	    track_nr = (int) dim_x_bar_configuration[i];
	    //track_nr must be less than HEADS_PER_HEADSTACK * NUM_HEADS_PER_DPU
	    //char the_active_tracks[i] should also == 1
	    if(the_active_tracks[i] != 1 || track_nr >= HEADS_PER_HEADSTACK * NUM_HEADS_PER_DPU)
	    {
	       cerr << "Init_Proc::do_tape_stats - active tracks wrong!" << endl;
	       this->do_abort();
	       this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	       return false;
	    }
	    
	    suc_stats_track_map[track_nr] = 1;
	    suc_stats_track_errs[track_nr].set_frames(av_stats_track_errs[i].get_frames());
	    suc_stats_track_errs[track_nr].set_parity_errs(av_stats_track_errs[i].get_parity_errs());
	    suc_stats_track_errs[track_nr].set_unexp_syncs(av_stats_track_errs[i].get_unexp_syncs());
	    suc_stats_track_errs[track_nr].set_missing_syncs(av_stats_track_errs[i].get_missing_syncs());
	    suc_stats_track_errs[track_nr].set_crc_errs(av_stats_track_errs[i].get_crc_errs());
	    suc_stats_track_errs[track_nr].set_bad_frames_disc(av_stats_track_errs[i].get_bad_frames_disc());
	 }
      }
      
      
      msg->set_sender_id(SU_CONTROL_ID);
      msg->set_action_code(SU_Action_Values::Unsolicited);
		   
      this->set_local_time();
      msg->set_local_time(local_time);
      msg->set_time_stamp(avsmsg->get_time_stamp());
		   
      //transmit msg to all medium handlers
      su_cmd->transmit_medium_handlers(msg); 
		   
      Connection *target = su_cmd->get_connection(DATA_HANDLER);

      if(target)
      {
	 msg->transmit(*target);
	 delete avsmsg;
	 avsmsg = 0;
	 delete msg;
	 msg = 0;
	 return true;
      }
      else
      {
	 delete avsmsg;
	 avsmsg = 0;
	 delete msg;
	 msg = 0;
	 this->do_abort();
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Data_Handler_Conn_Exception);
	 return false;
      }
   }
   else
   {
      delete avsmsg;
      avsmsg = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   return true;
}

//_______________________________________________________________________________

bool Init_Proc::set_disk_servo_error()
{
   if (recording) return true;

   Action_Status *msg = 0;

   Servo_Error *serverror = (Servo_Error *)response_iter->contents()->get_msg();
   delete response_queue.remove(response_iter);//msg is not deleted!

   if (ignore_next_servo_error) //don't use this servo error
     {
       cout << "Init_Proc::set_disk_servo_error: ignore_next_servo_error = " << (disk_servo_error/SYSTEM_RATE) << " (will be ignored)" << endl; 
       ignore_next_servo_error = false;
       delete serverror;
       return true;
     }

   disk_servo_error = serverror->get_cur_servo_err();

   //if servo error has not changed since last time, don't do anything at all
   if (disk_servo_error == temp_servo_error) 
     {
       cout << "Init_Proc::set_disk_servo_error: servo_error = " << (disk_servo_error/SYSTEM_RATE) << " (has not changed, will be ignored)" << endl; 
       delete serverror;
       serverror = 0;
       return true;
     }
   else temp_servo_error = disk_servo_error;
   
   
   //Actually do the servoing
   
   cout << "Init_Proc::set_disk_servo_error: disk_servo_error (in seconds) = " << (disk_servo_error/SYSTEM_RATE) << endl;
   
   if (!disk_unit.disk_servo(disk_servo_error, (int_32)speedup, eVLBI)) cout << "Disk servo failed" << endl;
   
   return true;

}

//_______________________________________________________________________________

bool Init_Proc::do_disk_stats(int &lock_resyncs)
{
   Action_Status *msg = 0;

   cout << "into Init_Proc::do_disk_stats" << endl;

   int i;//counter
   
   int_32 head_positions[4];
	       
   AvStats *avsmsg = (AvStats *)response_iter->contents()->get_msg();
   delete response_queue.remove(response_iter);//msg is not deleted!
   
   //write msg to file
   outFile.open(outFilename, ios::app);
   if(outFile)
   {
      debug_log_flag = true;
      avsmsg->print(outFile);
      debug_log_flag = false;
      outFile.close();
   }
   
   //maintain su_state
   su_status->set_SU_state(avsmsg->get_su_state());
	       
   //check if a serial link is down
   //the SUIM state (int_32) is a bit representation of the state of the SUIM
   //bit 31 if set indicates a serial link problem
	       
   if(avsmsg->get_suim_state()&(1<<31))
   {
      if(++lock_resyncs < ALLOWED_LOCK_RESYNCS)
      {
	 if(error_msg.set_link_receiver_id(0,SU_Control_Error_Codes::Receiver_Out_Of_Lock))
	 {
	    //send error msg to PROCESSOR_CONTROL
	    this->transmit_error(SU_Control_Error_Codes::Receiver_Out_Of_Lock);
	 }
	 else
	 {
	    cerr << "Link_Receiver_Out_Of_Lock\n";
	    this->do_abort();
	    this->do_exit_to_mounted(SU_Control_Error_Codes::Receiver_Out_Of_Lock);
	    return false;
	 }
      }
      else
      {
	 this->do_abort();
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Receiver_Out_Of_Lock);
	 return false;
      }
   }
	       
	       
   //record posn of headstacks in su_status
   //   head_positions[0] = avsmsg->get_hd0pos();
   //   head_positions[1] = avsmsg->get_hd1pos();
   //   head_positions[2] = avsmsg->get_hd2pos();
   //   head_positions[3] = avsmsg->get_hd3pos();
   //   su_status->set_head_positions(head_positions);
   //   num_head_stacks = (unsigned int)avsmsg->get_num_hdstks();

   //fake it!
   for(i = 0; i < 4; ++i) head_positions[i] = 1;
   su_status->set_head_positions(head_positions);
   num_head_stacks = 1;


   Short_Time_Value current_tot = avsmsg->get_cur_tot();
   last_tot = current_tot; // Remember for next time around
   
   //maintain current footage attained
   su_status->set_DPU_footage(disk_unit.du_status.get_play_ptr());
   
   char* av_stats_track_map = avsmsg->get_active_tracks();
   err_cnt_struct* av_stats_track_errs = avsmsg->get_errsbytrack();

   //the stastics are processed in some TBD way and sent on to all MEDIUM_HANDLERS
   
   //determine if parity error rate is above a threshold for any track
   unsigned int frames, par_errs;
   static bool parity_flag = false;
   
   for(i = 0; i < NUM_TRACKS; ++i)
   {
      if(av_stats_track_map[i] == 1)//ie if active track
      {
	 frames = (unsigned int)av_stats_track_errs[i].get_frames();
	 par_errs = (unsigned int)av_stats_track_errs[i].get_parity_errs();
	 if(par_errs >= (frames * 2500 / PARITY_ERROR_THRESHOLD) && (!parity_flag))
	 {
	    this->transmit_disk_status(SU_Control_SU_Status_Codes::Parity_Errors);
	    parity_flag = true;
	    break;//out of for loop
	 }
      }
   }
   
   if(parity_flag && i == NUM_TRACKS)
   {
      this->transmit_disk_status();
      parity_flag = false;
   }
   
   ieee_dbl servo_error = avsmsg->get_cur_servo_err();
   static bool servo_flag = false;
   
   if(servo_error > SERVO_ERROR_THRESHOLD)
   {
      this->transmit_disk_status(SU_Control_SU_Status_Codes::Servo_Error);
      servo_flag = true;
   }
   else
   {
      if(servo_flag)
      {
         this->transmit_disk_status();
         servo_flag = false;
      }
   }

		
   if(msg = new SU_Control_Medium_Stats(avsmsg->get_su_id(),
				      disk_unit.du_status.get_VSN_active().c_str(),
				      num_head_stacks,
				      head_positions[0],
				      head_positions[1],
				      head_positions[2],
				      head_positions[3],
				      disk_unit.du_status.get_play_ptr(),
				      avsmsg->get_cur_tot(),
				      avsmsg->get_cur_servo_err()))
   {
      //also need to supply other parameters to SU_Control_Medium_Stats AS as follows
      char* suc_stats_track_map = ((SU_Control_Medium_Stats *)msg)->get_active_tracks();
      err_cnt_struct* suc_stats_track_errs = ((SU_Control_Medium_Stats *)msg)->get_errsbytrack();
      
      int track_nr = 0;
      
      //need to map tracks(TRM I/P #) to tracks(on tape)!
      for(i = 0; i < NUM_TRACKS; ++i)
      {
	 if(av_stats_track_map[i] == 1)//ie this trm is active
	 {
	    //get the actual track # - its in char dim_x_bar_configuration[i]
	    track_nr = (int) dim_x_bar_configuration[i];
	    //track_nr must be less than HEADS_PER_HEADSTACK * NUM_HEADS_PER_DPU
	    //char the_active_tracks[i] should also == 1
	    if(the_active_tracks[i] != 1 || track_nr >= HEADS_PER_HEADSTACK * NUM_HEADS_PER_DPU)
	    {
	       cerr << "Init_Proc::do_tape_stats - active tracks wrong!" << endl;
	       this->do_abort();
	       this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	       return false;
	    }
	    
	    suc_stats_track_map[track_nr] = 1;
	    suc_stats_track_errs[track_nr].set_frames(av_stats_track_errs[i].get_frames());
	    suc_stats_track_errs[track_nr].set_parity_errs(av_stats_track_errs[i].get_parity_errs());
	    suc_stats_track_errs[track_nr].set_unexp_syncs(av_stats_track_errs[i].get_unexp_syncs());
	    suc_stats_track_errs[track_nr].set_missing_syncs(av_stats_track_errs[i].get_missing_syncs());
	    suc_stats_track_errs[track_nr].set_crc_errs(av_stats_track_errs[i].get_crc_errs());
	    suc_stats_track_errs[track_nr].set_bad_frames_disc(av_stats_track_errs[i].get_bad_frames_disc());
	 }
      }
      
      
      msg->set_sender_id(SU_CONTROL_ID);
      msg->set_action_code(SU_Action_Values::Unsolicited);
		   
      this->set_local_time();
      msg->set_local_time(local_time);
      msg->set_time_stamp(avsmsg->get_time_stamp());
		   
      //transmit msg to all medium handlers
      su_cmd->transmit_medium_handlers(msg); 

      Connection *target = su_cmd->get_connection(DATA_HANDLER);

      if(target)
      {
	 msg->transmit(*target);
	 delete avsmsg;
	 avsmsg = 0;
	 delete msg;
	 msg = 0;
	 return true;
      }
      else
      {
	 delete avsmsg;
	 avsmsg = 0;
	 delete msg;
	 msg = 0;
	 this->do_abort();
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Data_Handler_Conn_Exception);
	 return false;
      }
   }
   else
   {
      delete avsmsg;
      avsmsg = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   return true;
}

//________________________________________________________________

bool Init_Proc::pos_tape_to_footage(short reqd_tape_ftg, int the_tape_ftg_time_out)
{
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   Aux_Data *aux_table = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(reqd_tape_ftg < 0) reqd_tape_ftg = 0;
   
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Pos_Tape_To_Footage)) && 
      (table1 = new Tape_to_Footage_Table((unsigned int)reqd_tape_ftg)) &&
      (aux_table = new Tape_to_Footage_Aux(the_tape_ftg_time_out)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::Tape_Footage, table1, aux_table)) && 
      (cmd->add_table(gen_tab1)))
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   else
   {
      delete table1;
      delete aux_table;
      delete gen_tab1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      aux_table = 0;
      cmd = 0;
      //form su_abort_ops cmd and send to su
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
}

//____________________________________________________________________________________

bool Init_Proc::pos_tape_to_TOT(short reqd_tape_ftg, ieee_dbl reqd_TOT, int tape_TOT_time_out)
{
   Command_Message *cmd = 0;//outbound
   Table_Data *table1 = 0;
   Aux_Data *aux_table = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(reqd_tape_ftg < 0) reqd_tape_ftg = 0;
   
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Pos_Tape_To_TOT)) && 
      (table1 = new Tape_to_TOT_Table((unsigned int)reqd_tape_ftg, reqd_TOT)) &&
      (aux_table = new Tape_to_TOT_Aux(tape_TOT_time_out)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::Tape_Ft_TOT, table1, aux_table)) &&
      cmd->add_table(gen_tab1)
      )
   {
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   else
   {
      delete table1;
      delete aux_table;
      delete gen_tab1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      aux_table = 0;
      cmd = 0;

      //form su_abort_ops cmd and send to su
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
}

bool Init_Proc::do_auto_tracking(ieee_dbl obey_ROT, bool enable)
{
   const int dpu_cmd_len = 16;
   char period[3][dpu_cmd_len];

   int tracking_threshold;

   // Tracking threshold based on EVNDOC #109
   if (the_nominal_tape_speed==80) {
     tracking_threshold = 600;
   } else if (the_nominal_tape_speed==160) {
     tracking_threshold = 1800;
   } else if (the_nominal_tape_speed==320) {
     tracking_threshold = 2000;
   } else if (the_nominal_tape_speed==135) {
     tracking_threshold = 700;
   } else if (the_nominal_tape_speed==270) {
     tracking_threshold = 1400;
   } else {
     cerr << "Warning: Unknown tape speed. Tracking threshold set to 330" << endl;
     tracking_threshold = 330;
   }

   strcpy(period[0], "W C3 0001\r\n\0\0\0\0");

   sprintf(period[1], "W CF %04X\r\n\0\0\0\0", tracking_threshold);
   //strcpy(period[1], "W CF 0150\r\n\0\0\0\0");
   
   if(enable)
   {
      strcpy(period[2], "W CA 0010\r\n\0\0\0\0");//16 seconds
   }
   else
   {
      strcpy(period[2], "W CB 0000\r\n\0\0\0\0");//16 seconds
   }
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(3, period)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      cmd = 0;

      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(obey_ROT)
      {
	 cmd->set_obey_ROT(obey_ROT);
	 cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
      }
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   return true;
}

bool Init_Proc::monitor_power(ieee_dbl start_ROT, ieee_dbl end_ROT)
{
   char dpu_cmd[1][16] = {"R 60\r\n\0\0\0\0\0\0\0\0\0"};
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::DPU_Command)) ||
      !(table1 = new DPU_Cmd_Table(1, dpu_cmd)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      cmd = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }

   if (start_ROT > 0.0) {
     cmd->set_obey_ROT(start_ROT);
     cmd->set_rpt_intrvl(10 * SYSTEM_RATE); // Every 10 seconds
     cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
		    SU_Action_Request_Flags::Obey_ROT_Set);
     if (end_ROT > 0.0) {
       cmd->set_end_ROT(end_ROT);
       cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
		      SU_Action_Request_Flags::Obey_ROT_Set + 
		      SU_Action_Request_Flags::End_ROT_Set);
     }
   }

   if(this->transmit_to_SU(cmd)) {
     delete cmd;
     cmd = 0;
   } else {
     delete cmd;
     cmd = 0;
     this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
     return false;
   }
   return true;
}


bool Init_Proc::send_flush_su(void)
{
  //THIS IS NOT AN EVN42 message - its an invention of Phil Hazel

   Command_Message *cmd = 0;
   
   if((cmd = this->form_command(1)) && cmd->set_action_code(SU_Action_Values::Station_Unit))
   {
     if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   else
   {
      delete cmd;
      cmd = 0;
      return false;
   }
   return true;
}


//-----------------------------------------------------------------------------------------------------


bool Init_Proc::handle_enquire(SU_Control_Enquiry_Aux *aux_table)
{
  SU_Control_Status *msg = 0;
  Connection *target = cmd_iter->contents()->get_conn();
  unsigned int msg_id = (unsigned int) ((CCC_Msg *)cmd_iter->contents()->get_msg())->get_msg_id();
  unsigned int op_code = (unsigned int)(aux_table->get_op_code());
  
  //to allocate this SU to a job the job_id, bar_code, dpu_type, dpu and headstack make, 
  //dpu characteristics, and dpu_id must be correct - except that all but the job_id can be wildcarded
  //(ie bar_code == "", others 0XFFFFFFF)
  
  char bar_code[MAX_NR_BANKS][bar_code_len];
  allocated = false;
  
  if(target)
    {
      unsigned int status = 0;
      if(su_status->get_medium_mounted_status())
	{
	  status |= SU_Control_SU_Status_Codes::Medium_Mounted;
	  for (int bank = 0; bank < MAX_NR_BANKS; bank++) {
	    strncpy(bar_code[bank], disk_unit.du_status.get_VSN(bank),bar_code_len);
	  }
#ifdef DEBUG 
	  cout << "Received Enquire_SU_Control cmd. su_status = mounted" << endl;
#endif
	}
      else
	{
	  for (int bank = 0; bank < MAX_NR_BANKS; bank++) {
	    bar_code[bank][0] = '\0';
	  }
#ifdef DEBUG 
	  cout << "Received Enquire_SU_Control cmd. su_status = unmounted" << endl;
#endif
	}
      
      if(aux_table->get_allocate_reqd())//ie allocation is requested
	{
	  if(status)
	    {
	      cout << "Init_Proc::handle_enquire: aux_table->get_bar_code()= " <<  aux_table->get_bar_code() << endl;
	      cout << "Init_Proc::handle_enquire: bar_code= " <<  bar_code[0] << " " << bar_code[1] << endl;
	      const int actual_medium_id_len = 8;//actual length
	      bool medium_present = false;
	      for (int bank = 0; bank < MAX_NR_BANKS && !medium_present; bank++) {
		medium_present = !strncmp_mediumid(aux_table->get_bar_code(),  bar_code[bank], actual_medium_id_len);
	      }
	      allocated = (aux_table->get_bar_code()[0] == '\0' || medium_present);
#ifdef DEBUG 
	      if(allocated) cout << "just allocated = true" << endl;
	      else cout << "just allocated = false" << endl;
#endif
	    }
	  else 
	    {
	      allocated = false;
#ifdef DEBUG 
	      cout << "allocated false because status == 0" << endl;
#endif
	    }
	  
	  if(allocated &&
	     (aux_table->get_dpu_type_code() == dpu_type || aux_table->get_dpu_type_code() == 0XFFFFFFF) &&
	     (aux_table->get_dpu_make_code() == 0XFFFFFFF) &&
	     (aux_table->get_headstack_make_code() == 0XFFFFFFF) &&
	     (aux_table->get_dpu_chars_code() == 0XFFFFFFF) &&
	     (aux_table->get_dpu_id() ==  su_configuration->get_DPU_id() || 
	      aux_table->get_dpu_id() == 0XFFFFFFF))
	    {
	      // no dpu_make_code, headstack_make_code, dpu_chars_code defined ???
	      id_of_job = (unsigned int)aux_table->get_job_id();
	      allocated = true;
	      status |= SU_Control_SU_Status_Codes::SU_Allocated;
	      
#ifdef DEBUG 
	      cout << "just allocated" << endl;
#endif
	    }
	  else 
	    {
	      allocated = false;
#ifdef DEBUG 
	      cout << "allocated false because of wrong parameters" << endl;
#endif
	    }
	}
      else
	{
	  allocated = false;
#ifdef DEBUG 
	  cout << "allocated false - not required" << endl;
#endif
	}
      
      if(msg = new SU_Control_Status(the_suid_code, bar_code, dpu_type, 
				     su_configuration->get_DPU_id(), status,
				     allocated ? id_of_job : 0))
	{
	  cout << "handle_enquire: " << the_suid_code << " " << bar_code << " " << dpu_type << " " <<
	    su_configuration->get_DPU_id() << " " << status << " " << allocated << endl;
	  this->set_local_time();
	  msg->set_local_time(local_time); 
	  msg->set_time_stamp(time_stamp); 
	  msg->set_ref_id(msg_id);
	  msg->transmit(*target);
#ifdef DEBUG 
	  cout << "SU_Control_Status xmitted to TH" << endl;
#endif
	  delete msg;
	  msg = 0;
	}
      else
	{
	  //memory allocation exception
	  this->transmit_error(SU_Control_Error_Codes::Memory_Allocation_Exception,
			       CCC_Action_Request::Enquire_SU_Control, msg_id);
	  allocated = false;
#ifdef DEBUG 
	  cout << "SU_Control_Status NOT xmitted to TH" << endl;
#endif
	}
    }
  else
    {
      //conn to target down
      cout << "Init_Proc::handle_inquire() finds connection to target down" << endl;
      this->transmit_error(SU_Control_Error_Codes::Unspecified_Error,
			   CCC_Action_Request::Enquire_SU_Control, msg_id);
    }
  
  return allocated;
}


//-----------------------------------------------------------------------------------------------------


bool Init_Proc::start_tape(bool start, bool fwd, bool fast)
{
   const int dpu_cmd_len = 16;
   char the_commands[2][dpu_cmd_len] = {"W B1 0001\r\n\0\0\0\0",//start fwd
                                        "W B5 34BC\r\n\0\0\0\0"};//135 ips
   int i = 2;
   unsigned int status = 0;
   
   if(!start)
   {
      the_commands[0][3] = '0';//stop
      i = 1;
   }
   else
   {
      if(!fwd)
      {
	 the_commands[0][8] = '0';//reverse
	 status |= SU_Control_SU_Status_Codes::Tape_Moving_Reverse;
      }
      else
      {
	 status |= SU_Control_SU_Status_Codes::Tape_Moving_Forward;
      }
      
      if(fast)
      {
	 the_commands[0][3] = '2';//fast to low tape
	 the_commands[1][3] = '6';//following switches on the low tape sensor !
	 the_commands[1][5] = '0';
	 the_commands[1][6] = '0';
	 the_commands[1][7] = '0';
	 the_commands[1][8] = '1';
	 status |= SU_Control_SU_Status_Codes::Tape_Moving_Fast;
      }
   }
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(i, the_commands)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      cmd = 0;

      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(this->transmit_to_SU(cmd))
      {
	 this->transmit_tape_status(status);
	 delete cmd;
	 cmd = 0;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   return true;
}

bool Init_Proc::dpu_ftg_changed(void)
{
   //check footage changes
   //check for response to request_dpu_footage 
   
   unsigned int attained_footage = 0;
   short signed_footage = 0;
   short dpu_footage = (short)(su_status->get_DPU_footage());
   
   if(this->in_response_queue(SU_Action_Status::DPU_Cmd_Response))
   {
      char address[DPU_RESP_ADDR_LEN];
      char addr_contents[DPU_RESP_CONTS_LEN];
      strncpy(address, (((DPU_Cmd_Response *)response_iter->contents()->get_msg())->get_address()),
	      DPU_RESP_ADDR_LEN);
      strncpy(addr_contents,(((DPU_Cmd_Response *)response_iter->contents()->get_msg())
			     ->get_contents()),DPU_RESP_ADDR_LEN);
      
      if(strtoul(address,(char **)NULL,16) == 0X30)
      {
	 //OK address OK
	 attained_footage = (unsigned int)strtoul(addr_contents,(char **)NULL,16);
	 signed_footage = (short) attained_footage;
	 
	 if(attained_footage != 0XBAD99999 && signed_footage > -1000 && signed_footage < 20000 &&
	    dpu_footage != signed_footage)
	 {
	    //ie tape is moving
	    su_status->set_DPU_footage(attained_footage);
	    return true;
	 }
	 else return false;
      }
      
      delete response_iter->contents()->get_msg();
      delete response_queue.remove(response_iter);
   }
   
   return true;//assumption
}

bool Init_Proc::set_fundamental_parameters(Scan *scan)
{
   Record_Format* record_format = scan->get_record_format();
   if(!record_format)
   {
      //trouble
     cout << "Init_Proc::set_fundamental_parameters() cannot scan->get_record_format()" << endl;
       this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
      return false;
   }
   
   this->do_the_mask(record_format);
   
   fs = record_format->get_record_bit_rate();
   muxmode = record_format->get_fan_out_factor();
   bpi =  record_format->get_bits_per_inch(); 
   //dpu_type = DPUType; - constructer
   //dpu_accel = DPU_ACCELLERATION; - constructer
   num_bits = record_format->get_number_of_bits_sampling();
   formatter_error = scan->get_formatter_error();
   oversampling_factor = record_format->get_record_oversampling_factor();
   //get speedup
   //get action table and relevant rots from cjd
   Action_Table *table;
   table = cjd->get_action_table();
   //next get relevant entrys from table
   Begin_End_Entry* entry;
   Logical_Array* array = NULL;
	 
   if(table->reset_la_iterator())
     {
       do{
	 entry = table->get_current_la();
	 //	 if(entry->get_begin_ROT() >= sub_job_start_ROT &&
	 //	    entry->get_end_ROT() <= sub_job_ending_ROT)
	 if(entry->get_begin_ROT() < sub_job_ending_ROT &&
	    entry->get_end_ROT() > sub_job_start_ROT)
	   {
	     //get logical array	   
	     array = cjd->get_la(entry->get_la_id());
	     //get speedup
	     speedup = array->get_playback_speedup_rate();
	     bandwidth = array->get_correlation_bandwidth();
	     itsRecirculationFactor = array->get_recirculation_factor();
	     cout << "itsRecirculationFactor = " << itsRecirculationFactor << endl;
	     nr_tracks_in_scan = array->get_nr_of_channels() * (int)muxmode * num_bits;
	     break;//out of do
	   }
       }while(table->get_next_la());
     }
   else
     {
       //trouble
       cout << "Init_Proc::dpu_ftg_changed() cannot table->reset_la_iterator()" << endl;
       this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
       return false;
     }
   
   
   // This should really come from the cjd. But it is not there yet.
   //the_model_parameters = (int)(fs/(bandwidth*2)+0.1); // oversampling factor. Only works for 1,2,4;
   //the_model_parameters |= record_format->get_record_oversampling_factor();
   the_model_parameters &= 0xFFF8; // Force the lowest 3 bits to zero
   the_model_parameters |= 0x7 & record_format->get_record_oversampling_factor();
   the_nominal_tape_speed = (int)(current_scan->get_record_tape_speed()*speedup);
   the_nominal_data_rate = current_scan->get_data_rate()*speedup;

   if (disk_unit.remote_version() == Disk_Unit::mark5b) {
     cout << "setting fundametals for dimino" << endl;
     bit_stream_mask = Mark5B_Control::construct_bit_stream_mask(cjd, array, current_station, scan);

     /*
     int data_rate = nr_tracks_in_scan * (int)round(the_nominal_data_rate);
     switch (data_rate) {
     case (64) : bit_stream_mask = 0x000f0000; break;
     case (128) : bit_stream_mask = 0x000f000f; break;
     case (256) : bit_stream_mask = 0x00ff00ff; break;
     case (512) : bit_stream_mask = 0xffffffff; break;
     case (1024) : bit_stream_mask = 0xffffffff; break;
     default: cout << data_rate << " is not a recognized total data rate" << endl; break;
     }
     */
   } // if remote is a mark5b

   return true;
}

bool Init_Proc::run_children(bool head_0, bool head_1)
{
  cout << "into run_children" << endl;
  Peak_Head* unit = 0;
   unsigned int head_nr_1, head_nr_2;
   int nom_pos;
   
   //need to map logical to physical heads
   if(head_0)
   {
      int logical_head_nr = 0; //vex_head_nr - 1
      if (swap_head) logical_head_nr = 1;
      head_nr_1 = 1;//from db
      
      if(!this->config_dpu(head_nr_1)) return false;
      
      nom_pos = (int)(current_scan->get_headgroup_offset(logical_head_nr)) * 10;//0.1 micron units
      cout << "Init_Proc::run_children(), vex head " << logical_head_nr+1 << ": nom_pos = " << nom_pos << endl;
      
      if (unit = new Peak_Head(get_controller()->new_thread_id(),
			       this->get_id(),
			       get_controller(),
			       this,
			       nom_pos,
			       (current_scan->get_tape_direction() == forward) ? 
			       		head_0_fwd_offset : head_0_rev_offset,
			       head_nr_1,
			       logical_head_nr))
      {
	 if(this->add_child(unit) == ERROR_ID)
	 {
	    this->do_abort();
	    cout << "Init_Proc::run_children() cannot add_child()" << endl;
	    this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	    return false;
	 }
      }
      else
      {
	 delete unit;
	 unit = 0;
	 this->do_abort();
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	 return false;
      }
   }
   
   if(head_1)
   {
      int logical_head_nr = 1; //vex_head_nr - 1
      if (swap_head) logical_head_nr = 0;
      head_nr_2 = 2;//from db
      
      if(!this->config_dpu(head_nr_2)) return false;
      
      nom_pos = (int)(current_scan->get_headgroup_offset(logical_head_nr)) * 10;//0.1 micron units
      cout << "Init_Proc::run_children(), vex head " << logical_head_nr+1 << ": nom_pos = " << nom_pos << endl;
      
      if (unit = new Peak_Head(get_controller()->new_thread_id(),
			       this->get_id(),
			       get_controller(),
			       this,
			       nom_pos,
			       (current_scan->get_tape_direction() == forward) ? 
			       		head_1_fwd_offset : head_1_rev_offset,
			       head_nr_2,
			       logical_head_nr))
      {
	 //get_controller()->add_thread(unit);
	 if(this->add_child(unit) == ERROR_ID)
	 {
	    this->do_abort();
	    cout << "Init_Proc::run_children() finds add_child() with ERROR_ID" << endl;
	    this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
	    return false;
	 }
      }
      else
      {
	 delete unit;
	 unit = 0;
	 this->do_abort();
	 this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	 return false;
      }
   }
   
   //start the dpu
   this->set_dpuspeed();
   this->start_dpu(true);

  //block this process, so it waits for the dependents to return
  this->set_stat(blocked);
  return true;
}

bool Init_Proc::check_reset_peripherals() {
  if ( reset_peripherals_required) {
    last_received_rot = -1;
    poly_offset = 0;
    Command_Message *command = 0;
    if((command = this->form_command(SU_Station_Unit_Cmds::Reset_SU_Peripherals)) && 
       this->transmit_to_SU(command))
      {
	reset_peripherals_required = false;
	next_SU_command_time = time(0) + 15;
	cout << "Reset_SU_Peripherals command sent" << endl;	
	recording = false; // FIX do this on more places?
      }
    delete command;
    command = 0;
    return !reset_peripherals_required;
  }
  return false;
}

bool Init_Proc::config_su(General_CCC_Table *gen_tab1, General_CCC_Table *gen_tab2, 
			  General_CCC_Table *gen_tab3, General_CCC_Table *gen_tab4,
			  General_CCC_Table *gen_tab5, General_CCC_Table *gen_tab6,
			  Command_Message * cmd)
{
  /* this is done in pre_state (current_state == mounted)
   Command_Message *command = 0;
   if((command = this->form_command(SU_Station_Unit_Cmds::Reset_SU_Peripherals)) && 
      this->transmit_to_SU(command))
   {
      delete command;
      command = 0;
      sleep(15);
   }
   else
   {
      delete command;
      command = 0;
      return false;
   }
  */
   cout << "configuring su, skipping Reset_SU_Peripherals" << endl;
   reset_peripherals_required = true;
   if(!this->flush_suim()) return false;
   
   this->do_servoing(false, -1.0);//disable CCC control of servoing
   
   //Miscell_SU_Params first, 
   if(cmd &&
      cmd->add_table(gen_tab6) &&
      this->transmit_to_SU(cmd))
   {
      cmd->remove_table(SU_Table_Types::Miscell_SU_Params);
   }
   else return false;
   
   if(cmd->add_table(gen_tab1) &&
      this->transmit_to_SU(cmd))
   {
      cmd->remove_table(SU_Table_Types::DIM_X_Bar_Setting);
   }
   else return false;
	 
   if(cmd->add_table(gen_tab4) &&
      this->transmit_to_SU(cmd))
   {
      cmd->remove_table(SU_Table_Types::CRM_Settings);
   }
   else return false;
	 
   if(cmd->add_table(gen_tab2) &&
      this->transmit_to_SU(cmd))
   {
      cmd->remove_table(SU_Table_Types::TRM_Settings);
   }
   else return false;
   
   //now configure suim
   if(!this->configure_suim_2()) return false;
	
   // Don't sent aux table anymore
   //if(cmd->add_table(gen_tab5) &&
   //   (the_tape_format != 1 ?
   //    cmd->add_table(gen_tab3) : true) &&
   //   this->transmit_to_SU(cmd))
   //{
   //   cmd->remove_table(SU_Table_Types::DM_Configuration);
   //   if(the_tape_format != 1) cmd->remove_table(SU_Table_Types::Aux_Data_Check);
   //   return true;
   //}
   //else return false;

   if(cmd->add_table(gen_tab5) && this->transmit_to_SU(cmd))
   {
      cmd->remove_table(SU_Table_Types::DM_Configuration);
      return true;
   }
   else return false;
}


bool Init_Proc::get_bar_code(void)
{
   const int dpu_cmd_len = 16;
   char command[1][dpu_cmd_len] = {"W BA 0001\r\n\0\0\0\0"};//unready dpu
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(1, command)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      cmd = 0;

      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(!this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
      
      delete cmd; cmd = 0;
   }
   
   sleep(10);
   command[0][3] = '3';//ready dpu
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(1, command)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      cmd = 0;

      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(!this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
      
      delete cmd; cmd = 0;
   }
   
   
   return true;
}


bool Init_Proc::rewind_unload(void)
{
   const int dpu_cmd_len = 16;
   char command[1][dpu_cmd_len] = {"W B4 0001\r\n\0\0\0\0"};
   
   Command_Message *cmd = 0;
   Table_Data *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if(!(cmd = this->form_command(SU_Station_Unit_Cmds::Configure_DPU)) ||
      !(table1 = new DPU_Cmd_Table(1, command)) ||
      !(gen_tab1 = new General_CCC_Table(SU_Table_Types::DPU_Commands, table1, 0)) || 
      !(cmd->add_table(gen_tab1)))
   {
      delete gen_tab1;
      delete table1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      cmd = 0;

      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
   else
   {
      if(!this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
      
      delete cmd; cmd = 0;
      return true;
   }
}

bool Init_Proc::request_su_status(void)
{
   Command_Message *cmd = 0;
   
   if(cmd = this->form_command(SU_Station_Unit_Cmds::Req_SU_Status))
   {
      if(this->transmit_to_SU(cmd))
      {
         delete cmd;
	 cmd = 0;
         sleep(10);
         return true;
      }
      else
      {
         this->transmit_error(SU_Control_Error_Codes::SU_Connection_Exception);
         return false;
      }
   } 
   else
   {
      delete cmd;
      cmd = 0;
      this->transmit_error(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
}

bool Init_Proc::do_scan_start_stop(ieee_dbl start, ieee_dbl stop, ieee_dbl obey)
{
   Command_Message *cmd = 0;
   Starting_ROT_Table *table1 = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Set_Starting_ROT)) &&
      (table1 = new Starting_ROT_Table(start, stop)) &&
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::Starting_ROT, table1, 0)) &&
      (cmd->add_table(gen_tab1)))
   {
      cmd->set_obey_ROT(obey);
      cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   else
   {
      delete table1;
      delete gen_tab1;
      delete cmd;
      table1 = 0;
      gen_tab1 = 0;
      cmd = 0;

      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
#if 0
   //also need to send stop crm to be obeyed at stop

   if((cmd = this->form_command(SU_Station_Unit_Cmds::Stop_CRM)))
   {
      cmd->set_obey_ROT(stop);
      cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
      if(this->transmit_to_SU(cmd))
      {
	 delete cmd;
	 cmd = 0;
	 return true;
      }
      else
      {
	 delete cmd;
	 cmd = 0;
	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	 return false;
      }
   }
   else
   {
      delete cmd;
      cmd = 0;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
   }
#endif
   return true;
}

bool Init_Proc::do_servoing(bool enable, ieee_dbl obey_rot)
{
   Command_Message *cmd = 0;
   Set_Feature_Set_Table *table = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   unsigned int feature_2 = Feature_Set_2_Bit::Enable_Tape_Servo;
   unsigned int action = 0;
   
   if(enable)
   {
      action = Set_Feature_Set_Actions::Bit_Set;
   }
   else
   {
      action = Set_Feature_Set_Actions::Bit_Clear;
   }
   
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Set_Feature_Set_Cmd)) &&
      (table = new Set_Feature_Set_Table(action,
	                                  0XFFFFFFFF, //default all set
	                                  0XFFFFFFFF, // ditto
	                                  feature_2 , 
	                                  0)) && //default all unset
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::Set_Feature_Set, table, 0)) &&
      cmd->add_table(gen_tab1)
     )
   {
      if (obey_rot>0.0) 
      {
	cmd->set_obey_ROT(obey_rot);
	cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
      }
      if(this->transmit_to_SU(cmd))
      {
         delete cmd;
	 cmd = 0;
         return true;
      }
      else
      {
         delete cmd;
	 cmd = 0;
      	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
      }
   }
   else
   {  
      delete table;
      delete gen_tab1;
      delete cmd;
      table = 0;
      gen_tab1 = 0;
      cmd = 0;
     
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
   }
   
   return false;
}

bool Init_Proc::ccc_servoing(bool enable, ieee_dbl obey_rot)
{
   Command_Message *cmd = 0;
   Set_Feature_Set_Table *table = 0;
   General_CCC_Table *gen_tab1 = 0;
   
   unsigned int feature_2 = Feature_Set_2_Bit::CCC_Tape_Servo_Control;
   unsigned int action = 0;
   
   if(enable)
   {
      action = Set_Feature_Set_Actions::Bit_Set;
   }
   else
   {
      action = Set_Feature_Set_Actions::Bit_Clear;
   }
   
   if((cmd = this->form_command(SU_Station_Unit_Cmds::Set_Feature_Set_Cmd)) &&
      (table = new Set_Feature_Set_Table(action,
	                                  0XFFFFFFFF, //default all set
	                                  0XFFFFFFFF, // ditto
	                                  feature_2 , 
	                                  0)) && //default all unset
      (gen_tab1 = new General_CCC_Table(SU_Table_Types::Set_Feature_Set, table, 0)) &&
      cmd->add_table(gen_tab1)
      )
   {
      if (obey_rot>0.0) 
      {
	cmd->set_obey_ROT(obey_rot);
	cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
      }
      if(this->transmit_to_SU(cmd))
      {
         delete cmd;
	 cmd = 0;
         return true;
      }
      else
      {
         delete cmd;
	 cmd = 0;
      	 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
      }
   }
   else
   {  
      delete table;
      delete gen_tab1;
      delete cmd;
      table = 0;
      gen_tab1 = 0;
      cmd = 0;

      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
   }
   
   return false;
}

Polys *Init_Proc::get_polys(void)
{
  //ask db for polys
  Polys *polys = 0;
  unsigned int poly_pid = (unsigned int)(current_station->get_persistent_polys_id());
  cout << "poly_pid = " << poly_pid << endl;

  if((polys = (Polys *)mystore.get_object(POLYNOMIALS, poly_pid)))
    {
      return polys;
    }
  else
    {
      int count = 0;
      while (count<10)
	{
	  cout << "WARNING: did not get polys; Polys* = " << polys << "; retry # " << count << endl;
	  sleep(10);
	  if ((polys = (Polys *)mystore.get_object(POLYNOMIALS, poly_pid)))
	    {
	      return polys;
	    }
	  ++count;
	}
    }

  cout << "ERROR: did not get polys; Polys* = " << polys << "; giving up" << endl;
  //throw error
  this->do_exit_to_mounted(SU_Control_Error_Codes::Polynomials_Error);
  return 0;
}


bool Init_Proc::configure_su_next_scans(int &nr_tracks, ieee_dbl start_ROT) 
{
  Command_Message *cmd = 0; //outbound
  
  Table_Data *table1 = 0, *table2 = 0, *table3 = 0, *table4 = 0, 
    *table5 = 0, *table6 = 0;
  
  Aux_Data *aux_table = 0;
  
  General_CCC_Table *gen_tab1 = 0, *gen_tab2 = 0, *gen_tab3 = 0, *gen_tab4 = 0,
    *gen_tab5 = 0, *gen_tab6 = 0;

  ieee_dbl min_scan_gap;
  if (!use_mark5)
  min_scan_gap = MIN_SCAN_GAP * SYSTEM_RATE; // Minimum time between 
  // scans to merge
  else 
  min_scan_gap = 10 * SYSTEM_RATE; // Minimum time between 
  // scans to merge; even smaller in case of Mk5
  Scan *next_scan;
  
  ieee_dbl stop_ROT, first_ROT;
  int_64 start_byte = 0;//for MarkV
  
  first_ROT = start_ROT;

  scan_values_list.clear();
  scan_values start_stop_values;
  start_stop_values.start_rot = 0;
  start_stop_values.stop_rot = 0;
  start_stop_values.start_byte = 0;
  start_stop_values.startdpu = false;
  start_stop_values.firstscan = false;

  
  // Initialise the pseudo-global last_TOT to ensure the first tape_stats TOT
  // is accepted
  last_tot = -1;
  
  // Initialise iterator to first scan in this subjob
  if(!this->set_current_scan()) 
    {
      cout << "Init_Proc::configure_su_next_scans() cannot set_current_scan()" << endl;
      this->do_exit_to_mounted(SU_Control_Error_Codes::Unspecified_Error);
      return false;
    }
  
  stop_ROT = current_scan->get_stop_TOT();
  
  if (use_mark5) 
    {
      start_byte = (int_64)current_scan->get_start_footage();
      cout << "configure_su_next_scans: current_scan->get_start_footage() = " << current_scan->get_start_footage() << endl;
      cout << "configure_su_next_scans: start_byte = " << start_byte << endl;
    }
  else this->set_dpuspeed();// Set the DPU to the correct speed (no need to rot queue it) 
  

  bool newscan = false;
  bool newDIM  = false;
  bool newTRM  = false;
  bool newCRM  = false;
  bool newDCM  = false;
  bool newAUX  = false;
  bool startDPU = true;
  bool newbarrel = false;
  bool nextDIM  = false;
  bool nextTRM  = false;
  bool nextCRM  = false;
  bool nextDCM  = false;
  bool nextAUX  = false;
  bool nextbarrel = false;
  bool nextstartDPU = false;
  
  // Loop through the scans looking for gaps or configuration changes
  ieee_dbl prev_stop = 0.0;
  bool firstscan = true;
  while(scans->iterator().next()) 
    {
      cout << "test next scan" << endl;
      next_scan = &((Scan &) scans->iterator().current());
      
      // Check if next scan is in sub_job - if not break
      if(next_scan->get_start_TOT() >= sub_job_ending_ROT) 
		  {
			 cout << "next scan outwith sub job rots" << endl;
			 break;
		  }
      
      if (!use_mark5)
		  {
			 // Check for change in tape direction and break if so
			 if(current_scan->get_tape_direction() != next_scan->get_tape_direction()) 
				{
				  cout << "next scan outwith sub job - tape dirn" << endl;
				  break;
				}
		  }
      
      cout << "OK - doing next scan" << endl;
      
      // Do the tape/disk stop for long enough to treat as a new scan
      // Don't introduce gaps for eVLBI.
      if (next_scan->get_start_TOT() > stop_ROT+min_scan_gap && !eVLBI) 
		  {
			 nextstartDPU = true;
			 newscan = true;
		  }
      
      // Redo fundamental parameters
      if(!this->set_fundamental_parameters(next_scan)) 
		  {
			 cout << "set_fundamental_parameters failed" << endl;
			 break;
		  } 
      
      // Check if fan_factor or channel_track_link_table have changed
      if(current_scan->get_record_format()->get_fan_out_factor() !=
			next_scan->get_record_format()->get_fan_out_factor() ||
			next_scan->get_channel_track_table_ID() != current_scan->get_channel_track_table_ID()) 
		  {
			 
			 newscan = true;
			 // Most of the SU will need to be reconfigured
			 nextDIM = true;
			 nextCRM = true;
			 nextAUX = true;
			 nextTRM = true;
			 
			 // Has the barrel rolling setup changed
			 if(next_scan->get_barrel_rolling()->get_frames_per_phase() != 
				 the_frames_per_barrel_roll_ph[0]) 
				{
				  newbarrel = true;
				}
		  }
      
      // Check if only changes to the barrel rolling
      if((next_scan->get_barrel_rolling()->get_frames_per_phase() != 
			 the_frames_per_barrel_roll_ph[0]) ||
			(next_scan->get_barrel_rolling()->get_nr_of_periods() !=
			 current_scan->get_barrel_rolling()->get_nr_of_periods())) 
		  {
			 
			 newscan = true;
			 // Some of the SU will need to be reconfigured
			 nextCRM = true;
			 nextAUX = true;
			 nextbarrel = true;
		  }
		
		
		
      if (newscan) 
		  {
			 if (use_mark5) 
				{
				  // store start rot, stop rot and start byte in list
				  start_stop_values.start_rot = start_ROT;
				  start_stop_values.stop_rot = stop_ROT;
				  start_stop_values.start_byte = start_byte;
				  start_stop_values.startdpu = startDPU;
				  start_stop_values.firstscan = firstscan;
				  scan_values_list.push_back(start_stop_values);
				}
			 su_scan_config(firstscan, start_ROT, stop_ROT, prev_stop,
										newDIM, newTRM, newCRM, newDCM, newAUX, newbarrel, startDPU);
			 newscan = false;
			 firstscan = false;
			 newDIM = nextDIM;
			 newTRM = nextTRM;
			 newCRM = nextCRM;
			 newDCM = nextDCM;
			 newAUX = nextAUX;
			 newbarrel = nextbarrel;
			 startDPU = nextstartDPU;
			 prev_stop = stop_ROT;
			 
			 nextDIM = false;
			 nextTRM = false;
			 nextCRM = false;
			 nextDCM = false;
			 nextAUX = false;
			 nextbarrel = false;
			 nextstartDPU = false;
			 
			 start_ROT = next_scan->get_start_TOT();
			 stop_ROT = next_scan->get_stop_TOT();
			 if (use_mark5) start_byte = (int_64)next_scan->get_start_footage();
		  } 
      else 
		  { // Merge the two scans from the point of view of the SU
			 stop_ROT = next_scan->get_stop_TOT();
		  }
      
      // The stop_ROT should not go past the end of the subjob
      if (stop_ROT > sub_job_ending_ROT) stop_ROT = sub_job_ending_ROT;
      
    } // while next scan
  
  // The last scan in the loop will not have been done yet
  if (use_mark5) 
	 {
		// store start rot, stop rot and start byte
		start_stop_values.start_rot = start_ROT;
		start_stop_values.stop_rot = stop_ROT;
		start_stop_values.start_byte = start_byte;
		start_stop_values.startdpu = startDPU;
		start_stop_values.firstscan = firstscan;
		scan_values_list.push_back(start_stop_values);
	 }
  su_scan_config(firstscan, start_ROT, stop_ROT, prev_stop, newDIM, newTRM, 
							 newCRM, newDCM, newAUX, newbarrel, startDPU);

  // Set the update of Tape and Sampler Stats
  ieee_dbl interval;
  char *update_time;
  if ((update_time = getenv("STATS_UPDATE"))) 
    {
      interval = atoi(update_time) * SYSTEM_RATE;
    } 
  else 
    {
      interval = DATA_REQ_INTERVAL * SYSTEM_RATE;
    }
  
  start_ROT = first_ROT;

  // Send request for average statistics
  if (!(cmd = form_command(SU_Station_Unit_Cmds::Req_Av_Stats))) 
    {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
    }
  
  // Set obey ROT, End ROT and repeat interval
  cmd->set_obey_ROT(start_ROT);
  cmd->set_end_ROT(stop_ROT);
  cmd->set_rpt_intrvl(interval);
  cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
		 SU_Action_Request_Flags::Obey_ROT_Set +
		 SU_Action_Request_Flags::End_ROT_Set);
  
  if (!(this->transmit_to_SU(cmd))) 
    {
      this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
      delete cmd;
      cmd = 0;
      return false;
    }
  delete cmd; cmd = 0;

  // Send Request Sampler Stats
  if (!(cmd = form_command(SU_Station_Unit_Cmds::Req_Sampler_Stats))) 
    {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
    }
  
  // Set obey ROT, End ROT and repeat interval
  cmd->set_obey_ROT(start_ROT);
  cmd->set_end_ROT(stop_ROT);
  cmd->set_rpt_intrvl(interval);
  cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
					  SU_Action_Request_Flags::Obey_ROT_Set +
					  SU_Action_Request_Flags::End_ROT_Set);
  
  if (!(this->transmit_to_SU(cmd))) 
    {
      this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
      delete cmd;
      cmd = 0;
      return false;
    }
  delete cmd; 
  cmd = 0;

  if (nr_pcal_channels > 0)
    {
      // Cancel Request Phase Calibration Data at last stop_ROT.
      if (!(cmd = form_command(SU_Station_Unit_Cmds::Req_Phase_Cal_Data))
	  || !(aux_table = new Phase_Cal_Aux(0))
	  || !(gen_tab1 = new General_CCC_Table(SU_Table_Types::P_C_Data_Accum_Period, 0, aux_table))
	  || !(cmd->add_table(gen_tab1)))
	{
	  this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	  DELETE_ALL;
	  return false;
	}

      cmd->set_obey_ROT(stop_ROT);
      cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);

      if (!(this->transmit_to_SU(cmd)))
	{
	  this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	  DELETE_CMD;
	  return false;
	}

      DELETE_CMD;
    }

  // Send SU_Finish cmd at last stop_ROT
  if(cmd = this->form_command(SU_Station_Unit_Cmds::SU_Finish)) 
    {
      cmd->set_obey_ROT(stop_ROT);
      cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
    } 
  else 
    {
      DELETE_ALL;
      this->do_abort();
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
    }
  
  if(!(this->transmit_to_SU(cmd))) 
	 { //ie cmd was not transmitted
		DELETE_CMD;
		this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
		return false;
	 } 
  else 
    {
      DELETE_CMD;
    }
  
  
  // Monitor Power
  if (!monitor_power(first_ROT, stop_ROT)) return false;
  
  return true;
}


//______________________________________________________________


// Return the recommended Mark5A+ mode (see Mark5 memo #039) for the
// given number of recorded Mark5B bit streams and fanout.
//
const char *
plusmode(int num_bit_streams, int fanout)
{
  const char *modes[] = { "mark5a+0", "mark5a+1", "mark5a+2" };
  int map = -1;

  switch (num_bit_streams) {
  case 2:
    assert(fanout == 4);
    map = 0;
    break;
  case 4:
    assert(fanout != 1);
    /* FALLTHROUGH */
  case 8:
  case 16:
    switch(fanout) {
    case 1:
      map = 0;
      break;
    case 2:
      map = 2;
      break;
    case 4:
      map = 1;
      break;
    }
    break;
  case 32:
    assert(fanout == 1 || fanout == 2);
    map = 2;
    break;
  }

  assert(map != -1);
  return modes[map];
}

// Configure a single SU scan
// There should be a check on the scan start/stop times so that they are integral
bool Init_Proc::su_scan_config_disk()
{
  Command_Message *cmd = 0; //outbound
  ieee_dbl accln_time, dpu_start;
  ieee_dbl servo_interval = SERVO_REQ_INTERVAL * SYSTEM_RATE;
  ieee_dbl start_delay = START_SCAN_DELAY * SYSTEM_RATE * speedup;
  //ieee_dbl start_delay = 7 * SYSTEM_RATE;


  if (scan_values_list.empty()) return true;

  scan_values start_stop_values = scan_values_list[0];
  scan_values_list.erase(scan_values_list.begin());

  ieee_dbl start_rot = start_stop_values.start_rot;
  ieee_dbl start_ROT;
  char* data_time;
  ieee_dbl stop_rot = start_stop_values.stop_rot;
  int_64 start_byte = start_stop_values.start_byte;
  bool startdpu = start_stop_values.startdpu;
  bool firstscan = start_stop_values.firstscan;

  cout << "Init_Proc::su_scan_config_disk:" << endl
		 << "start_rot = " << start_rot << endl 
		 << "stop_rot = " << stop_rot << endl 
		 << "start_byte = " << start_byte << endl 
		 << "startdpu = " << startdpu << endl 
		 << "firstscan = " << firstscan << endl;




  // Send Request Servo Error
  
  if (!(cmd = form_command(SU_Station_Unit_Cmds::Req_Servo_Error)))
    {
      this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
      return false;
    }
  
  // Init_proc has just received a scan_finished message. First make sure nothing is playing/net2outing/in2netting;
  if (disk_unit.query_play_on()) disk_unit.stop_play();               
  if(eVLBI)
    {
      if (disk_unit.query_in2net_on()) disk_unit.in2net("off");
      if (disk_unit.query_e_station_play_on()) disk_unit.stop_e_station_play();//impossible to set net_protocol otherwise
      closeEVLBIout(true);
    }

  //set obey ROT, End ROT and repeat interval
  
  if (!eVLBI)
    {
      // Get the real time corresponding to byte number from data.
      disk_unit.stop_play(start_byte, 0);
      disk_unit.check_data();
      data_time = disk_unit.du_status.get_data_time();
      start_ROT = disk_unit.calc_rot(data_time);
      start_byte += disk_unit.du_status.get_data_byte_offset();
    }
  else start_ROT = start_rot;
  
  cmd->set_obey_ROT(start_ROT);
  
  cmd->set_action_code(SU_Action_Values::Station_Unit);
  cmd->set_end_ROT(stop_rot);
  cmd->set_rpt_intrvl(servo_interval);
  cmd->set_flags(SU_Action_Request_Flags::Repeat_Interval_Set +
		 SU_Action_Request_Flags::Obey_ROT_Set +
		 SU_Action_Request_Flags::End_ROT_Set);
  
  if (!(this->transmit_to_SU(cmd))) 
    {
      this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
      delete cmd;
      cmd = 0;
      return false;
    }
  delete cmd; 
  cmd = 0;
  
  if (firstscan)
    {
      const char* mode;
      char submode[50];
      ostringstream tracks;

      if (!eVLBI)
	{
	  if (strcmp("Mark5B", disk_unit.du_status.get_data_mode()) == 0)
	    {
	      // Special Mark5A+ mode

	      // Calculate the number of recorded bit streams based on the track frame period.
	      // A Mark5B frame has 10000 data bytes.
	      ieee_dbl track_frame_period =
		disk_unit.du_status.get_track_frame_period();
	      int num_bit_streams = (int)rint((10000 * 8)/(track_frame_period * fs));

	      mode = plusmode(num_bit_streams, (int)muxmode);
	      snprintf(submode, sizeof submode, "%d", num_bit_streams * (int)muxmode);
	    }
	  else
	    {
	      // Make playback mode = record mode
	      mode = disk_unit.du_status.get_data_mode();
	      strcpy(submode,disk_unit.du_status.get_data_submode());
	    }

	  // Set mode and rate on local Mark5
	  disk_unit.set_play_mode(mode, submode);
	  disk_unit.set_play_rate(the_nominal_data_rate); // from vex file

	  // Give the disk unit a chance to catch up with the ROT clock.
	  sleep((int)(5.0 / speedup));
	}
      else
	{
	  int effective_data_rate;
	  if (disk_unit.remote_version() == Disk_Unit::mark5b)
	    {
	      // Special Mark5A+ mode
	      int num_bit_streams = bitset<32>(bit_stream_mask).count();
	      mode = plusmode(num_bit_streams, (int)muxmode);
	      snprintf(submode, sizeof submode, "%d", num_bit_streams * (int)muxmode);
	      int clock_rate = 32;
#if 0
	      disk_unit.set_e_station_clock(clock_rate,"int");
	      disk_unit.set_e_station_dot();
#endif
	      int decimation = (clock_rate * num_bit_streams) / (int)round((the_nominal_data_rate * nr_tracks_in_scan));
	      disk_unit.set_e_station_dim_mode("ext", bit_stream_mask, decimation);
	      effective_data_rate = num_bit_streams * 32 / decimation;
	    }
	  else
	    {
	      switch(the_tape_format)
		{
		case MkIV:
		  mode = "mark4";
		  break;
		case VLB:
		  if (current_medium->get_recorder()->get_type() == "Mark5B")
		    mode = plusmode(nr_tracks_in_scan / (int)muxmode, (int)muxmode);
		  else
		    mode = "vlba";
		  break;
		default:
		  cout << "unknown tape format" << endl;
		  return false;
		}

	      // Reduce the number of tracks such that things will fit
	      // into the network bandwidth limit.
	      int num_tracks = nr_tracks_in_scan;
	      int connectivity = disk_unit.du_status.get_connectivity();

	      if (connectivity > 0)
		{
		  ieee_dbl data_rate = the_nominal_data_rate * 1000000;
		  while (num_tracks * data_rate > connectivity)
		    num_tracks /= 2;
		  if (num_tracks < 8)
		    num_tracks = 8;	// Minimum number of tracks allowed
		}

	      snprintf(submode, sizeof submode, "%d", num_tracks);

	      disk_unit.set_e_station_play_mode(mode, submode);
	      disk_unit.set_e_station_play_rate(the_nominal_data_rate); // from vex file
	      effective_data_rate = (int)round(the_nominal_data_rate * num_tracks);
	    }

	  // Set mode and rate on local and remote Mark5
	  disk_unit.set_play_mode(mode, submode);

	  disk_unit.set_play_rate(the_nominal_data_rate); // from vex file

	  cout << "effective data rate " << effective_data_rate << " delay " << eVLBI_clock_delay << endl;
	  int buffer_size = (int)round(eVLBI_clock_delay / SYSTEM_RATE * effective_data_rate * 1e6 / 8); // in bytes/s
	  cout << "buffer size " << buffer_size << endl;
	  int buffer_blocks = buffer_size / disk_unit.du_status.get_workbuf_size();
	  cout << "buffer blocks " << buffer_blocks << endl;

	  // Set network protocol on local and remote Mark5
	  disk_unit.set_protocol(disk_unit.du_status.get_protocol(),
				 disk_unit.du_status.get_sockbuf_size(),
				 disk_unit.du_status.get_workbuf_size(),
				 DEFAULT_EVLBI_NBLOCK,
				 buffer_blocks);
	  disk_unit.set_e_station_protocol(disk_unit.du_status.get_protocol(),
					   disk_unit.du_status.get_sockbuf_size(),
					   disk_unit.du_status.get_workbuf_size());
	}
      // Write modes to du_status
      if (!disk_unit.query_play_mode()) cout << "SU_Control: could not query play_mode" << endl;
      if (!disk_unit.query_play_rate()) cout << "SU_Control: could not query play_rate" << endl;

    }

  if (startdpu) 
    {
      if (!eVLBI)
	{
	  start_ROT -= SYSTEM_RATE * ((1.28e06 * muxmode + 2097152.0) / fs);
	  disk_unit.start_play(start_byte, (int_64)start_ROT, (int_64)speedup);
	}
      else
	{
	  if ((recording && !disk_unit.net2disk(form_net2disk_command(cjd->get_sub_job_starting_ROT().get_sysclks()).c_str())) || (!recording && !disk_unit.net2out("open"))) 
	    {
	      cout << "SU_Control: could not start net2";
	      if (recording) {
		cout << "disk" << endl;
	      }
	      else {
		cout << "out" << endl;
	      }
	      return false;
	    }
	  if (strlen(disk_unit.du_status.get_e_station_control_ip()) > 0 && !disk_unit.in2net("connect")) 
	    {
	      cout << "SU_Control: could not connect in2net" << endl;
	      return false;
	    }
	  if (strlen(disk_unit.du_status.get_e_station_control_ip()) > 0 && !disk_unit.in2net("on")) 
	    {
	      cout << "SU_Control: could not start in2net" << endl;
	      return false;
	    }
	}

      // grab playback pointer position, use it to start next scan
      //		disk_unit.query_pointer_position();
      //		disk_unit.start_play(disk_unit.du_status.get_play_ptr(), start_rot);

      // Set the Scan ROTs
      // [[Assumption]] Scan is not shorter than about 10 seconds
      if (!do_scan_start_stop(start_rot, stop_rot,   
			      start_rot-start_delay)) return false;

      if (!recording) {
	cout << "Enabling servoing with times " << Short_Time_Value(start_rot + SYSTEM_RATE).get_rot_string() << " and " << Short_Time_Value(stop_rot - 10 * SYSTEM_RATE).get_rot_string() << " in function su_scan_config_disk " << endl;
	
	// Enable servoing for this scan
	if (!(this->do_servoing(true, start_rot + SYSTEM_RATE)) ||
	    !(this->do_servoing(false, stop_rot - 10 * SYSTEM_RATE))) return false;
      }
      else {
	cout << "Skipping servoing since first scan is to be recorded" << endl;
      }

      double obey_rot = stop_rot;
      cout << "Report_Scan_Finished command formed (not first time) " << endl;
      if (cmd = form_command(SU_Station_Unit_Cmds::Report_Scan_Finished))
	{
	  cmd->set_action_code(SU_Action_Values::Station_Unit);
	  cmd->set_obey_ROT((Short_Time_Value)obey_rot);
	  cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
	}
      else
	{
	  this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	  return false;
	}
      
      if(!(this->transmit_to_SU(cmd))) 
	{ //ie cmd was not transmitted
	  delete cmd; cmd = 0;	
	  this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	  return false;
	} 
      else 
	{
	  cout << "Report_Scan_Finished sent (not first) " << endl;
	  delete cmd; cmd = 0;
	}
    }   
  return true;
}


//-----------------------------------------------------------------------------------------


// Configure a single SU scan
// There should be a check on the scan start/stop times so that they are integral
bool Init_Proc::su_scan_config(bool firstscan, ieee_dbl start_ROT, ieee_dbl stop_ROT,
										 ieee_dbl prev_stop, bool newDIM, bool newTRM, 
										 bool newCRM, bool newDCM, bool newAUX, 
										 bool newbarrel, bool startdpu) 
{
  Command_Message *cmd = 0; //outbound
  Aux_Data *aux_table = 0;
  General_CCC_Table *gen_tab1 = 0;
  ieee_dbl accln_time, dpu_start;
 
  ignore_next_servo_error = true;
 
  // Set the update of Tape and Sampler Stats
  ieee_dbl interval;
  char *update_time;
  if ((update_time = getenv("STATS_UPDATE"))) 
    {
      interval = atoi(update_time) * SYSTEM_RATE;
    } 
  else 
    {
      interval = DATA_REQ_INTERVAL * SYSTEM_RATE;
    }
  
  ieee_dbl servo_interval = SERVO_REQ_INTERVAL * SYSTEM_RATE;
  
  cout << "DEBUG: su_scan_config from " 
       << Short_Time_Value(start_ROT).get_rot_string()
       << " to " << Short_Time_Value(stop_ROT).get_rot_string() << endl;
  cout << "       firstscan= " << firstscan << endl;
  cout << "       startpdpu= " << startdpu << endl;
  cout << "       prevstop=" << Short_Time_Value(prev_stop).get_rot_string() << endl;
  
  ieee_dbl start_delay = START_SCAN_DELAY * SYSTEM_RATE * speedup;
  //ieee_dbl start_delay = 7 * SYSTEM_RATE;
  ieee_dbl scan_start;
  
  if (!use_mark5)
    {
      // Start the DPU a little early for the first scan
      // DPU acceleration 40ips. Time in wallclock sysclicks
      accln_time = ceil((9.0/8.0 * (fs/(muxmode * bpi)) * speedup * speedup / 40.0
								 + (0.6 * speedup))) * SYSTEM_RATE;
      
      if (firstscan) 
		  dpu_start = start_ROT - accln_time;
      else 
		  dpu_start = start_ROT;
      
      // Scan starts 1 sec after tape up to speed
      scan_start = dpu_start+accln_time+SYSTEM_RATE; 
    }
  
  // Start the PBUnit if requested, and stop the previous scan if applicable
  // This needs more thought. This is basically right for the first scan but
  // if it more complictated for the second. You have to allow for the fact that
  // It takes a while to get up to speed an consumes some footage. You want to
  // start the tape for the second scan a little early.
  
  if (startdpu) 
    {
      if (!use_mark5)
		  {
			 if (!firstscan) this->start_dpu(false, prev_stop);
			 this->start_dpu(true, dpu_start);
			 // Set the Scan ROTs
			 // [[Assumption]] Scan is not shorter than about 10 seconds
			 if (!do_scan_start_stop(scan_start, stop_ROT-SYSTEM_RATE, 
											 scan_start-start_delay)) return false;

			 cout << "Enabling servoing with times " << Short_Time_Value(scan_start + SYSTEM_RATE).get_rot_string() << " and " << Short_Time_Value(stop_ROT-10*SYSTEM_RATE).get_rot_string() << " in function su_scan_config " << endl;
			 
			 // Enable servoing for this scan
			 if (!(this->do_servoing(true, scan_start + SYSTEM_RATE)) ||
				  !(this->do_servoing(false, stop_ROT-10*SYSTEM_RATE))) return false;
			 // Enable auto-tracking
			 if(!this->do_auto_tracking((scan_start + 10*SYSTEM_RATE), true)) 
				{
				  this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
				  return false;
				}
		  }
    }


  // If firstscan, send off a Report_Scan_Finished command with obey_rot immediate
  // to set off sending play commands to Mk5
  if (firstscan && use_mark5)
	 {
		cout << "Report_Scan_Finished command formed (first time)" << endl;
		double obey_ROT = -1; // immediate 
		if (cmd = form_command(SU_Station_Unit_Cmds::Report_Scan_Finished))
		  {
			 cmd->set_action_code(SU_Action_Values::Station_Unit);
			 cmd->set_obey_ROT((Short_Time_Value)obey_ROT);
			 cmd->set_flags(SU_Action_Request_Flags::Obey_ROT_Set);
		  }
		else
		  {
			 this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
			 return false;
		  }
		
		if(!(this->transmit_to_SU(cmd))) 
		  { //ie cmd was not transmitted
			 delete cmd; cmd = 0;
			 this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
			 return false;
		  } 
		else 
		  {
			 cout << "first Report_Scan_Finished sent (first time)" << endl;
			 delete cmd; cmd = 0;			
		  }
	 }

  
  if (firstscan && nr_pcal_channels > 0)
    {
      // The Accumulation Period is specified in seconds.
      int acc_period = PCAL_ACC_PERIOD;
      char *value = getenv("PCAL_ACC_PERIOD");
      if (value)
	acc_period = atoi(value);

      // Send Request Phase Calibration Data.
      if (!(cmd = form_command(SU_Station_Unit_Cmds::Req_Phase_Cal_Data))
	  || !(aux_table = new Phase_Cal_Aux(acc_period))
	  || !(gen_tab1 = new General_CCC_Table(SU_Table_Types::P_C_Data_Accum_Period, 0, aux_table))
	  || !(cmd->add_table(gen_tab1)))
	{
	  this->do_exit_to_mounted(SU_Control_Error_Codes::Memory_Allocation_Exception);
	  delete cmd; cmd = 0;
	  delete aux_table; aux_table = 0;
	  delete gen_tab1; gen_tab1 = 0;
	  return false;
	}

      // Don't set a repeat interval here.  Send the command immediately,
      // the earlier we configure the PCM module for accumulation the
      // better.  ROT-queuing this for the start of the scan really screws
      // things up.

      if (!(this->transmit_to_SU(cmd)))
	{
	  this->do_exit_to_mounted(SU_Control_Error_Codes::SU_Connection_Exception);
	  delete cmd; cmd = 0;
	  aux_table = 0;
	  gen_tab1 = 0;
	  return false;
	}

#ifdef DEBUG
      cout << "Request for phase calibration data sent" << endl;
#endif

      delete cmd; cmd = 0;
      aux_table = 0;
      gen_tab1 = 0;
    }

  return true;
}

//-----------------------------------------------------------------------------------------

void
Init_Proc::write_data_message(Data_Message& msg)
{
  switch (msg.get_data_type())
    {
    case SU_Data_Types::Phase_Cal:
      General_CCC_Table& gen_table = dynamic_cast<General_CCC_Table &>
	(msg.get_table(SU_Table_Types::Phase_Cal_Data));

      try
	{
	  if (!pcalFile) throw;
	  PhaseCalDataRecord data(gen_table); *pcalFile << data;
	  pcalFile->flush();
	}
      catch (...)
	{
	  cout << "ERROR dumping phase cal data" << endl;
	}
    }
}


void Init_Proc::check_end_subjob_message() {
  // check for end message of previous subjob (if any)
  if (cjd && sending_dummy_data) 
    {
      Finish_ROT_Clock_Message *fcm = 0;
      Clock_Message_Entry *entry = 0;
      while(this->action_req_in_cmd_queue(SU_Action_Values::Finish_ROT))
	{
	  //check it's for us
	  fcm = (Finish_ROT_Clock_Message *) cmd_iter->contents()->get_msg();
	  delete cmd_queue.remove(cmd_iter);//msg is not deleted!
	  //iterate through its entries
	  entry = fcm->first_entry();
	  int i;
	  for(i = 0; i < fcm->how_many(); ++i)
	    {
	      if((entry->get_SU_array_code() != cjd->get_corr_job_ID()))
		{
		  entry = fcm->next_entry(entry);
		}
	      else
		{
		  break;//out of for loop
		}
	    }
	  if(i < fcm->how_many()) 
	    {
	      cout << "subjob finish message received, start reset" << endl;
	      sending_dummy_data = false;
	    }
	  delete fcm;
	}
    }   
  if (!sending_dummy_data) 
    {
      check_reset_peripherals();
    }
  else
    {
      cout << "suspending reset in mounted, data handler is still expecting  correct frames" << endl;
    }
}

bool Init_Proc::closeEVLBIout(bool query_first) {
  if (recording) {
    if (!query_first || disk_unit.query_net2disk_on()) {
      return disk_unit.net2disk("close");
    }
  }  
  else {
    if (!query_first || disk_unit.query_net2out_on()) {
      return disk_unit.net2out("close");
    }
    else {
      cout << "no need to close net2out; query_first = " << boolalpha << query_first << endl;
    }
  }
  return false;
}

void Init_Proc::forward_data_msg_to_dsm(Data_Message* msg) {
  cout << "trying to forward" << endl;
  if (dsm_service.is_connected() || dsm_service.try_open_connection(DH_DSM)) {
    cout << "Connection to DSM is open" << endl;
    Connection* conn = dsm_service.get_connection();
    if (conn) {
      cout << "Sending msg" << endl;
      msg->set_ref_id(the_suid_code);
      msg->transmit(*conn);
    }
    else {
      cout << "no conn" << endl;
    }
  }
  else {
    cout << "forwarding failed" << endl;
  }
}

//-----------------------------------------------------------------------------------------

#include <sys/types.h>
#include <sys/poll.h>
#include <sys/socket.h>

#include <errno.h>
#include <fcntl.h>

#ifdef HPUX_10_hp9000
# ifdef _XOPEN_SOURCE_EXTENDED
typedef size_t socklen_t;
# else
typedep int socklen_t;
# endif
#endif

// Timeout for socket operations (usec)
#define SOCKET_TIMEOUT		15 * 1000

// A version of connect(2) that allows us to specify the timeout (in usec).

int
connect_timeout (int s, const struct sockaddr *name, socklen_t namelen, int timeout)
{
  struct pollfd pfd[1];
  int nfds, flags;
  socklen_t len;
  int saved_errno = errno;
  int ret = -1;

  flags = fcntl(s, F_GETFL);
  if (flags == -1)
    return (-1);

  if (fcntl(s, F_SETFL, flags | O_NONBLOCK) == -1)
    return (-1);

  if (connect(s, name, namelen) == -1) {
    if (errno != EINPROGRESS)
      goto fail;

    pfd[0].fd = s;
    pfd[0].events = POLLOUT;
    nfds = poll(pfd, 1, timeout);
    if (nfds == -1)
      goto fail;
    if (pfd[0].revents & POLLNVAL) {
      errno = EBADF;
      goto fail;
    }
    if (pfd[0].revents & POLLERR) {
      len = sizeof(errno);
      getsockopt(s, SOL_SOCKET, SO_ERROR, &errno, &len);
      goto fail;
    }
    if (nfds == 0) {
      errno = ETIMEDOUT;
      goto fail;
    }
  }

  errno = saved_errno;
  ret = 0;

 fail:
  fcntl(s, F_SETFL, flags & ~O_NONBLOCK);
  return (ret);
}

// A version of send(2) that allows us to specify a timeout (in usec).

ssize_t
send_timeout (int s, const void *msg, size_t send_len, int send_flags, int timeout)
{
  struct pollfd pfd[1];
  int nfds, flags;
  socklen_t len;
  ssize_t nbytes = -1;

  flags = fcntl(s, F_GETFL);
  if (flags == -1)
    return (-1);

  if (fcntl(s, F_SETFL, flags | O_NONBLOCK) == -1)
    return (-1);

  do {
    pfd[0].fd = s;
    pfd[0].events = POLLOUT;
    nfds = poll(pfd, 1, timeout);
    if (nfds == -1)
      goto fail;
    if (pfd[0].revents & POLLNVAL) {
      errno = EBADF;
      goto fail;
    }
    if (pfd[0].revents & POLLERR) {
      len = sizeof(errno);
      getsockopt(s, SOL_SOCKET, SO_ERROR, &errno, &len);
      goto fail;
    }
    if (nfds == 0) {
      errno = ETIMEDOUT;
      goto fail;
    }
    nbytes = send(s, msg, send_len, send_flags);
  } while (nbytes == -1 && errno == EAGAIN);

 fail:
  fcntl(s, F_SETFL, flags & ~O_NONBLOCK);
  return (nbytes);
}

// A version of recv(2) that allows us to specify a timeout (in usec).

ssize_t
recv_timeout(int s, void *buf, size_t recv_len, int recv_flags, int timeout)
{
  struct pollfd pfd[1];
  int nfds, flags;
  socklen_t len;
  ssize_t nbytes = -1;

  flags = fcntl(s, F_GETFL);
  if (flags == -1)
    return (-1);

  if (fcntl(s, F_SETFL, flags | O_NONBLOCK) == -1)
    return (-1);

  do {
    pfd[0].fd = s;
    pfd[0].events = POLLIN;
    nfds = poll(pfd, 1, timeout);
    if (nfds == -1)
      goto fail;
    if (pfd[0].revents & POLLNVAL) {
      errno = EBADF;
      goto fail;
    }
    if (pfd[0].revents & POLLERR) {
      len = sizeof(errno);
      getsockopt(s, SOL_SOCKET, SO_ERROR, &errno, &len);
      goto fail;
    }
    if (nfds == 0) {
      errno = ETIMEDOUT;
      goto fail;
    }

    nbytes = recv(s, buf, recv_len, recv_flags);
  } while (nbytes == -1 && errno == EAGAIN);

 fail:
  fcntl(s, F_SETFL, flags & ~O_NONBLOCK);
  return (nbytes);
}


MarkV_Socket::MarkV_Socket()
{
  struct servent *s = getservbyname("m5drive", "tcp");

  socaddin.sin_family = PF_INET;
  socaddin.sin_port = s ? s->s_port : 2620;

  strcpy(outLine,"");
}


MarkV_Socket::~MarkV_Socket(){}


bool
MarkV_Socket::open_sock()
{
  sock = socket(PF_INET, SOCK_STREAM, 0);
  if (sock == -1)
    {
      cout << "socket(): " << strerror(errno) << endl;
      return false;
    }

  return true;
}


bool
MarkV_Socket::connect_sock(char *name)
{
  // Find IP address of machine to connect to
  struct hostent *h = gethostbyname(name);
  if (h == NULL)
    {
      cout << name << ": Host not found" << endl;
      close(sock); 
      return false;
    }

  assert(h->h_addr_list[0]);
  memcpy(&socaddin.sin_addr.s_addr, h->h_addr_list[0],
	 sizeof (socaddin.sin_addr.s_addr));

  // Connect this socket to Mark5A on machine
  if (connect_timeout(sock, (const struct sockaddr *) &socaddin,
		      sizeof(struct sockaddr_in), SOCKET_TIMEOUT) < 0)
    {
      cout << "Cannot connect to " << name << ": " << strerror(errno) << endl;
      close(sock);
      return false;
    }

  return true;
}


bool
MarkV_Socket::send_sock(const char *command)
{
  int len = strlen(command);

  assert(len > 4);
  if (send_timeout(sock, command, len, 0, SOCKET_TIMEOUT) < len)
    {
      cout << "socket(): " << strerror(errno) << endl;
      close(sock);
      return false;
    }

  return true;
}


bool
MarkV_Socket::receive_sock()
{
  ssize_t nbytes;
  size_t pos = 0;

  // Read reply
  do {
    nbytes = recv_timeout(sock, &outLine[pos], sizeof(outLine) - pos - 1, 0, SOCKET_TIMEOUT);
    if (nbytes == -1) {
      cout << "recv(): " << strerror(errno) << endl;
      close(sock);
      outLine[0] = '\0';
      return false;
    }
    pos += nbytes;
  } while (nbytes > 0 && outLine[pos - 1] != '\n');

  outLine[pos] = '\0';
  return true;
}


void
MarkV_Socket::remove_sock()
{
  if (sock != -1)
    close(sock);
}


//-----------------------------------------------------------------------------------------


Disk_Unit::Disk_Unit()
  :
  disk_available(0), _remote_version(unknown)
{}



Disk_Unit::~Disk_Unit(){}


bool Disk_Unit::start_play(int_64 playback_ptr, int_64 obey_ROT, int_32 speedup)
{


  char command[50];

  cout << "Disk_Unit: obey_ROT = " << obey_ROT << endl;

  if (obey_ROT == 0)
    {
      sprintf(command, "play=on:%lluULL\n", playback_ptr);
    }
  else
    {
      sprintf(command, "play=on:%lluULL:%lluULL:%d\n", playback_ptr, obey_ROT, speedup);
    }

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::start_play failed" << endl;
      return 0;
    }
  return 1;
}

int_64 Disk_Unit::calc_rot(char* time)
{
  int    itsYr = 0;
  int    itsDay = 0;
  int    itsHr = 0;
  int    itsMin = 0;
  double itsSec = 0.0;
  double itsSysclocks = 0.0;

  sscanf(time, "%dy%dd%dh%dm%lfs",
			&itsYr, &itsDay, &itsHr, &itsMin, &itsSec);
  //
  // Expand the year specification to 4 digits if necessary.
  //
  if (itsYr < 50)
    {
      itsYr += 2000;
    }
  else if (itsYr < 100)
    {
      itsYr += 1900;
    }
  
  itsSysclocks = 32.0e06 *
    ((((itsDay-1)*24 + itsHr)*60 + itsMin)*60 + itsSec);
  return (int_64)itsSysclocks;
}


bool
Disk_Unit::set_play_rate(ieee_dbl data, ieee_dbl clock, ieee_dbl clockgen)
{

  
  char command[50];

  if (data != 0.0)
    {
      sprintf(command,"play_rate=data:%lf\n", data);

      if (!handle_markv_command(command))
	{
	  cout << "Disk_unit:set_play_rate failed" << endl;
	  return 0;
	}
    }

  if (clock != 0.0)
    {
      sprintf(command,"play_rate=clock:%lf\n", clock);

      if (!handle_markv_command(command))
	{
	  cout << "Disk_unit:set_play_rate failed" << endl;
	  return 0;
	}
    }

  if (clockgen != 0.0)
    {
      sprintf(command,"play_rate=clockgen:%lf\n", clockgen);

      if (!handle_markv_command(command))
	{
	  cout << "Disk_unit:set_play_rate failed" << endl;
	  return 0;
	}
    }

  return 1;  
}


bool Disk_Unit::set_e_station_play_rate(ieee_dbl data)
{
  char command[50];
  bool e_station=true;

  sprintf(command,"play_rate=data:%lf\n", data);

  if (!handle_markv_command(command, e_station))
    {
      cout << "Disk_unit:set_e_station_play_rate failed" << endl;
      return 0;
    }
  return 1;  
}



bool Disk_Unit::stop_play(int_64 playback_ptr, int_64 obey_ROT)
{


  char command[50];
  if (obey_ROT == 0) sprintf(command,"play=off:%lluULL\n",playback_ptr);
  else sprintf(command,"play=off:%lluULL:%lluULL\n",playback_ptr,obey_ROT);
  
  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::stop_play failed" << endl;
      return 0;
    }
  return 1;
}


bool Disk_Unit::stop_play(void)
{


  char command[50];
  sprintf(command,"play=off\n");

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::stop_play failed" << endl;
      return 0;
    }
  return 1;
}


bool Disk_Unit::stop_e_station_play(void)
{
  char command[50];
  bool e_station = true;
  sprintf(command,"play=off\n");

  if (!handle_markv_command(command, e_station))
    {
      cout << "Disk_unit::stop_play at station failed" << endl;
      return 0;
    }
  return 1;
}



bool Disk_Unit::set_protocol(char* protocol, int_32 rcvbuf_size, int_32 workbuf_size, int nblock, int buffered_blocks)
{
  char command[128];

  if (strcmp(protocol, "udp") == 0)
    {
      sprintf(command,"mtu=%d\n", du_status.get_mtu_size());
      handle_markv_command(command);

      sprintf(command,"trackmask=%llx\n", du_status.get_trackmask());
      handle_markv_command(command);
    }
  
  sprintf(command,"net_protocol=%s:%d:%d:%d:%d\n", protocol, rcvbuf_size, workbuf_size, nblock, buffered_blocks);

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::set_net_protocol failed" << endl;
      return 0;
    }

  return 1;
}


bool Disk_Unit::set_e_station_protocol(char* protocol, int_32 rcvbuf_size, int_32 workbuf_size)
{
  char command[128];
  bool e_station = true;

  if (strcmp(protocol, "udp") == 0)
    {
      int mtu = du_status.get_mtu_size();
      sprintf(command,"mtu=%d\n", mtu);
      handle_markv_command(command, e_station);

      sprintf(command,"trackmask=%llx\n", du_status.get_trackmask());
      handle_markv_command(command, e_station);

      int ipd = du_status.get_ipd();
      if (set_inter_packet_delay(ipd)) {
	cout << "Set inter packet delay to " << ipd << " us" << endl;
      }
      else {
	cout << "Failed to set inter packet delay (" << ipd << ")" << endl;
      }
    }

  sprintf(command,"net_protocol=%s:%d:%d\n", protocol, rcvbuf_size, workbuf_size);

  if (!handle_markv_command(command, e_station))
    {
      cout << "Disk_unit::set_net_protocol at station failed" << endl;
      return 0;
    }

  return 1;
}

bool Disk_Unit::set_inter_packet_delay(int_32 us)
{
  char command[50];
  bool e_station=true;

  sprintf(command,"ipd=%d\n", us);

  if (!handle_markv_command(command, e_station))
    {
      cout << "Disk_unit:set_inter_packet_delay failed" << endl;
      return 0;
    }
  return 1;  
}

//bool Disk_Unit::start_net2out(int_32 rcvbuf_size)
//{
//  char command[50];
//  sprintf(command,"net2out=open:%d\n",rcvbuf_size);
//
//  if (!handle_markv_command(command))
//    {
//      cout << "Disk_unit::start_net2out failed" << endl;
//      return 0;
//    }
//  return 1;
//}


bool Disk_Unit::net2out(char* action)//open, close
{
  char command[50];

  if (string("open") == action) {
    sprintf(command,"net2out = open : %s\n", du_status.get_ext_ip());
  }
  else {
    sprintf(command,"net2out = %s\n",action);
  }

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::" << action << "_net2out failed" << endl;
      return 0;
    }
  return 1;
}

bool Disk_Unit::net2disk(const char* action)//open, close
{
#ifdef DO_ACTUAL_RECORDING
  char command[50];

  sprintf(command,"net2disk=%s\n",action);

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::" << action << "_net2disk failed" << endl;
      return false;
    }
#endif
  return true;
}

bool Disk_Unit::in2net(char* action)//on, off, connect, disconnect
{
  char command[50];
  bool e_station = true;
  char the_ext_ip[32];

  if (!strcmp(action, "connect"))
    {
      if (!strlcpy(the_ext_ip, du_status.get_ext_ip(), sizeof(the_ext_ip)))
	{
	  cout << "External IP address of disk unit not known" << endl;
	  return 0;
	}
        sprintf(command,"in2net=connect:%s\n",the_ext_ip);
    }
  else sprintf(command,"in2net=%s\n",action);      

  if (!handle_markv_command(command, e_station))
    {
      cout << "Disk_unit::" << action <<"_in2net failed" << endl;
      return 0;
    }
  return 1;
}


bool Disk_Unit::set_disk_task_ID(int_32 task_ID)
{


  //used in conjunction with play for synchronisation with ROT

  char command[50];
  sprintf(command,"task_ID=%d\n",task_ID);
  //  cout << "Disk_Unit::set_disk_task_ID: command = " << command << endl;

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::set_disk_task_ID failed" << endl;
      return 0;
    }
  return 1;  
}


bool Disk_Unit::set_play_mode(const char* outp_mode, const char* outp_submode)
{


  char command[50];

  sprintf(command,"mode=%s:%s\n",outp_mode, outp_submode);
  
  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::mode_command failed" << endl;
      return 0;
    }

  return 1;
}


bool Disk_Unit::set_e_station_play_mode(const char* outp_mode, const char* outp_submode)
{
  char command[50];
  bool e_station = true;

  sprintf(command,"mode=%s:%s\n",outp_mode, outp_submode);
  
  if (!handle_markv_command(command, e_station))
    {
      cout << "Disk_unit::mode_command failed at e_station" << endl;
      return 0;
    }

  return 1;
}


bool Disk_Unit::disk_servo(ieee_dbl servo_error, int_32 speedup, bool eVLBI)
{
  if (merlin_slave) return true;

  ieee_dbl req_byte_offset;
  long long actual_offset = 0;
  int_32 num_tracks;
  ieee_dbl bits_per_byte = 8;

  ieee_dbl clock_rate = du_status.get_clock_rate(); //track data rate *with* parity and vlba correction
  char* outp_data_mode = du_status.get_outp_data_mode(); 
  cout << "Disk_Unit::disk_servo: outp_data_mode = " << outp_data_mode << endl;

  if (!strcmp(outp_data_mode,"st")) 
    {
      num_tracks = 32;
      bits_per_byte = 8.0; //not stripped
    }
  else if (!strcmp(outp_data_mode,"mark4") || !strcmp(outp_data_mode,"vlba") || !strncmp(outp_data_mode,"mark5a+", 7))
    {
      num_tracks = atoi((const char*)du_status.get_outp_data_submode());
      bits_per_byte = 9.0;
    }
  else 
    {
      cout << "Disk_Unit::disk_servo: no recognized data format" << endl;
      return 0;
    }
  cout << "Disk_Unit::disk_servo: bits_per_byte = " << bits_per_byte << endl;
  cout << "Disk_Unit::disk_servo: num_tracks: " << num_tracks << endl;

  ieee_dbl offset_8 = 8;
  int counter = 0;
  long long req_multiple = 8;
  long long req_byte_offset_mult8;

  cout << "Disk_Unit::disk_servo: clock_rate = " << clock_rate << endl;

  static double damping = getenv("SERVO_DAMPING")?atof(getenv("SERVO_DAMPING")):1;
  cout << "Using servo damping " << damping << endl;
  req_byte_offset = damping * ((servo_error/(SYSTEM_RATE*speedup))*(ieee_dbl)num_tracks * (clock_rate*1e6))/bits_per_byte;


  cout << "Disk_Unit::disk_servo: req_byte_offset = " << req_byte_offset << endl;
  

  if (abs(req_byte_offset) > (1250*num_tracks))
    {  
      char command[50];
      // number of bytes must be a multiple of 8
      modf(req_byte_offset / req_multiple, &offset_8);
      req_byte_offset_mult8 = (long long) offset_8 * req_multiple;

      int loops = 3; //(du_status.get_trackmask() ? 1 : 3);
      int max_bytes_per_skip = numeric_limits<int>::max(); //(du_status.get_trackmask() ? 16777216 : 100000000);
      if (req_byte_offset_mult8 > max_bytes_per_skip) req_byte_offset_mult8 = max_bytes_per_skip;
      if (req_byte_offset_mult8 < -max_bytes_per_skip) req_byte_offset_mult8 = -max_bytes_per_skip;

  		
      long long skip_amount;

  		
      while ((req_byte_offset_mult8 != 0) && (counter < loops ))
	{
	  //should not attempt to skip too far, as execution of command is delayed when skip > space in buffer
	  //if      (req_byte_offset_mult8 >  100000000) skip_amount =  100000000;
	  //	  else if (req_byte_offset_mult8 < -100000000) skip_amount = -100000000;
	  skip_amount = req_byte_offset_mult8;
	  if (eVLBI) {
	    // save_skip will not skip if there are no bytes in the FIFO (as this seems to ruin the FIFO) (disabled until new version of jive5a)
	    sprintf(command,"skip=%lld\n", skip_amount);//move frame to center of 512k buffer/////		
	  }
	  else {
	    sprintf(command,"skip=%lld\n", skip_amount);//move frame to center of 512k buffer/////		
	  }
	  cout << "Disk_Unit::disk_servo: command = " << command << endl;
	  counter++;
	  if (!handle_markv_command(command))
	    {
	      cout << "Disk_unit::skip failed" << endl;
	      temp_servo_error = 0;
	      continue;
	    }
	  char query[50];
	  sprintf(query,"skip?\n");
	  
	  if (!handle_markv_command(query))
	    {
	      cout << "Disk_unit::skip_query failed" << endl;
	      break;
	    }
	  Command_Reply results = parse_command(markv_socket.outLine);

	  if (results.second.size() >= 2) {
	    sscanf(results.second[1].c_str(),"%lld",&actual_offset);
	  }
	  else {
	    cout << "Failed to parse actual_offset from " << markv_socket.outLine << endl;
	  }

	  cout << "Disk_Unit::disk_servo: actual_offset = " <<  actual_offset << endl;
	  req_byte_offset_mult8 -= actual_offset;
	}
      ignore_next_servo_error = true; // skipping one servo report after doing a servo
    }
  return 1;
}


bool Disk_Unit::reconnect_eVLBI(bool recording, const string& net2disk_command)
{
  in2net("off");
  if (recording) {
    net2disk("close");
  }
  else {
    net2out("close");
  }
  in2net("disconnect");

  // Restore settings, just in case the station Mark5 was rebooted.
  if (!set_e_station_protocol(du_status.get_protocol(),
			      du_status.get_sockbuf_size(),
			      du_status.get_workbuf_size()))
    {
      cout <<"SU_Control: could not set station net_protocol" << endl;
      return false;
    }
  if (!set_e_station_play_rate(du_status.get_data_rate()))
    {
      cout << "SU_Control: could not set station play_rate" << endl;
      return false;
    }
  if (!set_e_station_play_mode(du_status.get_data_mode(),
			       du_status.get_data_submode()))
    {
      cout << "SU_Control: could not set station play_mode" << endl;
      return false;
    }

  // Reconnect.
  if (recording) 
    {
      if (!net2disk(net2disk_command.c_str())) 
	{
	  cout << "SU_Control: could not restart net2disk" << endl;
	  return false;
	}
    }
  else
    {
      if (!net2out("open")) 
	{
	  cout << "SU_Control: could not restart net2out" << endl;
	  return false;
	}
    }
  if (!in2net("connect")) 
    {
      cout << "SU_Control: could not reconnect in2net" << endl;
      return false;
    }
  if (!in2net("on")) 
    {
      cout << "SU_Control: could not restart in2net" << endl;
      return false;
    }
  return true;
}

bool Disk_Unit::bank_switch()
{


  char command[50];
  sprintf(command,"bank_set=inc;\n");

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::bank_set failed" << endl;
      return false;
    }
  return true;
}


bool Disk_Unit::bank_set(char bank)
{


  char command[50];
  sprintf(command,"bank_set=%c;\n",bank);

  if (!handle_markv_command(command))
    {
      cout << "Disk_unit::bank_set failed" << endl;
      return false;
    }
  return true;
}


bool Disk_Unit::restart_transfer(void)
{
  in2net("off");
  
  if (!in2net("on")) 
    {
      cout << "SU_Control: could not restart in2net" << endl;
      return false;
    }
return true;
}


bool Disk_Unit::query_du_ready(void)
{
  // in case this method returns the du to be unavailable, we need to update the VSNs, since this would otherwise be done after a bank_set? query, but ofcourse this query can't be executed with an unavailable du
#define RETURN_ERROR do { du_status.set_VSN(VSN, -1); return false; } while (false)
  
  char VSN[MAX_NR_BANKS][MEDIUM_ID_LEN];
  for (int i = 0; i < MAX_NR_BANKS; i++) {
    VSN[i][0] = '\0';
  }

  if (!query_du_status()) RETURN_ERROR;

  if (check_status(ERROR)) 
    {
      print_status_error();
      RETURN_ERROR;
    }

  if (!check_status(SYSTEM_READY)) 
    {
      cout << "Disk_unit::SYSTEM_READY = 0" << endl;
      RETURN_ERROR;
    }

  
  if (check_status(BANK_A_SELECTED))
    {
      if (check_status(BANK_A_READY))
	{
	  return true;
	}
      else
	{
	  cout << "Bank A not ready" << endl; 
	  RETURN_ERROR;
	}
    }	
  else if (check_status(BANK_B_SELECTED))
    {	      
      if (check_status(BANK_B_READY))
	{
	  return true;
	}
      else
	{
	  cout << "Bank B not ready" << endl; 
	  RETURN_ERROR;
	} 
    }
  else
    {
      cout << "No bank selected" << endl; 
      RETURN_ERROR;
    }
      
}


bool Disk_Unit::query_du_ready_4evil(void)
{
  if (!query_du_status()) return 0;

  if (check_status(ERROR)) 
    {
      print_status_error();
      return 1;//not having a disk pack in returns an error (does not make sense)
    }

  if (!check_status(SYSTEM_READY)) 
    {
      cout << "Disk_unit::SYSTEM_READY = 0" << endl;
      return 1;//same here, system is not ready without disk...
    }
  return 1;
}


bool Disk_Unit::query_net2out_on(void)
{
  if (!query_du_status()) return 0;

  if (check_status(ERROR)) 
    {
      print_status_error();
      return 0;
    }

  if (check_status(NET2OUT_ON)) return 1;
  else return 0;
}

bool Disk_Unit::query_net2disk_on(void)
{
  if (!query_du_status()) return 0;

  if (check_status(ERROR)) 
    {
      print_status_error();
      return 0;
    }

  if (check_status(NET2DISK_ON)) return 1;
  else return 0;
}

bool Disk_Unit::query_in2net_on(void)
{
  if (!query_e_station_status()) return 0;

  if (check_e_station_status(ERROR)) 
    {
      print_status_error();
      return 0;
    }

  if (check_e_station_status(IN2NET_ON)) return 1;
  else return 0;
}


bool Disk_Unit::query_play_on(void)
{
  if (!query_du_status()) return 0;

  if (check_status(ERROR)) 
    {
      print_status_error();
      return 0;
    }

  if (check_status(PLAYBACK_ON)) return 1;
  else return 0;
}


bool Disk_Unit::query_e_station_play_on(void)
{
  if (!query_e_station_status()) return 0;

  if (check_status(ERROR)) 
    {
      print_status_error();
      return 0;
    }

  if (check_e_station_status(PLAYBACK_ON)) return 1;
  else return 0;
}


bool Disk_Unit::query_du_status(void)
{
  char query[50];
  int_32 status;
  
  sprintf(query,"status?\n");
  
  if (!handle_markv_command(query))
    {
      cout << "Disk_unit::status_query failed" << endl;
      return 0;
    }

  if (merlin_slave) {
    du_status.set_general_status(0x00300001); //SYSTEM_READY|BANK_A_SELECTED|BANK_A_READY
    return true;
  }

  Command_Reply results = parse_command(markv_socket.outLine);
  
  if (results.second.size() >= 2) {
    sscanf(results.second[1].c_str(), "%x", &status);
  }
  else {
    cout << "Failed to parse status from " << markv_socket.outLine << endl;
  }

  du_status.set_general_status(status);  
  //  cout << "Disk_Unit::query_du_status: status: " << status << endl;
  return 1;
}


bool Disk_Unit::query_e_station_status(void)
{
  char query[50];
  int_32 status;
  bool e_station = true;  

  sprintf(query,"status?\n");
  
  if (!handle_markv_command(query, e_station))
    {
      cout << "Disk_unit::status_query at station failed" << endl;
      return 0;
    }

  Command_Reply results = parse_command(markv_socket.outLine);
  
  if (results.second.size() >= 2) {
    sscanf(results.second[1].c_str(), "%x", &status);
  }
  else {
    cout << "Failed to parse status from " << markv_socket.outLine << endl;
  }

  du_status.set_e_station_general_status(status);  
  //  cout << "Disk_Unit::query_du_status: status: " << status << endl;
  return 1;
}


bool Disk_Unit::query_play_mode(void)
{
  char query[50];

  char data_mode[16];
  char data_submode[16];
  char outp_data_mode[16];
  char outp_data_submode[16];
  int_32 num_sync_attempts;

  sprintf(query,"mode?\n");
  
  if (!handle_markv_command(query))
    {
      cout << "Disk_unit::mode_query failed" << endl;
      return 0;
    }

  if (merlin_slave) {
    du_status.set_data_mode("");  
    du_status.set_data_submode("");  
    du_status.set_outp_data_mode("");  
    du_status.set_outp_data_submode("");  
    du_status.set_num_sync_attempts(0);      
  }
  Command_Reply results = parse_command(markv_socket.outLine);
  
  if (results.second.size() >= 7) {
    sscanf(results.second[1].c_str(),"%s",data_mode);
    sscanf(results.second[2].c_str(),"%s",data_submode);
    sscanf(results.second[3].c_str(),"%s",outp_data_mode);
    sscanf(results.second[4].c_str(),"%s",outp_data_submode);
    sscanf(results.second[6].c_str(),"%d",&num_sync_attempts);
  }
  else {
    cout << "Failed to parse mode? reply from from " << markv_socket.outLine << endl;
  }


  du_status.set_data_mode(data_mode);  
  du_status.set_data_submode(data_submode);  
  du_status.set_outp_data_mode(outp_data_mode);  
  du_status.set_outp_data_submode(outp_data_submode);  
  du_status.set_num_sync_attempts(num_sync_attempts);  

  return 1;
}

bool Disk_Unit::query_VSN(void)
{
  char query[50];
  char VSN[MAX_NR_BANKS][MEDIUM_ID_LEN];

  sprintf(query,"bank_set?\n");

  if (!handle_markv_command(query))
    {
      cout << "Disk_unit:VSN_query failed" << endl;
      return 0;
    }

  if (merlin_slave) {
    return true; // VSN will be set in query_du_parameters for a merlin slave station
  }

  // parse the response, which should be of any of the following forms (0,1 and 2 banks active resp.):
  // "!bank_set? 0 : - :  : - :  ;"
  // "!bank_set? 0 : A : MED-0038/3200/1024 : - :  ;"
  // "!bank_set? 0 : B : WSRT-040/2000/1024 : A : OSOD-009/1317/1024 ;"
  Command_Reply results = parse_command(markv_socket.outLine);
  char active_bank, inactive_bank;
  if (results.second.size() >= 5) {
    sscanf(results.second[1].c_str(),"%c",&active_bank);
    strncpy(VSN[0],results.second[2].c_str(),8);
    sscanf(results.second[3].c_str(),"%c",&inactive_bank);
    strncpy(VSN[1],results.second[4].c_str(),8);
    VSN[0][8] = VSN[1][8] = '\0';
    cout << " => bank " << active_bank << " '" << VSN[0] << "' bank " << inactive_bank << " '" << VSN[1] << "'" << endl;

  }
  else 
    {
      cout << " => Error parsing VSN" << endl;
      VSN[0][0] = VSN[1][0] = '\0';
      du_status.set_VSN(VSN, -1);
      return 0;
    }
  if (strlen(VSN[0]) < VSN_LEN) 
    {
      VSN[0][0] = '\0';
    }
  if (strlen(VSN[1]) < VSN_LEN) 
    {
      VSN[1][0] = '\0';
    }
  int bank_nr;
  switch (active_bank) {
  case 'A' : bank_nr = 0; break;
  case 'B' : {
    // if bank B is the active bank, VSN's have to be swapped
    char swap[MEDIUM_ID_LEN];
    strncpy(swap,VSN[0],MEDIUM_ID_LEN);
    strncpy(VSN[0],VSN[1],MEDIUM_ID_LEN);
    strncpy(VSN[1],swap,MEDIUM_ID_LEN);
    bank_nr = 1; 
    break;
  }
  default : bank_nr = -1; break;
  }
  du_status.set_VSN(VSN,bank_nr);
			    
  return 1;
}

bool Disk_Unit::query_play_rate(void)
{
  char query[50];

  ieee_dbl data_rate;
  ieee_dbl clock_rate;
  ieee_dbl clock_gen_freq;

  sprintf(query,"play_rate?\n");

  if (!handle_markv_command(query))
    {
      cout << "Disk_unit:play_rate_query failed" << endl;
      return 0;
    }

  if (merlin_slave) {
    // these values are not used for disk based playback
    du_status.set_data_rate(0);
    du_status.set_clock_rate(0);
    du_status.set_clock_gen_freq(0);   
  }

  Command_Reply results = parse_command(markv_socket.outLine);
  
  if (results.second.size() >= 4) {
    sscanf(results.second[1].c_str(), "%lf", &data_rate);
    sscanf(results.second[2].c_str(), "%lf", &clock_rate);
    sscanf(results.second[3].c_str(), "%lf", &clock_gen_freq);
  }
  else {
    cout << "Failed to parse play_rate from " << markv_socket.outLine << endl;
  }

  du_status.set_data_rate(data_rate);
  du_status.set_clock_rate(clock_rate);
  du_status.set_clock_gen_freq(clock_gen_freq);


  return 1;  
}


bool Disk_Unit::query_pointer_position(void)
{

  char query[50];
  int_64 play_ptr;

  sprintf(query,"position?\n");
  
  if (!handle_markv_command(query))
    {
      cout << "Disk_unit::pointer_position_query failed" << endl;
      return 0;
    }

  if (merlin_slave) {
    // only used for logging
    du_status.set_play_ptr(0);
  }

  Command_Reply results = parse_command(markv_socket.outLine);
  
  if (results.second.size() >= 3) {
    sscanf(results.second[3].c_str(), "%llu", &play_ptr);
  }
  else {
    cout << "Failed to parse play pointer from " << markv_socket.outLine << endl;
  }

  du_status.set_play_ptr(play_ptr);  
  cout << "Disk_Unit::query_pointer_position: play_pointer: " << play_ptr << endl;
  return 1;
}


bool Disk_Unit::check_data(void)
{

  if (!handle_markv_command("data_check?\n"))
    {
      cout << "Disk_unit::check_data failed" << endl;
      return 0;
    }

  if (merlin_slave) {
    du_status.set_data_mode("");
    du_status.set_data_submode("");
    du_status.set_data_time("9999y366d23h59m59s");
    du_status.set_data_byte_offset(0);
    du_status.set_track_frame_period(0);
  }


  Command_Reply results = parse_command(markv_socket.outLine);
  
  if (results.second.size() >= 6) {
    du_status.set_data_mode(results.second[1].c_str());
    du_status.set_data_submode(results.second[2].c_str());
    du_status.set_data_time(results.second[3].c_str());
    du_status.set_data_byte_offset(atoll(results.second[4].c_str()));
    du_status.set_track_frame_period(atof(results.second[5].c_str()));
  }
  else {
    cout << "Failed to parse status from " << markv_socket.outLine << endl;
  }

  return 1;
}


bool Disk_Unit::check_status(GENERAL_STATUS status_bit)
{
  int_32 status;
  status = du_status.get_general_status();
  //  cout << "Disk_Unit::check_status: status: " << status << endl;
  //  cout << "Disk_Unit::check_status: status_bit: " << status_bit << endl;
  int_32 bit = 1;
  return (status & (bit << status_bit));
}


bool Disk_Unit::check_e_station_status(GENERAL_STATUS status_bit)
{
  int_32 status;
  status = du_status.get_e_station_general_status();
  //  cout << "Disk_Unit::check_status: status: " << status << endl;
  //  cout << "Disk_Unit::check_status: status_bit: " << status_bit << endl;
  int_32 bit = 1;
  return (status & (bit << status_bit));
}


void Disk_Unit::print_status_error()
{
  char query[50];
  
  char status_error[50];
  sprintf(query,"error?\n");
  
  if (!handle_markv_command(query))
    {
      cout << "Disk_unit::error_query failed" << endl;
      //      return 0; // no need to overdo it
    }
  else
    {
      if (merlin_slave) {
	cout << "A merlin slave should never return an error in its status...." << endl;
	return;
      }

      Command_Reply results = parse_command(markv_socket.outLine);
      
      if (results.second.size() >= 3) {
	sscanf(results.second[2].c_str(),"%50c",status_error);
	status_error[strlen(markv_socket.outLine)-strlen(query)-14]='\0';
	cout << "Disk_Unit::print_status_error: " << status_error << endl;
      }
      else {
	cout << "Failed to parse error from " << markv_socket.outLine << endl;
      }
    }
}


void Disk_Unit::print_command_response(const char* command, int return_code)
{
  switch (return_code)
    {
    case (0): cout << "Response to " << command << " : action succesfully completed" << endl; break;
    case (1): cout << "Response to " << command << " : action initiated or enabled, but not completed" << endl; break;
    case (2): cout << "Response to " << command << " : command not implemented or not relevant to this DTS" << endl; break;
    case (3): cout << "Response to " << command << " : syntax or parameter error" << endl; break;
    case (4): cout << "Response to " << command << " : error encountered" << endl; break;
    case (5): cout << "Response to " << command << " : currently too busy to service request" << endl; break;
    case (6): cout << "Response to " << command << " : inconsistent or conflicting request" << endl; break;
    case (7): cout << "Response to " << command << " : no such keyword" << endl; break;
    case (8): cout << "Response to " << command << " : parameter error" << endl; break;
    case (9): cout << "Response to " << command << " : indeterminate state" << endl; break;
    default: cout << "Response to " << command << " : unknown return code " << return_code << endl; break;
    }
}

bool Disk_Unit::set_e_station_clock(int freq, string mode)
{
  char command[50];
  sprintf(command,"clock_set=%d:%s:%d\n", freq, mode.c_str(), freq);
  
  if (!handle_markv_command(command, true))
    {
      cout << "Disk_unit::clock_set failed at e_station" << endl;
      return false;
    }

  /* turned off until we know how to handle this per station
  sprintf(command,"1pps_source=none\n");
  
  if (!handle_markv_command(command, true))
    {
      cout << "Disk_unit::1pps_source failed at e_station" << endl;
      return false;
    }
  */
  

  return true;
}

bool Disk_Unit::set_e_station_dot()
{
  char command[50];

  sprintf(command,"dot_set=\n"); // supply null value, which defaults to OS time
  
  if (!handle_markv_command(command, true))
    {
      cout << "Disk_unit::dot_set failed at e_station" << endl;
      return false;
    }

  return true;
}

bool Disk_Unit::set_e_station_dim_mode(std::string data_source, unsigned int bit_stream_mask, int decimation) {
  char command[50];

  sprintf(command,"mode=%s:0x%x:%d\n",data_source.c_str(),bit_stream_mask,decimation);
  
  if (!handle_markv_command(command, true))
    {
      cout << "Disk_unit::mode failed at e_station" << endl;
      return false;
    }

  return true;
} 

bool Disk_Unit::set_remote_version() {
  char command[50];
  
  sprintf(command,"DTS_id?\n");
  if (!handle_markv_command(command, true))
    {
      cout << "Disk_unit::DTS_id? failed at e_station" << endl;
      _remote_version = unknown;
      return false;
    }

  if (merlin_slave) {
    cout << "!!!!!!!!!!!!!! Merlin slave should not be used in eVLBI !!!!!!!!!" << endl;
    _remote_version = unknown;
  }
  
  Command_Reply results = parse_command(markv_socket.outLine);

  if (results.second.size() >= 2)
    {
      if (results.second[1].find("mark5b") != string::npos) {
	_remote_version = mark5b;
      }
      else if (results.second[1].find("mark5A") != string::npos) {
	_remote_version = mark5a;
      }
      else {
	cout << "Cannot parse remote mark5 version from DTS_id?: '" << markv_socket.outLine << "'" << endl;
	_remote_version = unknown;
      }
    }
  else
    {
      cout << "Disk_unit::DTS_id? return unexpected reply: '" << markv_socket.outLine << "'" << endl;
      _remote_version = unknown;
      return false;
    }
  return true;
  
}

bool Disk_Unit::handle_markv_command(const char* command, bool e_station)
{
  if (merlin_slave) return true;

  char the_ip[16];
  int return_code = -1;
  bool is_query;

  strstr(command, "?") ? is_query = 1 : is_query = 0;
  
  if (!e_station)
    {
      if (!strlcpy(the_ip, du_status.get_ip(), sizeof(the_ip)))
	{
	  cout << "IP address of disk unit not known" << endl;
	  return 0;
	}
    }
  else 
    {
      if (!strlcpy(the_ip, du_status.get_e_station_control_ip(), sizeof(the_ip)))
	{
	  cout << "IP address of eVLBI station not known" << endl;
	  return 0;
	}
    }
  cout << "handle_markv_command: command = " << command << ", ip address = " << the_ip << endl;

  if (!markv_socket.open_sock())
    {
      cout << "Disk_Unit: failed to open socket at " << the_ip << endl;
      return 0;
    }
  if (!markv_socket.connect_sock(the_ip))
    {
      cout << "Disk_Unit: failed socket connection to " << the_ip << endl;
      return 0;
    }
  if (!markv_socket.send_sock(command))
    {
      cout << "Disk_Unit: failed to send command: " << command << " to address: " << the_ip << endl;
      return 0;
    }
  if (!markv_socket.receive_sock())
    {
      cout << "Disk_Unit: failed to receive reply from command: " << command << " at IP address: " << the_ip << endl;
      return 0;
    }
  markv_socket.remove_sock();
  
  cout << "MarkV_Command outline '" << markv_socket.outLine << "'" << endl;
  Command_Reply results = parse_command(markv_socket.outLine);
  if( results.second.empty() ) {
    cout << "Disk_Unit: Bogus response '" << markv_socket.outLine << "'"
	 << endl;
    return 0;
  }

  if ( sscanf(results.second[0].c_str(),"%d", &return_code) != 1) {
    cout << "handle_markv_command: unable to find return code '" << markv_socket.outLine << "'"
	 << endl;
    return 0;
  }
  if (return_code != 0 && return_code != 1 && return_code != 2)
    {
      print_command_response(command, return_code);
      return 0;
    }

  return 1;
}

string DU_Status::get_VSN_active() {
  if (0 <= _active_bank && _active_bank < MAX_NR_BANKS) {
    return get_VSN(_active_bank);
  }
  else {
    return "";
  }
}  

void  DU_Status::set_VSN(const char newval[MAX_NR_BANKS][MEDIUM_ID_LEN], int new_active_bank)  {
  for (int bank = 0; bank < MAX_NR_BANKS; bank++) {
    strncpy(_VSN[bank],newval[bank],MEDIUM_ID_LEN);
  }
  _active_bank = new_active_bank;
}

//-----------------------------------------------------------------------------------------

const int DU_Status::MTU_NOT_SET = numeric_limits<int_32>::max();
const int DU_Status::CONNECTIVITY_NOT_SET = numeric_limits<int_32>::max();


DU_Status::DU_Status()
  :
  _general_status(0),
  _play_ptr(0),
  _start_rot(0),
  _track_select(0),
  _data_rate(8),
  _clock_rate(9),
  _clock_gen_freq(9)
{
  _VSN[0][0] = _VSN[1][0] = '\0';
  strcpy(_ip,"");
  strcpy(_play_status,"");
  strcpy(_outp_data_mode,"");
  strcpy(_outp_data_submode,"");
  strcpy(_data_mode,"");
  strcpy(_data_submode,"");
  strcpy(_data_time,"");
  _mtu_size = MTU_NOT_SET;
  _connectivity = CONNECTIVITY_NOT_SET;
}

DU_Status::~DU_Status(){}
